//---------------------------------------------------------------------------
//
//  Time-stamp: <2003-11-24 18:16:55 sutner> 
//
// :File:      automata.cc
// :Purpose:   run parser interactively
//             preload all standard machine operations
//             
//---------------------------------------------------------------------------

#include  <ctype.h>
#include  <iostream.h>
#include  <fstream.h>
#include  <getopt.h>

#include  "globop.h"
#include  "listutil.h"
#include  "makeatom.h"
#include  "parser.h"
#include  "ptree.h"
#include  "symtab.h"
#include  "urlist.h"

#define  YYSTYPE  UList
#include  "bparser.h"

//--------------------------------------------------------------------------------
// messages

const  char *aut_version_string = "aut: version 4.3 (990304)";

char  *expr = NULL;


const  char *aut_help_string = \
"Usage:  aut [-heqsv]  cmdfiles \n"
"h:    print this help message and quit\n"
"q:    quiet mode, no prompts, no timing\n"
"s:    silent mode, no output \n"
"v:    print version number and quit \n"
"e:    evaluate the following expression and quit \n"
"      cmdfiles are read and executed in order, use '-' for stdin\n"
"      defaults to stdin if no files are given"; 


// output formatting

char  *Format::lp = "(";
char  *Format::rp = ")";
char  *Format::cm = ", ";

//----------------------------------------------------------------------
// check for quiet mode, insert default stdin if no files are specified
//----------------------------------------------------------------------

int getopt(int argc, char * const argv[], const char *optstring);

static  void   process_cmd_line( int argc, char  *argv[], 
				 list<string>& nml, Parser::PrntT& q, bool& ex )
{
//   extern int optind;
//   extern char *optarg;
    int c;
    q = Parser::LOUD;
    ex = 0;
    char  *sp, *tp;
    
    // first, process options 
    while( (c=getopt(argc,argv,"hvqse:")) != EOF )
	switch( c )
	{
	case 'v':  
	    cout << aut_version_string << endl;     
	    exit(0);
	    break;

	case 'h':  
	    cout << aut_version_string << endl;     
	    cout << aut_help_string << endl; 
	    exit(0); break;
	
	case 'q':  q = Parser::QUIET;  break;
	case 's':  q = Parser::SILENT; break;

	case 'e':  ex = 1; 
	    expr = new char[strlen(optarg) + 2];
	    for( sp = optarg, tp = expr; *sp != '\0'; )
		*tp++ = *sp++;
	    *tp++ = '\n';
	    *tp++ = '\0';
	    break;

	default:   break;
	}
   

    if( optind < argc )
    {
	while( optind < argc )
	    nml.push_back( argv[optind++] );
    }
    else
    {
	nml.push_back( "-" );	// stdin only 
    }
}


//--------------------------------------------------------------------------------

int   main( int argc, char *argv[] )
{
    bool    eflag;
    Parser::PrntT   ld;
    Parser  par;
    
    process_cmd_line( argc, argv, par.streams(), ld, eflag );
    
				// evaluate just one expression
    if( eflag )
	par.evaluate( expr );
    else
    {
	par.set_loud_0( ld );
	par.run();
    }
    
    if( !par.is_quiet() ) UListInitializer::showtime = 1;
}

//--------------------------------------------------------------------------------

