//---------------------------------------------------------------------------
//
//  Time-stamp: <2003-11-24 18:18:33 sutner> 
//
// :File:      chebyshev.cc
// :Purpose:   
//---------------------------------------------------------------------------

#include <cstdlib> 
#include <cctype>
 
#include  "chebyshev.h"
#include  "basic.h"
#include  "bitvect.h"
#include  "util.h"
#include  "listutil.h"
#include  "atom.h"


//----------------------------------------------------------------------
//
// Various procedures for doubly symmetric m by m matrices over GF(2). 
//
// A matrix is represented by a boolVec: 
// a 0/1-vector containing the area above and including the diagonals in 
// the real matrix. 
//
// _mm:       size of original matrix,
// length():  length of corresponding boolVec,
// _dep:      number of levels in boolVec, each a partial row.
//
// matrix in the following always refers to a boolVec
//----------------------------------------------------------------------


				// set matrix to 0/1
void  boolVec::setValue( bool b )
{
    size_t i;
    for( i = 0; i < size(); i++ )
	(*this)[i] = 0;

    if( b )
    {
	size_t d = 0;
	size_t dd = _mm;
       
	for( i = 0; i < _dep; i++ )
	{	
	    (*this)[d] = 1;
	    d += dd;
	    dd -= 2;
	}
    }
}

				// add two matrices, c = *this + b
void  boolVec::addmatrix( const boolVec& B, boolVec& C ) const
{
    for( size_t i = 0; i < size(); i++ )
	C[i] =  (*this)[i] ^ B[i];
}

				// multiply matrix by sigma, b = (*this) * sigma
				// rr = 0/1:  determines sigma/sigma^+
void   boolVec::multsigma( boolVec& b, bool rr ) const
{
    size_t  t = size();
    size_t  m = _mm;
    size_t  i, j, d = m, l = m, r = 2*m - 3, l1 = 1, r1 = m - 2;

    b[0]   = rr ? (*this)[0]^(*this)[1] : (*this)[1];
    b[m-1] = rr ? (*this)[m-2]^(*this)[m-1] : (*this)[m-2];
    for( j = 1; j < m-1; j++ )
	b[j] = rr ? (*this)[j-1]^(*this)[j]^(*this)[j+1] : (*this)[j-1]^(*this)[j+1];
   

    for( i = 1; i < m/2; i++ )
    {
	b[l] = rr ? (*this)[l1]^(*this)[l]^(*this)[l+1] : (*this)[l1]^(*this)[l+1];
	b[r] = rr ? (*this)[r1]^(*this)[r]^(*this)[r-1] : (*this)[r1]^(*this)[r-1];
	for ( j = l+1; j < r; j++ )
	    b[j] = rr ? (*this)[j-1]^(*this)[j]^(*this)[j+1] : (*this)[j-1]^(*this)[j+1];
	d -= 2;
	l1 = l + 1; 

	r1 = r - 1;
	l += d;     

	r  = l + d - 3;
    }
   

    if( m & 1 )                  // odd size matrix, handle center element 
	if( rr ) 
	    b[t-1] = (*this)[t-1];
	else
	    b[t-1] = 0;
   
}

				// expand a boolVec to a real m by m bit-matrix 
void   boolVec::expand_to_bitmatrix(  BitMatrix& bmat )
{
    size_t  s = _dep;
    size_t  m = _mm;
   
    size_t  i, j, val, d = 0, r = m, NW1 = 0, NW2 = 0, SE1 = m-1, SE2 = m-1;

    for ( i = 0; i < s; i++ )
    {
	for ( j = 0; j < r; j++ )
	{ 
	    val = (*this)[d+j];
	    bmat[NW1 + j]->setbit( NW2, val );
	    bmat[NW1]->setbit( NW2+j, val );
	    bmat[SE1 - j]->setbit( SE2, val );
	    bmat[SE1]->setbit( SE2-j, val );
	}
	NW1++; NW2++; SE1--; SE2--;
	d += r;
	r -= 2;
    }
}


//----------------------------------------------------------------------
//  
//   compute  pi_n[ S_m ]  or  pi_n[ S^+_m ] 
//   print matrix or rank, only n or i=0,..,n
//   sigmatype = 0/1:   rule 90/150
//   prnmat  = 0/1:     return only co-rank / return whole matrix
//   prnall  = 0/1:     return list of all matrices/co-ranks or just last


static UList  make_matrix( int m, int n, bool sigmatype, bool prnmat, bool prnall )
{
    UList       res;
    boolVec    *boolvecarr[3];
    BitMatrix   bmat(m,m);
    int         j;

    boolvecarr[0] = new boolVec(m);
    boolvecarr[1] = new boolVec(m);
    boolvecarr[2] = new boolVec(m);
   
    boolvecarr[0]->setValue(0);
    boolvecarr[1]->setValue(1);

    if( prnall )
	if( prnmat )
	{
	    boolvecarr[0]->expand_to_bitmatrix( bmat );
	    res = bitmatrix_to_ulist( bmat ) -= nil;
	    boolvecarr[1]->expand_to_bitmatrix( bmat );
	    res = bitmatrix_to_ulist( bmat ) -= res;
	}
	else
	{
	    res = 0 -= m -= nil;
	    res = 0 -= res;
	}

    for ( j = 1; j < n; j++ )
    {
	(boolvecarr[j % 3])->multsigma(*(boolvecarr[(j+1) % 3]), sigmatype );

	(boolvecarr[(j+1) % 3])->addmatrix(*boolvecarr[(j-1) % 3], *boolvecarr[(j+1)%3] );

	if( prnall )
	{
	    boolvecarr[(j+1) % 3]->expand_to_bitmatrix( bmat );
	    if( prnmat )
		res = bitmatrix_to_ulist( bmat ) -= res;
	    else
		res = new_int( m - bmat.matrixRank() ) -= res;
	}
    }


    if( !prnall )
    {
	boolvecarr[n % 3]->expand_to_bitmatrix( bmat );
	if( prnmat )
	    res = bitmatrix_to_ulist( bmat );
	else
	    res = new_int( m - bmat.matrixRank() );
    }
    else
	res = reverse( res );

    delete   boolvecarr[0];
    delete   boolvecarr[1];
    delete   boolvecarr[2];

    return res;
}       


//  simple wrapper

UList  piMatrix( UList L )
{
    int  m, n;
    bool sig, prm, pra;
    m = ivalue(*L);
    ++L;
    n = ivalue(*L);
    ++L;
    sig = (ivalue(*L) > 0);
    ++L;
    prm = (ivalue(*L) > 0);
    ++L;
    pra = (ivalue(*L) > 0);
   
    return make_matrix( m, n, sig, prm, pra );
}

//----------------------------------------------------------------------



