//---------------------------------------------------------------------------
//
//  Time-stamp: <2003-11-24 18:37:56 sutner> 
//
// :File:     Clock.h
// :Purpose:  Primitive stop watch, has a lap time feature. 
//
//
//  constructor:  starts clock
//  start():      starts clock, resets if already running
//  stop():       obvious
//  lapTime():    difference start, now
//  lapTime(1):   difference start, now; then reset
//  elapsedTime:  difference start, stop
//  
//  units():      should return granularity in seconds, is based on 
//  system constant CLOCKS_PER_SEC (time.h), which hopefully is the number 
//  of ticks per second. Seems to be called CLK_TCK on some systems.
//
//---------------------------------------------------------------------------

#ifndef _CLOCK_H_
#define _CLOCK_H_

#include  <time.h> 

				// does this really fix the problem, always???
#ifndef CLOCKS_PER_SEC 
#define CLOCKS_PER_SEC CLK_TCK
#endif 

//--------------------------------------------------------------------------------

class Clock
{
public:

    Clock(): _starttime(clock()) {}

    void    start();
    void    stop();

    double  lapTime( bool rs=false );
    double  elapsedTime() const;

    double  units() const    { return 1.0 / CLOCKS_PER_SEC; }


private:

    clock_t  _starttime;   
    float    _elapsed;
};


//--------------------------------------------------------------------------------

inline  void Clock::start()
{
    _starttime = clock();
}


inline  void Clock::stop()
{
    _elapsed = (double)  (clock() - _starttime)/CLOCKS_PER_SEC;
}


inline  double Clock::lapTime( bool rs )
{
    double diff =  (double)(clock() - _starttime)/CLOCKS_PER_SEC;
    if( rs )  start();
    return diff;
}

inline  double Clock::elapsedTime() const
{
    return  _elapsed;
}


//--------------------------------------------------------------------------------
#endif
