//---------------------------------------------------------------------------
//
//  Time-stamp: <2003-11-24 15:00:08 sutner> 
//
// :File:     basic.h
// :Purpose:  macros, helper functions
//            
//---------------------------------------------------------------------------

#ifndef  _BASIC_H
#define  _BASIC_H

//----------------------------------------------------------------------

#include  "stl-wrap.h"
#include  <cassert>
#include  <cctype>
#include  <fstream>
#include  <iomanip>
#include  <iostream>
#include  <list>
#include  <cstdlib>
#include  <string>
#include  <vector>
#include  <hashtable.h>

using namespace std;

//------------------------------------------------------------------------------
// miscellaneous types, macros, ...
//------------------------------------------------------------------------------


#define    MAIN        main( int argc, char **argv )
#define    ALINE       cout << endl << \
                       "------------------------------------------------" << endl
#define    WAITNL(x)   cout << "--hit return" << '\n'; cin.ignore(x)   
#define    COMMA  ", "


// basic containers

typedef   vector<int>       ivector;
typedef   vector<size_t>    uvector;
typedef   list<int>         ilist;
typedef   list<size_t>      ulist;

//--------------------------------------------------------------------------------
// cheesy formatting

struct Format 
{
    static  char *lp;		// left paren    '{' or '('
    static  char *rp;		// right paren   '}' or ')'
    static  char *cm;		// comma-space   ", " or " "
};

//--------------------------------------------------------------------------------
// only used in semimodule generation, product automata

typedef  pair<size_t,size_t>  uupair;

namespace __gnu_cxx
{

struct  hash<uupair>
{
    size_t  operator()( const uupair& p ) const
    {
	return   (p.first << 8) ^  p.second;
    }
};

}

static inline  size_t  pdecode1( size_t b, size_t p )
{
    return  p / b;
}

static inline  size_t  pdecode2( size_t b, size_t p )
{
    return  p % b;
}

static inline  uupair  pdecode( size_t b, size_t p )
{
    return  uupair( p / b, p % b );
}

static inline  size_t  pcode( size_t b, size_t x, size_t y )
{
    return  x * b + y;
}

//--------------------------------------------------------------------------------
// transitions, not using Triple<A,B,C>

struct  Trans
{
    int p, s, q;
    Trans():p(0), s(0), q(0) {}
    Trans( int pp, int ss, int qq ):p(pp), s(ss), q(qq) {}
};

typedef  vector<Trans>  TransV;

bool  operator==( const Trans& S, const Trans& T );
bool  operator<( const Trans& S, const Trans& T );
ostream&  operator<<( ostream& os, const Trans& T );

// this is non-standard order: 2 1 3
struct  compare_trans: public binary_function<bool,Trans,Trans>
{
    compare_trans() {}
    bool  operator()( const Trans& S, const Trans& T ) const
    {
	return ( S.s < T.s || ( S.s == T.s && S.p < T.p ) );
    }
};

struct  reverse_trans: public unary_function<Trans,Trans>
{
    reverse_trans() {}
    Trans  operator()( const Trans& T ) const
    {
	return Trans( T.q, T.s, T.p );
    }
};


//--------------------------------------------------------------------------------
// some useful functions 

int  ipow( int b, int e );
int  random( int a, int b );

void  write_string( char*& p, const char* s );

//--------------------------------------------------------------------------------
// hashing, is this really worth the effort???

namespace __gnu_cxx
{

struct hash<uvector>
{
    size_t  operator()( const uvector& A ) const
    {
        size_t h = 0;
        size_t a = 31415;
        size_t b = 27183;
	uvector::const_iterator it;
        for( it = A.begin(); it != A.end(); ++it )
	{
	    a = a*b % 805306457;
	    h = a * h + *it;
	}
        return h;
    }
};

struct   hash<ivector>
{
    size_t  operator()( const ivector& A ) const
    {
        size_t h = 0;
        size_t a = 31415;
        size_t b = 27183;
	ivector::const_iterator it;
        for( it = A.begin(); it != A.end(); ++it )
	{
	    a = a*b % 805306457;
	    h = a * h + *it;
	}
        return h;
    }
};
}


//--------------------------------------------------------------------------------
//  need to put exceptions everywhere, in particular parser

void  Error( const char *a );


//--------------------------------------------------------------------------------

#if  DEBUG > 2
#define    DEBUGMSSG(str)    cerr << (str) << endl;
#else
#define    DEBUGMSSG(str)
#endif


#ifdef  DEBUGPARSE
#define    PARSEMSSG(str)    cerr << "\t\t" << (str) << endl;
#else
#define    PARSEMSSG(str)
#endif



#ifdef  DEBUGEVAL
#define    EVALMSSG(str)    cerr << "\t" << (str) << endl;
#else
#define    EVALMSSG(str)
#endif

//--------------------------------------------------------------------------------
#endif // _BASIC_H 


