//---------------------------------------------------------------------------
//
//  Time-stamp: <2003-11-24 18:38:28 sutner> 
//
// :File:      functoid.h
// :Purpose:   function atoms
//             Function:      rich handle to FuncOb, derived from Atom
//             FuncOb:        base class for actual function objects
//             NFuncOb:       nullary
//             UFuncOb:       unary, may be wrapped
//             BFuncOb:       binary, may be wrapped
//             TFuncOb:       ternary
//             
//---------------------------------------------------------------------------


#ifndef  _FUNCTION_H_
#define  _FUNCTION_H_

#include  "basic.h"
#include  "urlist.h"
#include  <string>

class SymTable;

//--------------------------------------------------------------------------------
// function atoms


struct  FuncOb
{
    int  rcnt;
    FuncOb(): rcnt(1) {}

    virtual  ~FuncOb() {}
    virtual  UList  operator()() const { return nil; }
    virtual  UList  operator()(UList) const { return nil; }
    virtual  UList  operator()(UList,UList) const { return nil; }
    virtual  UList  operator()(UList,UList,UList) const { return nil; }
};

struct  NFuncOb: public FuncOb
{
    UList   (*ff)();
    
    NFuncOb( UList (*gg)() ): ff(gg) {} 
    ~NFuncOb() {}
    UList   operator()() const { return ff(); }
};

struct  UFuncOb: public FuncOb
{
    UList   (*ff)(UList);
    
    UFuncOb( UList (*gg)(UList) ): ff(gg) {} 
    ~UFuncOb() {}
    UList     operator()(UList a) const { return ff(a); }
};

struct  WUFuncOb: public FuncOb
{
    int  (*ff)(int);
    
    WUFuncOb( int (*gg)(int) ): ff(gg) {} 
    ~WUFuncOb() {}
    UList     operator()(UList a) const { return new_int(ff( ivalue(a) )); }
};

struct  BFuncOb: public FuncOb
{
    UList   (*ff)(UList,UList);
    
    BFuncOb( UList (*gg)(UList,UList) ): ff(gg) {} 
    ~BFuncOb() {}
    UList   operator()(UList a,UList b) const { return ff(a,b); }
};

struct  WBFuncOb: public FuncOb
{
    int  (*ff)(int,int);
    
    WBFuncOb( int (*gg)(int,int) ): ff(gg) {} 
    ~WBFuncOb() {}
    UList     operator()(UList a, UList b) const 
    { 
	return new_int( ff( ivalue(a), ivalue(b) )); 
    }
};

struct  TFuncOb: public FuncOb
{
    UList   (*ff)(UList,UList,UList);
    
    TFuncOb( UList (*gg)(UList,UList,UList) ): ff(gg) {} 
    ~TFuncOb() {}
    UList   operator()(UList a,UList b,UList c) const { return ff(a,b,c); }
};


//--------------------------------------------------------------------------------

struct  Function: public Atom
{
    FuncOb  *fp;
    int      ari;
    char    *fname;

public:    
    Function();
    Function( NFuncOb* pp, const char *nm = "" );
    Function( UFuncOb* pp, const char *nm = "" );
    Function( BFuncOb* pp, const char *nm = "" );
    Function( WUFuncOb* pp, const char *nm = "" );
    Function( WBFuncOb* pp, const char *nm = "" );
    Function( TFuncOb* pp, const char *nm = "" );

    Function( const Function& ff );
    Function&  operator=( const Function& src );

    ~Function(); 

    void    print( ostream& os ) const { os << fname << "  " << ari; }

    UList   operator()() const { return (*fp)(); }
    UList   operator()( UList a ) const { return (*fp)(a); }
    UList   operator()( UList a, UList b ) const { return (*fp)(a,b); }
    UList   operator()( UList a, UList b, UList c ) const { return (*fp)(a,b,c); }
    int     arity() const { return ari; }

    UList   eval( SymTable& ) { return copy(); }
    AType   atom_type() const { return FUNC; }
				// really just ref counting
    const  Function  *copy()    { return new Function(*this); }
    Function         *destroy() { return this; }
};



#endif 
//--------------------------------------------------------------------------------

//  Local Variables: ***
//  eval: (setq case-fold-search nil) ***
//  eval: (setq case-replace nil) ***
//  End: ***

