//--------------------------------------------------------------------------------
//
//  Time-stamp: <2003-11-24 18:39:29 sutner>
// 
//  :File:     graph.h
//  :Purpose:  header file for directed graphs and a few graph algoritms
//
//--------------------------------------------------------------------------------

#ifndef _GRAPH_H
#define _GRAPH_H

//----------------------------------------------------------------------

#include   "intset.h"
#include   <list>
#include   <vector>

//----------------------------------------------------------------------
// defined herein

class  Graph;

//----------------------------------------------------------------------
// Graph class for directed graphs, possibly with int labels
// property bits:  currently unused

class Graph
{
    typedef  vector<ulist >   AdjacencyList;
    typedef  ulist::iterator  iterator;
    typedef  ulist::const_iterator  const_iterator;

    size_t  _nv;		// number of vertices 0,.., _nv-1
    size_t  _ne;		// number of edges
    size_t  _nl;		// label alphabet (integer) 0,..,_nl-1 
    int     _pr;		// special properties 
   
    AdjacencyList  adjl;		// adjacency lists 
    AdjacencyList  labl;		// label lists 

public:

    Graph();
    Graph( size_t nn, size_t ne=0, size_t nl=0 ): 
	_nv(nn), _ne(ne), _nl(nl), _pr(0), adjl(nn), labl(nl?nn:0) {} 
   
    ~Graph();

    void initialize( size_t nv, size_t ne=0, size_t nl=0 );
    
    Graph( const Graph& src );
    Graph& operator=( const Graph& src );

    size_t&  numVertex()       { return _nv; }
    size_t&  numEdge()         { return _ne; }
    size_t&  numLabel()        { return _nl; }
    size_t   numVertex() const { return _nv; }
    size_t   numEdge()   const { return _ne; }
    size_t   numLabel()  const { return _nl; }

    ulist&        operator[]( size_t i )       { return adjl[i]; }
    ulist&        labelList( size_t i )        { return labl[i]; }
    const ulist&  operator[]( size_t i ) const { return adjl[i]; }
    const ulist&  labelList( size_t i ) const  { return labl[i]; }

    // insert edge, un-/labelled 
    void insertEdge(size_t x, size_t y) { adjl[x].push_back(y); _ne++; }
    void insertEdge(size_t x, size_t y , size_t ll) 
    { adjl[x].push_back(y); labl[x].push_back(ll); _ne++; }

    // Test for presence of edge(x,y), un-/labelled 
    bool  edgeQ(size_t x, size_t y) const;
    bool  edgeQ(size_t x, size_t y , size_t s ) const;
   
    // place weak component of vertex v (or vertex set I) into IntSet reach
    // return number of reached points 
    size_t  weakComponent( size_t v, IntSet& reach ) const;
    size_t  weakComponent( const IntSet& I, IntSet& reach ) const;
   
    // compute all strongly connected components of graph
    // components stored in ivector part, their cardinalities in ivector card
    size_t  stronglyConnectedComponents( uvector& part, uvector& card ) const;

    // Compute collapse graph gc of graph
    // also return SCC information 
    size_t  makeCollapse( Graph& gc, uvector& part, uvector& card ) const;
   
    // compute reverse of a directed graph 
    void  reverse( Graph& gr ) const;

    // longest path in acyclic part 
    size_t  longestPath() const;
   
    friend ostream&  operator<<( ostream& out, const Graph& gr );
   

private:			// helper functions 
    void  dfs_search( size_t v, IntSet& reach ) const;
    void  make_scc( size_t v, uvector& comps, uvector& card ) const;
};


//----------------------------------------------------------------------
#endif
