//---------------------------------------------------------------------------
//
//  Time-stamp: <2003-11-24 18:40:31 sutner> 
//
// :File:      integer.h
// :Purpose:   really just a primitive class wrapper for GNU gmp.
// 
//---------------------------------------------------------------------------

#ifndef  _INTEGER_H_
#define  _INTEGER_H_

#include <iostream>
#include <basic.h>
#include <gmp.h>


//--------------------------------------------------------------------------------

class Integer
{
    mpz_t val;

public:

    Integer()                     { mpz_init(val); }
    Integer(const char *digs )    { mpz_init_set_str(val, digs, 10); }
    Integer(long l)               { mpz_init_set_si(val, l); }
    Integer(int i)                { mpz_init_set_si(val, i); }
    Integer(const Integer &src)   { mpz_init_set(val, src.val); }

    ~Integer()                    { mpz_clear(val);}


    size_t  num_bits() const { return  mpz_sizeinbase(val,2); }
	

    unsigned long int as_long() const  
    {
	return  mpz_get_ui(val);
    }

    void  set(const char *digs, int b = 10)
    {
	mpz_set_str(val, digs, b);
    }

				// Operators
    const Integer &operator=(const Integer &rhs )
    {
	mpz_set(val, rhs.val);
	return *this;
    }

    const Integer &operator+=(const Integer &rhs)
    {
	mpz_add(val, val, rhs.val);
	return *this;
    }

    const Integer &operator-=(const Integer &rhs)
    {
	mpz_sub(val, val, rhs.val);
	return *this;
    }

    const Integer &operator*=(const Integer &rhs)
    {
	mpz_mul(val, val, rhs.val);
	return *this;
    }

    const Integer &operator/=(const Integer &rhs)
    {
	mpz_tdiv_q(val, val, rhs.val);
	return *this;
    }

    friend const Integer operator+(const Integer &a, const Integer &b)
    {
	Integer m;
	mpz_add(m.val, a.val, b.val);
	return m;
    }

    friend const Integer operator-(const Integer &a, const Integer &b)
    {
	Integer m;
	mpz_sub(m.val, a.val, b.val);
	return m;
    }

    friend const Integer operator*(const Integer &a, const Integer &b)
    {
	Integer m;
	mpz_mul(m.val, a.val, b.val);
	return m;
    }

    friend const Integer operator/(const Integer &a, const Integer &b)
    {
	Integer m;
	mpz_tdiv_q(m.val, a.val, b.val);
	return m;
    }

    friend const Integer operator%(const Integer &a, const Integer &b)
    {
	Integer m;
	mpz_mod(m.val, a.val, b.val);
	return m;
    }

    // Relational Operators

    friend bool operator==(const Integer &a, const Integer &b)
    {
	return !mpz_cmp(a.val, b.val);
    }

    friend bool operator!=(const Integer &a, const Integer &b)
    {
	return mpz_cmp(a.val, b.val);
    }
  
    friend bool operator<(const Integer &a, const Integer &b)
    {
	return mpz_cmp(a.val, b.val) < 0;
    }
  
    friend bool operator<=(const Integer &a, const Integer &b)
    {
	return mpz_cmp(a.val, b.val) <= 0;
    }

    friend bool operator>(const Integer &a, const Integer &b)
    {
	return mpz_cmp(a.val, b.val) > 0;
    }

    friend bool operator>=(const Integer &a, const Integer &b)
    {
	return mpz_cmp(a.val, b.val) >= 0;
    }

    // Manipulation functions
    void negate()
    {
	mpz_neg(val, val);
    }
  
    void abs()
    {
	mpz_abs(val, val);
    }
  
    friend const Integer sqrt(const Integer &m)
    {
	Integer z;
	mpz_sqrt(z.val, m.val);
	return z;
    }
  
    friend const Integer random(long m)
    {
	Integer z;
	mpz_random(z.val, m);
	return z;
    }

    friend const Integer gcd(const Integer &a, const Integer &b)
    {
	Integer z;
	mpz_gcd(z.val, a.val, b.val);
	return z;
    }

    // Output operator
    friend ostream& operator<<( ostream &os, const Integer &m )
    {
	char *str = new char[mpz_sizeinbase(m.val, 10) + 2];
	mpz_get_str(str, 10, m.val);
	os << str;
	delete [] str;
	return os;
    }
};

#endif
