//---------------------------------------------------------------------------
//
//  Time-stamp: <2003-11-24 18:45:30 sutner> 
//
//  :File:      regex.h
//  :Purpose:   regular expressions
//              should be dumped and rewritten from scratch
// 
//---------------------------------------------------------------------------

#ifndef  _REGEXP_H_
#define  _REGEXP_H_

#include "befa.h"
#include <iostream.h>


class  Expr;		// handle for expressions, ref counted
class  Expr;		// base class


class Expr 
{
    friend class Expr;
    friend ostream& operator<<( ostream&, const Expr& );
    int rcnt;
protected:
    Expr(): rcnt(1) {}
    virtual void print(ostream&) const = 0;
    virtual BEFA  eval() const = 0;
    virtual ~Expr() {}
};

class Expr 
{
    friend class Expr;
    friend ostream& operator<<(ostream&, const Expr&);

    Expr* ptr;

public:

    Expr( const Expr& src ) { ptr = src.ptr; ++ptr->rcnt; }
    ~Expr() { if (--ptr->rcnt == 0) delete ptr; }
    Expr& operator=( const Expr& );


    Expr( const char* );
    Expr( Expr, Expr );

    void  print( ostream& os ) const { ptr->print(os); }
    BEFA  eval() const { return ptr->eval(); }
};


//--------------------------------------------------------------------------------


class BaseNode: public Expr 
{
    friend class Expr;
    char  *str;
    
    BaseNode( const char *ss ): str(new char[strlen(ss)+1])
    {
	strcpy( str, ss );
    }
    ~BaseNode() { delete [] str; }
    void print( ostream& os ) const { os << str; }
    BEFA  eval() const { return BEFA(str); }            
};

class UnaryNode: public Expr 
{
    friend class Expr;
    Expr  sexp;

    UnaryNode( Expr e ): sexp(e) {}
    void print( ostream& os ) const { os << "(" << sexp << ")"; }
    BEFA  eval() const
    { 
	BEFA  m = sexp.eval();
	m.star();
	return m;
    }
};

class BinaryNode: public Expr 
{
    friend class Expr;
    Expr  left;
    Expr  right;

    BinaryNode( Expr a, Expr b ): left(a), right(b) {}
    void print( ostream& os ) const
    { os << "(" << left << "+" << right << ")"; }
    BEFA  eval() const 
    { 
	return left.eval() + right.eval(); 
    }
};


#endif






