//---------------------------------------------------------------------------
//
//  Time-stamp: <2003-11-24 14:48:23 sutner> 
//
// :File:      stl-wrap.h
// :Purpose:   STL interface and wrappers.
//             Needs more testing, use with caution. 
//---------------------------------------------------------------------------

#ifndef  _STLWRAP_H
#define  _STLWRAP_H

//----------------------------------------------------------------------

#include  <iostream>
#include  <iomanip>

#include  <cstdarg>

#include  <algorithm>
#include  <functional>
#include  <vector>
#include  <list>
#include  <string>
#include  <iterator>
    
using namespace std;

//----------------------------------------------------------------------
//  simplified operations on containers
//----------------------------------------------------------------------
// pre:    Cont a container with iterator, Func a functor class
// post:   mutating, replace each x in cont by ff(x) 

template<class Cont, class Func >
inline void   Transform( Cont& cont, Func ff )
{
    typename Cont::iterator itr;
    for( itr=cont.begin(); itr!=cont.end(); ++itr )
	*itr = ff(*itr);
}

template<class Cont, class T >
inline void   Iota( Cont& cont, const T& t )
{
    iota( cont.begin(), cont.end(), t );
}

template<class Cont, class T >
inline void   Fill( Cont& cont, const T& t )
{
    fill( cont.begin(), cont.end(), t );
}

//---------------------------------------------------------------------- 
// iteration macro
// pre:  A a container, it an iterator of suitable type

#define  foreach_item(it,A) \
for( (it)=(A).begin(); (it)!=(A).end(); ++(it) )


//--------------------------------------------------------------------------------
// data entry into container
// pre:  container must support push_back operation

template<class Cont>
void  enter( Cont& cont, int n, ... )
{
   va_list va;
   va_start(va,n);
   for( int i=0; i<n; i++ )
     cont.push_back( va_arg(va,int) );
   va_end(va);
}

//--------------------------------------------------------------------------------
// output for vectors and linked lists

template<typename T>
ostream&  operator<<( ostream& out, const vector<T>& V )
{
    copy( V.begin(), V.end(), ostream_iterator<T>(out, "  "));
    return out;
}

template<class T>
ostream&  operator<<( ostream& out, const list<T>& L )
{
    copy( L.begin(), L.end(), ostream_iterator<T>(out, "  "));
    return out;
}

//--------------------------------------------------------------------------------

template<class T>
inline  void   Erase( list<T>&  S )
{
    S.erase( S.begin(), S.end() );
}
    
//--------------------------------------------------------------------------------
// initializing arrays

template<class T>
void   initialize( vector<T>& v, int n, const T& t )
{
    v.resize(n);
    fill( v.begin(), v.end(), t );
}



//----------------------------------------------------------------------
// vector operations, non-mutating, apply op pointwise
// res = A op B

template<class T, class BinOp>
vector<T>  vector_binop( const vector<T>& A, const vector<T>& B, BinOp op )
{
    assert( A.size()==B.size() );
    
    vector<T>  res(A.size());
    transform( A.begin(), A.end(), B.begin(), res.begin(), op );
    return res;
}

// scalar/vector operations, non-mutating, apply op pointwise
// res = a op B

template<class T, class BinOp>
vector<T>  scalar_vector_binop( const T& a, const vector<T>& A, BinOp op )
{
    vector<T>  res(A.size());
    transform( A.begin(), A.end(), res.begin(), bind1st(op,a) );
    return res;
}

//----------------------------------------------------------------------
// length-lex order on string, char*

struct string_lenlex: public binary_function<string,string,bool>
{
    string_lenlex() {}
    bool   operator()( const string& a, const string& b) const
    {
	return ( a.size() < b.size()) || ( a.size() == b.size() && a < b);
    }
};


struct cstr_lenlex: public binary_function<char*,char*,bool>
{
    cstr_lenlex(){}
    bool operator()(const char* s1, const char* s2) const
    {
	return strcmp(s1, s2) < 0;
    }
};

//----------------------------------------------------------------------
//  length-lex order on sequential containers
//  pre:   Cont is a container class over type T (list, vector, array, ...)
//         T must have  operator<  and  operator>

template<class Cont>
struct cont_lenlex: public binary_function<Cont,Cont,bool>
{
    cont_lenlex(){}
    bool  operator()( const Cont& A, const Cont& B ) const
    {
	if( A.size() < B.size() ) return 1;
	if( A.size() > B.size() ) return 0;
    
	typename Cont::const_iterator  ia, ib;

	ia = A.begin();
	ib = B.begin();
	
	while( ia != A.end() )
	{
	    if( *ia < *ib ) return 1;
	    if( *ia > *ib ) return 0;
	    ia++;
	    ib++;
	}
	return 0;
    }
};



//----------------------------------------------------------------------
#endif // _STLWRAP_H 


