//---------------------------------------------------------------------------
//
//  Time-stamp: <2003-11-24 18:47:43 sutner> 
//
//  :File:      symtab.h
//  :Purpose:   Symbol table for UList valued variables.
//              Permissive semantics: return nil when undefined.
//              Use chaining to get vistas.
//
//              A pain: must use string rather than char* as key type.
//              Otherwise deallocation of char* wreaks havoc in table.
//              
//---------------------------------------------------------------------------

#ifndef _SYMTAB_H_
#define _SYMTAB_H_

#include  <string>
#include  <hash_map.h>
#include  "basic.h"
#include  "urlist.h"
#include  "parser.h"

//--------------------------------------------------------------------------------
// defined herein

class SymTable;


struct hash_string
{
    size_t  operator()(const string s) const
    {
 	return  hash<const char*>()(s.c_str());
    }
};


typedef  hash_map<string, UList, hash_string, equal_to<string> >  sym_tab;

// slower? bigger? 
// typedef  map<string,Function,less<string> >  sym_tab;


//--------------------------------------------------------------------------------

class SymTable:  public sym_tab
{
private:
    
    SymTable   *next;
    
public:

    SymTable( SymTable *env=NULL ): next(env) {}

    UList   lookup( const char *fn );                   // search in whole vista

    bool    assign( UList L, const char *nm );        // 0: no prior value
    bool    assign( UList (*ff)(), const char *nm );  
    bool    assign( UList (*ff)(UList), const char *nm );
    bool    assign( UList (*ff)(UList,UList), const char *nm );
    bool    assign( int (*ff)(int), const char *nm );
    bool    assign( int (*ff)(int,int), const char *nm );
    bool    assign( UList (*ff)(UList,UList,UList), const char *nm );

    bool    assign_local( UList L, const char *nm );    // local symtab
    void    assign_list( UList LL, char **names );  // assign whole list, local

    void    print( ostream& os, bool all = 0 ) const;

private:
    bool   find_it( sym_tab::iterator& it, const char *fn );
};


//--------------------------------------------------------------------------------
# endif


