//---------------------------------------------------------------------------
//
//  Time-stamp: <2003-11-25 14:56:45 sutner> 
//
// :File:      linktest.cc
// :Purpose:   test for MathLink
//             
//---------------------------------------------------------------------------


#include  <cctype>
#include  <fstream>

#include  "globop.h"
#include  "listutil.h"
#include  "makeatom.h"
#include  "parser.h"
#include  "ptree.h"
#include  "symtab.h"
#include  "urlist.h"
#include  "extoutput.h"

extern "C"
{
#include  "mathlink.h"
}

//----------------------------------------------------------------------
//  the MathLink parser

Parser    *ml_parse;		// a quiet parser
SymTable  *ml_env;		// its symbol table
ofstream   ml_logfile;		// log file for MathLink traffic
bool       ml_logflag;		// control logging

// output formatting

char  *Format::lp = "{";
char  *Format::rp = "}";
char  *Format::cm = ", ";


int     ML_needs_token;

extern "C" 
{
   void   ML_parse( char *instr );
}

// Mathematica transmits  newlines as  '\''0''1''2'
static  void  fix_newlines( char *bf )
{
//   MLPutString( stdlink, bf );
   char  *p = bf;
   while( *p != '\0' )
      if( *p == '\\' && *(p+1) == '0' && *(p+2) == '1' && *(p+3) == '2' )
      {
	 *p++ = '\n'; *p++ = ' '; *p++ = ' '; *p++ = ' ';
      }
      else
	 p++;
//   MLPutString( stdlink, bf );
}

//--------------------------------------------------------------------------------
// should make ml_logflag controllable 

void   ML_parse( char *instr )
{
   extern  ofstream  ml_logfile;
   extern  bool  ml_logflag;
    
   fix_newlines( instr );

   char  *input =  "range[10]\n";

   ml_parse->attach( "string", input );

   ML_needs_token = 1;
   
   UList  val = ml_parse->parse( *ml_env );

   cerr << "round 1"  << endl;
   cerr << ": " << ml_parse->is_go() << endl;
   cerr << ": " << ML_needs_token << endl;
   cerr << ": " << ml_parse->do_print() << endl;

   if( ml_parse->is_go() ) 
   {
      if( ML_needs_token )
	 if( ml_parse->do_print() )
	 {
   cerr << "write val"  << endl;
	    ML_write_urlist( val );
	    if( ml_logflag ) ml_logfile << val << endl;
	 }
	 else
	 {
   cerr << "write Null"  << endl;
	    MLPutSymbol( stdlink, "Null" );
	    if( ml_logflag ) ml_logfile << "Null" << endl;
	 }
   }
   else
   {
   cerr << "write Failed"  << endl;
      MLPutSymbol( stdlink, "$Failed" );
      if( ml_logflag ) ml_logfile << "$Failed" << endl;
   }

   cerr << "before attach"  << endl;
   cerr << "string: " << input  << endl;
   cerr << "attach: " << (ml_parse->attach( "string", input )) << endl;
   cerr << "after attach"  << endl;

   ML_needs_token = 1;
   
   val = ml_parse->parse( *ml_env );

   cerr << "round 2"  << endl;
   cerr << ": " << ml_parse->is_go() << endl;
   cerr << ": " << ML_needs_token << endl;
   cerr << ": " << ml_parse->do_print() << endl;

   if( ml_parse->is_go() ) 
   {
      if( ML_needs_token )
	 if( ml_parse->do_print() )
	 {
	    ML_write_urlist( val );
	    if( ml_logflag ) ml_logfile << val << endl;
	 }
	 else
	 {
	    MLPutSymbol( stdlink, "Null" );
	    if( ml_logflag ) ml_logfile << "Null" << endl;
	 }
   }
   else
   {
      MLPutSymbol( stdlink, "$Failed" );
      if( ml_logflag ) ml_logfile << "$Failed" << endl;
   }

}

//----------------------------------------------------------------------

int  main( int argc, char *argv[] )
{
   ml_parse  = new Parser;
   ml_env    = new SymTable(NULL);
    
   MLMain( argc, argv );

   delete  ml_parse;
   delete  ml_env;
    
   return EXIT_SUCCESS;
}

//----------------------------------------------------------------------


