//---------------------------------------------------------------------------
//
//  Time-stamp: <2003-11-26 00:50:33 sutner> 
//
// :File:      atom.cc
// :Purpose:   Create user atoms from data given as ULists.
//---------------------------------------------------------------------------

#include  "makeatom.h"
#include  "util.h"
#include  "useratom.h"

//--------------------------------------------------------------------------------

UList   create_atom( UList atn, UList data )
{
    if( strcmp( svalue(atn), "CA" )==0 )
	return create_ca( data ); 
    if( strcmp( svalue(atn), "SA" )==0 )
	return create_sa( data ); 
    if( strcmp( svalue(atn), "DFA" )==0 )
	return create_dfa( data ); 

    return nil;
}

// awkward after switch to CA[w,k,r] in Mta

UList   create_ca( UList data )
{
    int  w = 3, k = 2;

    UList  R, W, K;

    data >> W;

    if( data ) { data >> K; w = ivalue(W); }
    
    if( data ) { data >> R; k = ivalue(K); }
    
    if( R.atom_type() == INT )
	return  new CAAtom( CA( ivalue(R), w, k ) );

    if( R.atom_type() == INTL )
	return  new CAAtom( CA( Ivalue(R), w, k ) );

    return nil;
}

UList   create_sa( UList data )
{
    UList  N, K, T;
    data >> N >> K >> T;

    UList res = new SAAtom( ivalue(N), ivalue(K) );
    SA  &sa = savalue( res );
    sa.eps_free() = ulist_to_transition( T, sa.transitions() ) ? TRUE : FALSE;
    return  res;
}

UList   create_fa( UList data )
{
    UList   nn, kk, T, ini, fin;
				// should be checked !!! 
    data >> nn >> kk >> T >> ini >> fin;

    int n = ivalue(nn);
    int k = ivalue(kk);
    
    UList  res = new FAAtom( n, k );
    FA  &fa = favalue( res );
    fa.eps_free() = ulist_to_transition( T, fa.transitions() ) ? TRUE : FALSE;
    ulist_to_intset( ini, n, fa.initial(), -1 );
    ulist_to_intset( fin, n, fa.final(), -1 );
    return   res;
}

UList   create_dfa( UList data )
{
    UList   N,K,T,ini,fin;
    data >> N >> K >> T >> ini >> fin;

    int n = ivalue(N);
    int k = ivalue(K);
    UList  res = new DFAAtom( n, k );
    DFA  &dfa = dfavalue( res );
    ulist_to_intarray( flatten(T), dfa.transitions(), -1 );
    dfa.initial() = ivalue(ini) - 1;
    ulist_to_intset( fin, n, dfa.final(), -1 );
    return   res;
}

UList   create_graph( UList data )
{
    UList ll, nv, ne, T, H;
    data >> nv >> ne >> T >> ll;

    int t = ivalue(T);
    assert( t==1 || t==4 );

    Graph  gr( ivalue(nv), ivalue(ne) );
   
    if( t==1 )
    {
	gr.numEdge() = 0;		// will be incremented during insertions
	UList  src, targ;
	while( ll )
	{
	    ll >> H;
	    H  >> src >> targ;
	    gr.insertEdge( ivalue(src)-1, ivalue(targ)-1 );
	}
    }
    else
    {
	int i=0;
	while( ll )
	{
	    ll >> H;
	    ulist_to_intlist( H, gr[i], -1 );
	    i++;
	}
    }

    return new GraphAtom(gr);
}

//--------------------------------------------------------------------------------
