//---------------------------------------------------------------------------
//
//  Time-stamp: <2003-11-24 18:23:36 sutner> 
//
// :File:      semimod.cc
// :Purpose:   testing templated semimodule generation
//---------------------------------------------------------------------------


#include  <ctype>
#include  <iostream>
#include  <hash_map.h>
#include  <algorithm>

#include  "stl-wrap.h"
#include  "basic.h"
#include  "pairs.h"
#include  "intset.h"
#include  "semimod.h"

//--------------------------------------------------------------------------------

typedef  vector<bool>   bvector;

struct   hash<bvector>
{
    size_t  operator()( const bvector& A  ) const
    {
	size_t h = 0;
	for( size_t i = 0; i < A.size(); ++i )
	    h = h*103 + A[i];
	return h;
    }
};



struct  one_perm: public binary_function<bvector,size_t,bvector>
{
    int   qq;			// shift factor for b transitions
    
    one_perm( int q ): qq(q) {} 

    bvector  operator()( const bvector& A, size_t s ) const 
    {
	bvector  B(A);

	if( s == 0 )
	{
	    if( B[0] && !B[qq-1] ) return B;
	    rotate( B.begin(), B.end() - 1, B.end() );
	}
	else
	{
	    B[0] = 0;
	    rotate( B.begin(), B.end() - qq, B.end() );
	}
	
	return  B;
    }
};

typedef  hash_map<size_t,string,hash<size_t>,equal_to<size_t> >  WitTable;


// pre:   sm is fully constructed, generator q0 is in position 0
// post:  wit[p] is the length-lex minimal witness for p in Q


template<class A, class B>
void   extract_witnesses( SemiModule<A,B>& sm, WitTable& wit )
{
    size_t  n = sm.Q.size();
    size_t  k = sm.kk;
    
    assert( sm.del.size() == n * k );

    IntSet  R(n);
    wit[0] = "";
    R.insert(0);
    
    size_t p, s, q;
    ulist::iterator  it = sm.del.begin();
    
    for( p = 0; p < n; ++p )
	for( s = 0; s < k; ++s )
	{
	    q = *it++;
	    if( !R.find(q) )
	    {
		wit[q] = wit[p] + char('a'+s);
		R.insert(q);
	    }
	}
}



typedef  Triple<size_t,bvector,string>  Item;

size_t  cardinality( const bvector& X ) 
{
    return accumulate( X.begin(), X.end(), 0 );
}


struct comp_2
{
    bool  operator()( const Item& X, const Item& Y ) const
    {
	size_t cx = cardinality( X.second() ); 
	size_t cy = cardinality( Y.second() );
	if( cx < cy ) return 1;
	if( cx == cy ) return  X.third() < Y.third();
	return 0;
    }
};


void  display( const bvector& A )
{
    size_t n = A.size();
    cout << A[n-2] << " " << A[n-1] << " .";
    copy( A.begin(), A.end()-2, ostream_iterator<int>( cout, " " ) );
    cout << endl;
}
    

main( int argc, char **argv )
{
    size_t n = atoi(argv[1]);
    size_t q = atoi(argv[2]);
    
    size_t i;
    
    bvector   A( n, true );

    one_perm  op( q );

    SemiModule<bvector,one_perm> SM( 2, A, op );

    size_t  m = SM.generate();
    cout << "*** " << n << "  " << q << " " << m << endl;

    vector<Pair<size_t,bvector> >  tab(m);
    tab.resize(0);

    SemiModule<bvector,one_perm>::iterator  it;
    foreach_item( it, SM.Q )
	tab.push_back( Pair<size_t,bvector>(it->second,it->first) );

    sort( tab.begin(), tab.end() );
    

    WitTable  W;
    extract_witnesses( SM, W );

//     for( size_t i = 0; i < m; ++i )
// 	cout << tab[i].first() << "\t" << tab[i].second() << "\t" << W[tab[i].first()] << endl;

//     vector<Item>  ordtab(m);
//     for( size_t i = 0; i < m; ++i )
// 	ordtab[i] = Item( tab[i].first(), tab[i].second(), W[tab[i].first()]);

//    sort( ordtab.begin(), ordtab.end(), comp_2() );
    
//     for( size_t i = 0; i < m; ++i )
// 	cout << ordtab[i].first() << "\t" << ordtab[i].second() << "\t" << ordtab[i].third() << endl;

    bvector  B(n,true );
    i = 0;
    for( char *p = argv[3]; *p; ++p )
	B[i++] = (*p=='0') ? 0 : 1 ;

    
    string  w  = W[SM.Q[B]]; 
    cout << "    ";
    display( A );
    
    for( i = 0; i < w.size(); ++i )
    {
	A = op( A, (w[i]=='a') ? 0 : 1 );
	cout << w[i] << ":  ";
	display( A );
    }
    

}


