//---------------------------------------------------------------------------
//
//  Time-stamp: <2003-11-24 18:25:34 sutner> 
//
// :File:      refcount.cc
// :Purpose:   reference counting for UList
// 
//---------------------------------------------------------------------------


#include  "refcount.h"

//--------------------------------------------------------------------------------

RefCount::RefCount(): rcnt(new int(1)) { }

RefCount::RefCount(const RefCount& u): rcnt(u.rcnt) { ++*rcnt; }

RefCount::~RefCount() { if( --*rcnt == 0 ) delete rcnt; }

bool RefCount::unique() { return *rcnt == 1; }


// switch to new refcount, free old one if count was 1
// false:  old count > 1
// true:   old count == 1
bool  RefCount::reattach(const RefCount& src)
{
	++*src.rcnt;
	if( --*rcnt == 0 ) 
	{
		delete rcnt;
		rcnt = src.rcnt;
		return 1;
	}
	rcnt = src.rcnt;
	return 0;
}

// false:  count already 1
// true:   count was > 1, now new integer == 1
bool  RefCount::make_unique()
{
	if( *rcnt == 1 ) return 0;
	--*rcnt;
	rcnt = new int(1);
	return 1;
}

int  RefCount::refcount() const { return *rcnt; }

ostream& operator<<( ostream& os, const RefCount& rc )
{
    return  os << *rc.rcnt;
}

//--------------------------------------------------------------------------------
// eof
