//---------------------------------------------------------------------------
//
//  Time-stamp: <2003-11-24 18:30:32 sutner> 
//
// :File:      symtab.cc
// :Purpose:   Symbol table for UList valued variables.
//             Permissive semantics: return nil when undefined.
//             Use chaining to get vistas. 
//---------------------------------------------------------------------------


#include  "basic.h"
#include  "symtab.h"

//--------------------------------------------------------------------------------

bool  SymTable::find_it( sym_tab::iterator& it, const char *nm )
{
    it = find( string(nm) );
    if( it != end() ) return 1;
    
    if( next ) return next->find_it( it, nm );

    return 0;
}

bool  SymTable::assign( UList L, const char *nm )
{
    iterator  it;
    
    if( find_it( it, nm ) )  { it->second = L; return 1; }
    
    (*this)[string(nm)] = L;
    return 0;
}

bool    SymTable::assign( UList (*ff)(), const char *nm )
{
    UList  F = new Function( new NFuncOb(ff), nm );
    return assign( F, nm );
}

bool    SymTable::assign( UList (*ff)(UList), const char *nm )
{
    UList  F = new Function( new UFuncOb(ff), nm );
    return assign( F, nm );
}

bool    SymTable::assign( UList (*ff)(UList,UList), const char *nm )
{
    UList  F = new Function( new BFuncOb(ff), nm );
    return assign( F, nm );
}

bool    SymTable::assign( int (*ff)(int), const char *nm )
{
    UList  F = new Function( new WUFuncOb(ff), nm );
    return assign( F, nm );
}

bool    SymTable::assign( int (*ff)(int,int), const char *nm )
{
    UList  F = new Function( new WBFuncOb(ff), nm );
    return assign( F, nm );
}


bool    SymTable::assign( UList (*ff)(UList,UList,UList), const char *nm )
{
    UList  F = new Function( new TFuncOb(ff), nm );
    return assign( F, nm );
}



bool  SymTable::assign_local( UList L, const char *nm )
{
    iterator  it = find( string(nm) );
    
    if( it != end() ) { it->second = L; return 1; }
    
    (*this)[string(nm)] = L;
    return 0;
}

// pre:   vals is evaluated, length(vals)==length(names)
//        does not check length, types
// post:  name-value pairs entered in local environment
void  SymTable::assign_list( UList LL, char **names )
{
    UList  L;
    while( LL )
    {
	LL >> L;
	assign_local( L, *names++ );
    }
    
}

UList  SymTable::lookup( const char *nm )
{
    iterator  it;

    if( find_it( it, nm ) )
	return  it->second;

    return nil;
}

void   SymTable::print( ostream& os, bool all ) const 
{
    const_iterator it;
    os << "\nsymbol table: " << size() << endl;

    vector<Pair<string,int> > A;
    for( it = begin(); it!= end(); ++it  )
	A.push_back( Pair<string,int>( string(it->first), arity(it->second) ) );

    sort( A.begin(), A.end() );
    
    for( size_t i = 0; i < A.size(); ++i ) 
    {
	os << A[i].first();
	os << ( ( A[i].first().size() >= 8 ) ? "\t" : "\t\t" );
	os << A[i].second() << endl;
    }
    os << endl;

    if( all && next )
	next->print( os, 1 );
}

//--------------------------------------------------------------------------------

//  Local Variables: ***
//  eval: (setq case-fold-search nil) ***
//  eval: (setq case-replace nil) ***
//  End: ***
