(************** Content-type: application/mathematica **************
                     CreatedBy='Mathematica 5.0'

                    Mathematica-Compatible Notebook

This notebook can be used with any Mathematica-compatible
application, such as Mathematica, MathReader or Publicon. The data
for the notebook starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do
one of the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the
  application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing
the word CacheID, otherwise Mathematica-compatible applications may
try to use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
*******************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     14660,        488]*)
(*NotebookOutlinePosition[     16654,        558]*)
(*  CellTagsIndexPosition[     16535,        550]*)
(*WindowFrame->Normal*)



Notebook[{
Cell["\[Copyright] 2003 K. Sutner ", "SmallText"],

Cell[TextData[StyleBox["External Code",
  FontFamily->"Charter"]], "Title",
  Evaluatable->False,
  AspectRatioFixed->True,
  CellTags->"c:1"],

Cell[CellGroupData[{

Cell["Compiling and Installation", "Section",
  Evaluatable->False,
  AspectRatioFixed->True,
  CellTags->"c:7"],

Cell["\<\
The external code consists mainly of a collection of C++ programs. \
It should compile (perhaps after some minor adjustments) in any standard Unix \
environment. The following set-up is known to work under RedHat 8/9:\
\>", \
"Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["\<\
g++/gcc\t\t\t\t3.2
bison \t\t\t\t1.35
gmp\t\t\t\t\t4.1.2
MLMATHVERSION  \t\t5.0.0\
\>", "Program",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  "You should only have to modify three paths in the ",
  StyleBox["Makefile", "MR"],
  ". "
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[StyleBox["BINDIR = /home/sutner/Bin\nEXTBINDIR = \
/home/sutner/.Mathematica/Applications/Automata/\nMLDIR = \
/usr/local/.../MathLink/DeveloperKit/Linux/CompilerAdditions", "MR"]], \
"Program",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  "Then type  ",
  StyleBox["make all", "MR"],
  ".  This will compile and install the following three executables:\n\t",
  StyleBox["extaut", "MR"],
  "\tused via MathLink\n\t",
  StyleBox["autom", "MR"],
  "\t\tcommandline executable\n\t",
  StyleBox["automata", "MR"],
  "\tinteractive version, input directly to parser\nas well as a number of \
other programs that are useful in studying Fischer automata. Examples for the \
use of ",
  StyleBox["autom", "MR"],
  " and ",
  StyleBox["automata", "MR"],
  " can be found in the  ",
  StyleBox["External/Examples", "MR"],
  "  subdirectory of the distribution."
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["Limitations", "Subsubsection"],

Cell[TextData[{
  "I regret not being able to provide any support for Windows or Mac.  If \
someone ports the external code to one of these environments I would \
appreciate if you would let me know.\nSee the ",
  StyleBox[" ",
    FontVariations->{"CompatibilityType"->0}],
  Cell[BoxData[
      FormBox[
        ButtonBox["README",
          ButtonData:>{"README", None},
          ButtonStyle->"Hyperlink"], TraditionalForm]]],
  "  file for more information on bugs/limitations of the external code."
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Parser and Command Language", "Section",
  Evaluatable->False,
  AspectRatioFixed->True,
  CellTags->"c:7"],

Cell[TextData[{
  "The reentrant parser that is used in all three executable is currently \
rather simple and has essentially no error handling.  This is relevant only \
for ",
  StyleBox["automata", "MR"],
  " where the user directly interacts with the parser rather than being \
shielded from syntax details. It will be rewritten in the near future.  At \
that point it is likely that the command language will also change, so we \
refrain from giving a complete description of the current (entirely \
atrocious) syntax.  Instead, we present a list of usage examples.  All \
examples can be found in subdirectories of\n\t\t",
  StyleBox["External/Examples/", "MR"],
  "\t\nin the distribution."
}], "Text"],

Cell[CellGroupData[{

Cell["autom", "Subsection",
  Evaluatable->False,
  AspectRatioFixed->True,
  CellTags->"c:7"],

Cell[TextData[{
  "Some operations on finite state machines can be performed directly from \
the command line via ",
  StyleBox["autom", "MR"],
  ", the stand-alone version of the external code. The general syntax is "
}], "Text"],

Cell["autom -acCdiIfFkmnpPqsStuw [infile] [outfile]", "Program"],

Cell["\<\
where the options control the type of operation to be performed.   \
Here is a complete listing of all admissible parameters.\
\>", "Text"],

Cell["\<\
a:      compute accessible part
c:      classify cellular automaton
C:      convert cellular automaton to semiautomaton
d:      copy input atom to output file
f:      compute Fischer data
F:      compute synchronizing word for minimal Fischer automaton
i:      compute product DFA (intersection) of two DFAs
I:      check if two DFAs are isomorphic
k:      compute kernel automaton of a semiautomaton
m:      compute minimal automton
n:      turn normalization off
p:      compute power automaton
P:      compute size of power automaton
u:      compute (left) quotient of two DFAs
s:      generate syntactic semigroup
S:      generate syntactic semigroup and D-class decomposition
t:      display timing
u:      compute product DFA (union) of two DFAs
w:      convert cellular automaton to Welch automata\
\>", "Program"],

Cell[TextData[{
  "For more complicated or repeated computations it is often best to write a \
shell script that controls the calls to ",
  StyleBox["autom", "MR"],
  ".  As an example, here is the main loop of a tcsh script that uses ",
  StyleBox["zigzag", "MR"],
  " and ",
  StyleBox["autom", "MR"],
  " to compute the size of the the minimal DFA associated with all 4096 \
1-permutation cellular automata of width 5 over a binary alphabet."
}], "Text"],

Cell[TextData[{
  "while( $b < 128 )\n        while( $e < 32 )\n                set size = \
`",
  StyleBox["zigzag",
    FontColor->RGBColor[0, 0, 1]],
  " -S $e -b $b $DEBRUIJN | ",
  StyleBox["autom",
    FontColor->RGBColor[0, 0, 1]],
  " -P `\n                @ d = 65536 - $size\n                echo  \
${b}\\\\t${e}\\\\t${size}\\\\t${d}\n                @ e = $e + 1\n        end\
\n        @ e = 0\n        @ b = $b + 1\nend"
}], "Program"],

Cell[TextData[{
  "Note that the calls to ",
  StyleBox["autom", "MR"],
  " on the various inputs are independent.  This incurs a certain overhead \
that can be avoided using ",
  StyleBox["automata", "MR"],
  "."
}], "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["automata", "Subsection",
  Evaluatable->False,
  AspectRatioFixed->True,
  CellTags->"c:7"],

Cell[TextData[{
  "The executable ",
  StyleBox["automata", "MR"],
  " behaves like a simple interactive calculator that allows the user to give \
input directly to the parser.  Since the parser at present has only the most \
primitive error recovery some care is required in preparing the input. Often \
it is best to store the input in a file.  For example, the following code \
fragment will generate the syntactic semigroups associated with the 256 \
elementary cellular automata and print out the sizes of the minimal DFAs as \
well as the semigroup sizes. "
}], "Text"],

Cell["\<\
# ca2sg.prg

x = 0;
L = ();
while( x < 256 )
{
        c = CA[3,2,x]
        s = tosa[c]
        d = minimize[todfa[s]]
        g = togen[d]
        s = generate[g]
        L = pre[ L, (x, size[d], size[s]) ]
        x = inc[x]
};
\
\>", "Program"],

Cell[TextData[{
  "The program can be called like so. It generates no output, but stores the \
results in a list ",
  StyleBox["L", "MR"],
  ".  After execution, ",
  StyleBox["automata", "MR"],
  " accepts input from ",
  StyleBox["stdin", "MR"],
  ".  In the session below, we reverse ",
  StyleBox["L", "MR"],
  " and display element number 100."
}], "Text"],

Cell["\<\
$ automata ca2sg.prg -

> L = rev[L]; length[L]

>   256

> at[ L, 100 ]
  (100, 5, 21)

> *** time: 2.24\
\>", "Program"],

Cell[TextData[{
  "A fairly large group of examples and tests can be found in ",
  StyleBox["longtest.prg", "MR"],
  ".  It prints a few data objects and then goes through a sequence of \
computations that should all result in output 1."
}], "Text"],

Cell["\<\
$ automata longtest.prg
urelements  --------------------------------------------------
123456
-123456
11111111111111122222222222222222223333333333333333333344444444444444444444
abcdefgh
((1, 2, ((aa, bb), 3), -5555), (4, (5, (6))))
FA[ 12, 2, ((1,1,2), (2,1,3), (4,1,5), (5,1,6), (7,2,8), (8,2,9), (4,0,7), \
(6,0,9), (3,0,4), (4,0,10), (9,0,4), (10,2,11), (11,2,12)), (1), (12)]
FA[ 12, 2, ((1,1,2), (2,1,3), (2,1,4), (2,1,7), (2,1,10), (4,1,5), (5,1,6), \
(5,1,9), (5,1,4), (5,1,7), (5,1,10), (7,2,8), (8,2,9), (8,2,4), (8,2,7), \
(8,2,10), (10,2,11), (11,2,12)), (1), (12)]
DFA[ 8, 2, ((2, 4, 3, 5, 7, 3, 5, 5), (3, 3, 3, 6, 3, 8, 6, 6)), 1, (8)]
DFA[ 6, 2, ((5, 3, 4, 3, 5, 3), (2, 1, 5, 1, 5, 5)), 6, (2)]
CA[ 7, 2, 170141183460469231731687303715884105728]
list operations  --------------------------------------------------
1
1
1
1
1

[ elided ]

1
1
graphs  --------------------------------------------------
1
1
1
1\
\>", "Program"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["The External Link", "Section",
  Evaluatable->False,
  AspectRatioFixed->True,
  CellTags->"c:7"],

Cell[CellGroupData[{

Cell["extaut", "Subsection",
  Evaluatable->False,
  AspectRatioFixed->True,
  CellTags->"c:7"],

Cell[TextData[{
  "Several of the internal commands in Automata have an alternative external \
implementation.  When the system variable ",
  StyleBox["$defaultexternal", "MR"],
  " is set to ",
  StyleBox["True", "MR"],
  " these command use the external rather than the internal code. This \
behavior can be overridden locally using the option ",
  StyleBox["External->False", "MR"],
  "."
}], "Text"],

Cell[BoxData[
    FormBox[GridBox[{
          {"ExtAccessibleFA", "AccessibleFA"},
          {"ExtBalancedQCA", "-"},
          {"ExtClassifyCA", "ClassifyCA"},
          {"ExtComposeCA", "ComposeCA"},
          {"ExtDefiniteQFA", "DefiniteQFA"},
          {"ExtEpsilonEliminationFA", "EpsilonEliminationFA"},
          {"ExtEquivalentQFA", "EquivalentQFA"},
          {"ExtGenerateSG", "GenerateSG"},
          {"ExtIsomorphicQDFA", "IsomorphicQDFA"},
          {"ExtMinimizeFASize", "MinimizeFA"},
          {"ExtMinimizeFA", "MinimizeFA"},
          {"ExtOrbitCA", "OrbitCA"},
          {"ExtProductFA", "ProductFA"},
          {"ExtQuotientFA", "QuotientDFA"},
          {"ExtREToFA", "RegularExpressionToFA"},
          {"ExtStronglyConnectedComponents", "StronglyConnectedComponents"},
          {"ExtSyntacticSG", "SyntacticSG"},
          {"ExtToDFASize", "ToDFA"},
          {"ExtToDFAWitnesses", "ToDFA"},
          {"ExtToDFA", "ToDFA"},
          {"ExtToKernelFA", "ToKernelFA"},
          {"ExtToWelchCA", "ToWelchCA"},
          {"ExtWelchCA", "WelchCA"}
          },
        ColumnAlignments->{Left}], TraditionalForm]], "DefinitionBox",
  GridBoxOptions->{RowSpacings->0.4,
  RowLines->False},
  CellTags->"S5.18.1"],

Cell[TextData[{
  "Some effort has gone into making the output of the external code appear as \
similar to the output of the internal code as possible. However, since the \
algorithms differ in some places slight differences in the output are \
inevitable. For example, an isomorphic machine may be produced or the \
representation of an object such as a ",
  Cell[BoxData[
      \(TraditionalForm\`D\)]],
  "-class may change. "
}], "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["SendProgram", "Subsection",
  Evaluatable->False,
  AspectRatioFixed->True,
  CellTags->"c:7"],

Cell[TextData[{
  "All the commands in the last section use the same interface to the \
external code, a function ",
  StyleBox["SendProgram", "MR"],
  " that is a wrapper for the actual link function ",
  StyleBox["MLParseProg", "MR"],
  "."
}], "Text"],

Cell["\<\
:Begin:
:Function:\t\t\tML_parse
:Pattern:\t\t\tMLParseProg[str_String]
:Arguments:\t\t\t{str}
:ArgumentTypes:\t\t{String}
:ReturnType:\t\tManual
:End:\
\>", "Program"],

Cell[TextData[{
  "It is possible to use  ",
  StyleBox["SendProgram", "MR"],
  "  directly, but that requires some understanding of the command language \
used the external code.  It is probably a good idea to test the code in ",
  StyleBox["automata", "MR"],
  " before using it over ",
  StyleBox["MathLink",
    FontSlant->"Italic"],
  ". The occasion where such direct programming might be advantageous if it \
eliminates ping-pong traffic over the link: instead of sending intermediate \
results back and forth repeatedly, one sends the original data only, performs \
all the necessary compuations in the external code, then sends back only the \
final result."
}], "Text"],

Cell[BoxData[
    \(ExtAutomata[]\)], "Input"],

Cell[BoxData[
    \(SendProgram[12345]\)], "Input"],

Cell["\<\
The external code maintains state between calls, so data can be \
stored externally.\
\>", "Text"],

Cell[BoxData[{
    \(SendProgram["\<r = range[10];\>"]\), "\n", 
    \(2\  + \ 2\), "\n", 
    \(SendProgram["\<r\>"]\)}], "Input"],

Cell[TextData[{
  StyleBox["SendProgram", "MR"],
  " will stringify some expressions such as machines and cellular automata to \
facilitate input."
}], "Text"],

Cell[BoxData[{
    \(\(SendProgram["\<c =\>", CA[3, 2, 51]];\)\), "\n", 
    \(SendProgram["\<analyze[c]\>"]\)}], "Input"]
}, Closed]]
}, Closed]]
},
FrontEndVersion->"5.0 for X",
ScreenRectangle->{{0, 1280}, {0, 1024}},
AutoGeneratedPackage->None,
WindowToolbars->{},
CellGrouping->Automatic,
WindowSize->{1111, 848},
WindowMargins->{{Automatic, 3}, {Automatic, 44}},
PrintingStartingPageNumber->1,
PrivateNotebookOptions->{"ColorPalette"->{RGBColor, 128}},
ShowSelection->True,
ShowCellLabel->False,
CellLabelAutoDelete->True,
ShowCellTags->False,
RenderingOptions->{"ObjectDithering"->True,
"RasterDithering"->False},
CharacterEncoding->Automatic,
Magnification->1.5,
StyleDefinitions -> "TextDemoMod.nb"
]

(*******************************************************************
Cached data follows.  If you edit this Notebook file directly, not
using Mathematica, you must remove the line containing CacheID at
the top of  the file.  The cache data will then be recreated when
you save this file from within Mathematica.
*******************************************************************)

(*CellTagsOutline
CellTagsIndex->{
  "c:1"->{
    Cell[1806, 53, 142, 4, 107, "Title",
      Evaluatable->False,
      CellTags->"c:1"]},
  "c:7"->{
    Cell[1973, 61, 112, 3, 77, "Section",
      Evaluatable->False,
      CellTags->"c:7"],
    Cell[4337, 145, 113, 3, 45, "Section",
      Evaluatable->False,
      CellTags->"c:7"],
    Cell[5186, 167, 94, 3, 63, "Subsection",
      Evaluatable->False,
      CellTags->"c:7"],
    Cell[7743, 245, 97, 3, 63, "Subsection",
      Evaluatable->False,
      CellTags->"c:7"],
    Cell[10432, 351, 103, 3, 45, "Section",
      Evaluatable->False,
      CellTags->"c:7"],
    Cell[10560, 358, 95, 3, 63, "Subsection",
      Evaluatable->False,
      CellTags->"c:7"],
    Cell[12777, 420, 100, 3, 63, "Subsection",
      Evaluatable->False,
      CellTags->"c:7"]},
  "S5.18.1"->{
    Cell[11064, 375, 1232, 29, 519, "DefinitionBox",
      CellTags->"S5.18.1"]}
  }
*)

(*CellTagsIndex
CellTagsIndex->{
  {"c:1", 15647, 517},
  {"c:7", 15750, 521},
  {"S5.18.1", 16428, 543}
  }
*)

(*NotebookFileOutline
Notebook[{
Cell[1754, 51, 49, 0, 41, "SmallText"],
Cell[1806, 53, 142, 4, 107, "Title",
  Evaluatable->False,
  CellTags->"c:1"],

Cell[CellGroupData[{
Cell[1973, 61, 112, 3, 77, "Section",
  Evaluatable->False,
  CellTags->"c:7"],
Cell[2088, 66, 291, 7, 69, "Text",
  Evaluatable->False],
Cell[2382, 75, 156, 7, 117, "Program",
  Evaluatable->False],
Cell[2541, 84, 168, 6, 42, "Text",
  Evaluatable->False],
Cell[2712, 92, 256, 5, 97, "Program",
  Evaluatable->False],
Cell[2971, 99, 689, 20, 225, "Text",
  Evaluatable->False],

Cell[CellGroupData[{
Cell[3685, 123, 36, 0, 64, "Subsubsection"],
Cell[3724, 125, 564, 14, 109, "Text",
  Evaluatable->False]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[4337, 145, 113, 3, 45, "Section",
  Evaluatable->False,
  CellTags->"c:7"],
Cell[4453, 150, 708, 13, 228, "Text"],

Cell[CellGroupData[{
Cell[5186, 167, 94, 3, 63, "Subsection",
  Evaluatable->False,
  CellTags->"c:7"],
Cell[5283, 172, 230, 5, 69, "Text"],
Cell[5516, 179, 64, 0, 57, "Program"],
Cell[5583, 181, 149, 3, 42, "Text"],
Cell[5735, 186, 831, 20, 417, "Program"],
Cell[6569, 208, 457, 10, 96, "Text"],
Cell[7029, 220, 449, 11, 237, "Program"],
Cell[7481, 233, 225, 7, 69, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[7743, 245, 97, 3, 63, "Subsection",
  Evaluatable->False,
  CellTags->"c:7"],
Cell[7843, 250, 575, 10, 123, "Text"],
Cell[8421, 262, 258, 16, 337, "Program"],
Cell[8682, 280, 361, 11, 69, "Text"],
Cell[9046, 293, 132, 11, 237, "Program"],
Cell[9181, 306, 249, 5, 69, "Text"],
Cell[9433, 313, 950, 32, 657, "Program"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[10432, 351, 103, 3, 45, "Section",
  Evaluatable->False,
  CellTags->"c:7"],

Cell[CellGroupData[{
Cell[10560, 358, 95, 3, 63, "Subsection",
  Evaluatable->False,
  CellTags->"c:7"],
Cell[10658, 363, 403, 10, 96, "Text"],
Cell[11064, 375, 1232, 29, 519, "DefinitionBox",
  CellTags->"S5.18.1"],
Cell[12299, 406, 441, 9, 96, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[12777, 420, 100, 3, 63, "Subsection",
  Evaluatable->False,
  CellTags->"c:7"],
Cell[12880, 425, 254, 7, 69, "Text"],
Cell[13137, 434, 178, 8, 177, "Program"],
Cell[13318, 444, 679, 14, 150, "Text"],
Cell[14000, 460, 46, 1, 51, "Input"],
Cell[14049, 463, 51, 1, 51, "Input"],
Cell[14103, 466, 108, 3, 42, "Text"],
Cell[14214, 471, 131, 3, 97, "Input"],
Cell[14348, 476, 159, 4, 42, "Text"],
Cell[14510, 482, 122, 2, 74, "Input"]
}, Closed]]
}, Closed]]
}
]
*)



(*******************************************************************
End of Mathematica Notebook file.
*******************************************************************)

