(************** Content-type: application/mathematica **************
                     CreatedBy='Mathematica 5.0'

                    Mathematica-Compatible Notebook

This notebook can be used with any Mathematica-compatible
application, such as Mathematica, MathReader or Publicon. The data
for the notebook starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do
one of the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the
  application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing
the word CacheID, otherwise Mathematica-compatible applications may
try to use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
*******************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[    113121,       3808]*)
(*NotebookOutlinePosition[    118573,       3958]*)
(*  CellTagsIndexPosition[    118344,       3946]*)
(*WindowFrame->Normal*)



Notebook[{
Cell["\[Copyright] 2003 K. Sutner", "SmallText"],

Cell[CellGroupData[{

Cell["Automata Examples", "Title"],

Cell[CellGroupData[{

Cell[TextData[StyleBox["Requirements",
  FontColor->RGBColor[0, 0, 1]]], "Subsubsection"],

Cell[TextData[{
  "This notebook depends on ",
  StyleBox["Automata", "MR"],
  ", available at  ",
  ButtonBox["www.cs.cmu.edu/~sutner",
    ButtonData:>{
      URL[ "http://www.cs.cmu.edu/~sutner"], None},
    ButtonStyle->"Hyperlink"],
  ". "
}], "Text"],

Cell[BoxData[{
    \(Needs["\<Automata`automata`\>"]\), "\[IndentingNewLine]", 
    \(Get["\<Automata`experimental`\>"]\), "\[IndentingNewLine]", 
    \(Get["\<Automata`obsolete`\>"]\)}], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["Sample Languages", "Section",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["Forbidden Factors", "Subsection",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  "Let ",
  Cell[BoxData[
      \(TraditionalForm\`L\)]],
  " be the the language of all words not containing any of the factors   ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          StyleBox["aaa",
            FontSlant->"Italic"], ",", " ", 
          StyleBox["aba",
            FontSlant->"Italic"], ",", " ", 
          StyleBox["bab",
            FontSlant->"Italic"], ",", " ", 
          StyleBox["bbb",
            FontSlant->"Italic"]}], TraditionalForm]]],
  "  over the two-letter alphabet ",
  Cell[BoxData[
      \(TraditionalForm\`\[CapitalSigma]\  = \ {a, b}\)]],
  ".   We construct the minimal DFA for ",
  Cell[BoxData[
      \(TraditionalForm\`L\)]],
  "  using  ",
  ButtonBox["ConcatenateFA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[BoxData[
    \(With[\ {all\  = \ AllDFA[2], \ \[IndentingNewLine]\t
        m0\  = \ BuildDFA[{aaa, bbb, aba, bab}, 2]}, \[IndentingNewLine]\t
      m = MinimizeFA[ComplementFA[ConcatenateFA[all, m0, all]]]\ ]\)], "Input",\

  AspectRatioFixed->True],

Cell["The growth function appears to be eventually constant.", "Text"],

Cell[BoxData[
    \(LanguageFA[m, \(-20\), SizeOnly \[Rule] True]\)], "Input",
  AspectRatioFixed->True],

Cell["\<\
Which fact we can verify via the generating function and some \
simplification of infinite series.\
\>", "Text"],

Cell[BoxData[{
    \(Clear[x]\), "\[IndentingNewLine]", 
    \(gs\  = \ GrowthSeriesDFA[m, x]\)}], "Input"],

Cell[BoxData[
    \(Series[\ gs, \ {x, 0, 10}\ ]\)], "Input"],

Cell[TextData[{
  "A complete proof (assuming we trust ",
  ButtonBox["Simplify",
    ButtonStyle->"RefGuideLink"],
  " and ",
  ButtonBox["GrowthSeriesDFA",
    ButtonStyle->"RefGuideLink"],
  ")."
}], "Text"],

Cell[BoxData[{
    \(1\  + \ 2  x\  + \ \ 4\ Sum[\ x^i, \ {i, 2, Infinity}\ ]\  // \ 
      Together\), "\[IndentingNewLine]", 
    \(gs\  == \ %\ \  // \ Simplify\)}], "Input"],

Cell[TextData[{
  "The complement  ",
  Cell[BoxData[
      \(TraditionalForm\`\(\[CapitalSigma]\^*\) - \ L\)]],
  "  clearly is an ideal."
}], "Text"],

Cell[BoxData[
    \(IdealQFA[ComplementFA[m]]\)], "Input"],

Cell["We compute a DFA accepting a base for this ideal.", "Text"],

Cell[BoxData[{
    \(ib\  = \ IdealBaseFA[\ ComplementFA[m]]\), "\[IndentingNewLine]", 
    \(LanguageFA[ib, Full \[Rule] True]\)}], "Input"],

Cell[TextData[{
  "The language consists of all factors of the infinite word  ",
  Cell[BoxData[
      FormBox[
        SuperscriptBox[
          RowBox[{"(", 
            StyleBox["aabb",
              FontSlant->"Italic"], 
            StyleBox[")",
              FontSlant->"Italic"]}], \(\(\ \)\(\[Omega]\)\)], 
        TraditionalForm]]],
  "."
}], "Text"],

Cell[BoxData[
    \(LanguageFA[m, \(-12\)]\  // \ EnumForm0\)], "Input",
  AspectRatioFixed->True],

Cell[TextData[{
  "Clearly, ",
  Cell[BoxData[
      \(TraditionalForm\`L\)]],
  " is locally testable. Here is a semiautomaton obtained from ",
  ButtonBox["LocalUniversalFA",
    ButtonStyle->"AddOnsLink"],
  "  which turns out to be equivalent to ",
  Cell[BoxData[
      \(TraditionalForm\`m\)]],
  ". "
}], "Text"],

Cell[BoxData[{
    \(sa\  = \ 
      ToSA@LocalUniversalFA[2, 2, 
          EdgeLU \[Rule] \((\(! MemberQ[{"\<aaa\>", "\<bbb\>", "\<aba\>", \
"\<bab\>"}, #]\) &)\)]\), "\n", 
    \(EquivalentQFA[sa, m]\)}], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["Squares of Languages", "Subsection",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  "We construct a simple example of a regular language ",
  Cell[BoxData[
      \(TraditionalForm\`L\  \[NotEqual] \ \(\[CapitalSigma]\^*\)\)]],
  " such that ",
  Cell[BoxData[
      \(TraditionalForm\`L\ \[CenterDot]\ L\  = \ \(\[CapitalSigma]\^*\)\)]],
  "  over the alphabet  ",
  Cell[BoxData[
      \(TraditionalForm\`\[CapitalSigma]\  = \ {a, b}\)]],
  ". \nHere is a regular expression for ",
  Cell[BoxData[
      \(TraditionalForm\`L\)]],
  ":   ",
  Cell[BoxData[
      \(TraditionalForm\`\[Epsilon]\  + \ 
        a\ \(\[CapitalSigma]\^*\) + \ \(\[CapitalSigma]\^*\) b\)]],
  "."
}], "Text"],

Cell[BoxData[{
    \(Clear[a, b, c]\), "\[IndentingNewLine]", 
    \(\(all\  = \ RES[REP[a, b]];\)\), "\n", 
    \(\(exp\  = \ \ \ REP[Eps\ , \ RET[a, all]\ , \ 
          RET[all, b]\ ];\)\), "\[IndentingNewLine]", 
    \(exp\  // \ PrintRE\)}], "Input"],

Cell[BoxData[
    \(m = MinimizeFA[RegularExpressionToFA[exp, 2]]\)], "Input",
  AspectRatioFixed->True],

Cell["We check the growth function.", "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[BoxData[
    \(LanguageFA[m, \(-10\), SizeOnly \[Rule] True]\)], "Input",
  AspectRatioFixed->True],

Cell[TextData[{
  "It is easier to inspect a few words in the complement of ",
  Cell[BoxData[
      \(TraditionalForm\`L\)]],
  "."
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[BoxData[
    \(LanguageFA[ComplementFA[m], \(-5\)]\)], "Input",
  AspectRatioFixed->True],

Cell[TextData[{
  "It seems the complement of ",
  Cell[BoxData[
      \(TraditionalForm\`L\)]],
  "  has the form  ",
  Cell[BoxData[
      \(TraditionalForm\`b\ \(\[CapitalSigma]\^*\)\ a\)]],
  ", a claim easily verified by constructing the corresponding machines."
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[BoxData[
    \(\(\(expc\)\(\ \)\(=\)\(\ \ \ \)\(RET[b, all, a]\)\(\ \)\)\)], "Input"],

Cell[BoxData[
    \(EquivalentQFA[ComplementFA[m], 
      RegularExpressionToFA[expc, 2]]\)], "Input",
  AspectRatioFixed->True],

Cell[TextData[{
  "Next we compute a machine for ",
  Cell[BoxData[
      \(TraditionalForm\`L\[CenterDot]L\)]],
  ", and check that indeed  ",
  Cell[BoxData[
      \(TraditionalForm\`L\ \[CenterDot]\ L\  = \ \(\[CapitalSigma]\^*\)\)]],
  "."
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[BoxData[{
    \(mm = ConcatenateFA[m, m]\), "\[IndentingNewLine]", 
    \(mm\  // \ FullQFA\)}], "Input",
  AspectRatioFixed->True],

Cell[TextData[{
  "Only the relative order of ",
  Cell[BoxData[
      \(TraditionalForm\`a\)]],
  " and ",
  Cell[BoxData[
      \(TraditionalForm\`b\)]],
  " matters: both correspond to idempotent elements in the syntactic \
semigroup."
}], "Text"],

Cell[BoxData[{
    \(\({S, W, eq, dcl} = 
        SyntacticSG[m, DClasses \[Rule] True];\)\), "\[IndentingNewLine]", 
    \(S\  // \ Length\)}], "Input",
  AspectRatioFixed->True],

Cell[BoxData[
    \(PrintWitnesses[S, W]\)], "Input",
  AspectRatioFixed->True],

Cell[BoxData[
    \(PrintEquations[SimplifyEquations[S, W, eq]]\)], "Input",
  AspectRatioFixed->True],

Cell["All the elements are idempotent.", "Text"],

Cell[BoxData[{
    \(\(\(PlotT[#, 5, DisplayFunction \[Rule] Identity] &\)\  /@ \ 
        ToList[S];\)\), "\[IndentingNewLine]", 
    \(\(ShowArray[%];\)\)}], "Input"],

Cell[BoxData[
    \(IdemSG[S]\)], "Input"],

Cell[TextData[{
  "There is only one regular ",
  Cell[BoxData[
      \(TraditionalForm\`D\)]],
  "-class, the whole semigroup."
}], "Text"],

Cell[BoxData[
    \(DClassSize[dcl, Full \[Rule] True]\)], "Input"],

Cell[BoxData[{
    \(WitnessFunction[witf, S, W]\), "\n", 
    \(\(\(DClassToBlocks[dcl]\  // \ First\) // \ 
        witf\)\  // \ \ TableForm\)}], "Input",
  AspectRatioFixed->True],

Cell["The multiplication table of the syntactic semigroup.", "Text"],

Cell[BoxData[
    \(TableSG[S, W]\)], "Input"],

Cell[TextData[{
  "A slightly more ambitious example over a three-letter alphabet with \
regular expression:\n\n\t\t",
  Cell[BoxData[
      \(TraditionalForm\`\(\(c\^\(\(*\)\(\ \)\)\)(\[Epsilon]\  + \ 
            a\ \ \(\[CapitalSigma]\^*\) + \ \(\[CapitalSigma]\^*\) 
              b)\)\ \(c\^*\)\)]],
  "\n"
}], "Text"],

Cell[BoxData[{
    \(\(all\  = \ RES[REP[a, b, c]];\)\), "\n", 
    \(exp\  = \ \ RET[\ RES[c], \ REP[Eps\ , \ RET[a, all]\ , \ RET[all, b]], 
        RES[c]]\)}], "Input"],

Cell[BoxData[
    \(m = MinimizeFA[RegularExpressionToFA[exp, 3]]\)], "Input",
  AspectRatioFixed->True],

Cell[TextData[{
  "The complement is a bit more complicated in this case and is denoted by \
the regular expression  ",
  Cell[BoxData[
      \(TraditionalForm\`\(c\^*\) b\ \(\[CapitalSigma]\^*\)\ a\ \(c\^*\)\)]],
  ". "
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[BoxData[
    \(LanguageFA[ComplementFA[m], \(-5\)]\)], "Input",
  AspectRatioFixed->True],

Cell[BoxData[{
    \(\(expc\  = \ 
        RET[\ RES[c], b, all, a, RES[c]];\)\), "\[IndentingNewLine]", 
    \(EquivalentQFA[\ RegularExpressionToFA[expc, 3], \ 
      ComplementFA[m]]\)}], "Input"],

Cell["\<\
It may help to construct the syntactic semigroup to understand what \
is going on here. \
\>", "Text"],

Cell[BoxData[{
    \(\({S, W, eq, dcl} = 
        SyntacticSG[m, DClasses \[Rule] True];\)\), "\[IndentingNewLine]", 
    \(S\  // \ Length\)}], "Input",
  AspectRatioFixed->True],

Cell[BoxData[
    \(PrintWitnesses[S, W]\)], "Input",
  AspectRatioFixed->True],

Cell[BoxData[
    \(PrintEquations[SimplifyEquations[S, W, eq]]\)], "Input",
  AspectRatioFixed->True],

Cell[BoxData[
    \(TableSG[S, W]\)], "Input"],

Cell[TextData[{
  "The ",
  Cell[BoxData[
      \(TraditionalForm\`c\)]],
  "'s are simply ignored.  In hindsight, this is clear from the DFA (see the \
last row of the transition matrix), but it certainly is not obvious from the \
regular expression.  There are now two ",
  Cell[BoxData[
      \(TraditionalForm\`D\)]],
  "-classes, one as in the last example, and a second one consisting only of \
the idempotent ",
  Cell[BoxData[
      \(TraditionalForm\`c\)]],
  "."
}], "Text"],

Cell[BoxData[
    \(DClassSize[dcl, Full \[Rule] True]\)], "Input"],

Cell[BoxData[{
    \(WitnessFunction[witf, S, W]\), "\n", 
    \(\(\(DClassToBlocks[dcl]\  // \ First\) // \ 
        witf\)\  // \ \ TableForm\)}], "Input",
  AspectRatioFixed->True],

Cell[BoxData[
    \(\(\(DClassToBlocks[dcl]\  // \ Last\) // \ 
        witf\)\  // \ \ TableForm\)], "Input",
  AspectRatioFixed->True],

Cell[TextData[{
  "For the suspicious, a check that ",
  Cell[BoxData[
      \(TraditionalForm\`L\^\(\(\ \)\(2\)\) = \ \[CapitalSigma]\^\(\(\ \
\)\(*\)\)\)]],
  ". "
}], "Text"],

Cell[BoxData[
    \(ConcatenateFA[m, m]\  // \ FullQFA\)], "Input"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Regular Expressions", "Section",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["RE for odd-odd", "Subsection",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  "It is fairly straightforward to construct a regular expression for the \
language of all strings over ",
  Cell[BoxData[
      \(TraditionalForm\`\[CapitalSigma]\  = \ {a, b}\)]],
  "  that have an even-even count for ",
  Cell[BoxData[
      \(TraditionalForm\`a\)]],
  " and ",
  Cell[BoxData[
      \(TraditionalForm\`b\)]],
  ". "
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[BoxData[{
    \(\(e0 = REP[aa, bb];\)\), "\n", 
    \(\(e1 = REP[ab, ba];\)\), "\[IndentingNewLine]", 
    \(e00\  = \ RES[\ REP[\ e0, \ RET[e1, RES[e0], e1]\ ]\ ]\)}], "Input",
  AspectRatioFixed->True],

Cell[BoxData[
    \(\(e00\  // \ RegularExpressionToFA\)\  // \ \ MinimizeFA\)], "Input"],

Cell[TextData[{
  "For the corresponding odd-odd language the expression is a bit larger: \
there has to a be an ",
  Cell[BoxData[
      \(TraditionalForm\`a\)]],
  " adjacent to a ",
  Cell[BoxData[
      \(TraditionalForm\`b\)]],
  ", and we can split the string into two even-even parts at that point. \
Since letters do not commute, we need two expressions of this type."
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[BoxData[
    \(e11 = \ 
      REP[\ RET[\ e00, \ ab, \ e00], RET[\ e00, \ ba, \ e00]]\)], "Input"],

Cell[BoxData[
    \(\(e11\  // \ RegularExpressionToFA\)\  // \ \ MinimizeFA\)], "Input",
  AspectRatioFixed->True],

Cell[TextData[{
  "So how about odd-even?  Unfortunately a simple split at an occurrence of \
",
  Cell[BoxData[
      \(TraditionalForm\`a\)]],
  " does not work."
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[BoxData[
    \(e10 = \ RET[e00, a, e00]\)], "Input",
  AspectRatioFixed->True],

Cell[BoxData[
    \(mm\  = \ \(e10\  // \ 
          RegularExpressionToFA\)\  // \ \ MinimizeFA\)], "Input"],

Cell[TextData[{
  "The problem lies with splits such as  ",
  Cell[BoxData[
      FormBox[
        StyleBox["bab",
          FontSlant->"Italic"], TraditionalForm]]],
  ".  We construct a machine for odd-even by hand and check that our \
expression so far misses these words."
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[BoxData[{
    \(\(m10\  = \ 
        IntersectionFA[\ CounterDFA[a, 1, 2, Modulus \[Rule] 2], 
          CounterDFA[b, 0, 2, Modulus \[Rule] 2]];\)\), "\n", 
    \(LanguageFA[\ ComplementFA[m10, mm], \ \(-6\)\ ]\)}], "Input"],

Cell[TextData[{
  "So we use two splits:  even ",
  Cell[BoxData[
      \(TraditionalForm\`b\)]],
  "-count on both sides or odd ",
  Cell[BoxData[
      \(TraditionalForm\`b\)]],
  "-count on both sides."
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[BoxData[
    \(e10 = 
      REP[\ RET[e00, a, e00], RET[e00, b, e00, a, e00, b, e00]]\)], "Input",
  AspectRatioFixed->True],

Cell["\<\
We obtain large intermediate machines, but the final automaton only \
has 4 states.\
\>", "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[BoxData[{
    \(e10\  // \ RegularExpressionToFA\), "\[IndentingNewLine]", 
    \(%\  // \ ToDFA\), "\[IndentingNewLine]", 
    \(%\  // \ \ MinimizeFA\)}], "Input"],

Cell[BoxData[
    \(EquivalentQFA[%, m10]\)], "Input"],

Cell["Works.", "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["RE avoiding a factor", "Subsection",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  "We construct an expression for all words not containing the factor ",
  Cell[BoxData[
      FormBox[
        StyleBox["aba",
          FontSlant->"Italic"], TraditionalForm]]],
  " by solving the corresponding system of equations."
}], "Text"],

Cell[BoxData[
    \(LanguageFA[\ WordToFA[aba, 2], \ \(-5\)\ ]\)], "Input"],

Cell[BoxData[{
    \(FormatX[]\), "\[IndentingNewLine]", 
    \(m = \ 
      MinimizeFA[ComplementFA[WordToFA[aba, 2, Full \[Rule] True]]]\), "\n", 
    \(\({e1, e2, e3, e4} = ToEquationsDFA[m];\)\), "\n", 
    \({e1, e2, e3, e4}\  // \ ColumnForm\)}], "Input",
  AspectRatioFixed->True],

Cell[BoxData[
    \(e44 = ArdenSolve[e4]\)], "Input",
  AspectRatioFixed->True],

Cell[BoxData[
    \(e33 = SubstituteRE[e3, e44]\)], "Input",
  AspectRatioFixed->True],

Cell[BoxData[
    \(e21 = SubstituteRE[e2, e33]\)], "Input",
  AspectRatioFixed->True],

Cell[BoxData[
    \(e22 = ArdenSolve[e21]\)], "Input",
  AspectRatioFixed->True],

Cell[BoxData[
    \(e11 = SubstituteRE[e1, e22]\)], "Input",
  AspectRatioFixed->True],

Cell[BoxData[
    \(sol = ArdenSolve[e11]\)], "Input",
  AspectRatioFixed->True],

Cell[BoxData[
    \(sol\)], "Input"],

Cell[BoxData[
    \(m\  = \ \(\(sol\  // \ Last\)\  // \ RegularExpressionToFA\)\  // \ 
        MinimizeFA\)], "Input",
  AspectRatioFixed->True],

Cell[BoxData[
    \(LanguageFA[ComplementFA[m], \(-5\)]\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[{
  "Conway's ",
  Cell[BoxData[
      \(TraditionalForm\`R\_n\)]]
}], "Subsection",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  "The following expressions are equivalent to ",
  Cell[BoxData[
      \(TraditionalForm\`\((a + b)\)\^\(\(\ \)\(*\)\)\)]],
  ", for all ",
  Cell[BoxData[
      \(TraditionalForm\`n\)]],
  ".\nHowever, this equivalence cannot be proven from axioms ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          FormBox[\(C\_\(\(\ \)\(1\)\)\),
            "TraditionalForm"], "-", \(C\_14\)}], TraditionalForm]]],
  " . "
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[BoxData[{
    \(Clear[REPower, e, a, b, c, d]\), "\n", 
    \(\(REPower[ee_, 0] := Eps;\)\), "\n", 
    \(\(REPower[ee_, 1] := ee;\)\), "\n", 
    \(\(REPower[ee_, n_Integer?NonNegative] := 
        RET @@ Table[ee, {n}];\)\)}], "Input",
  AspectRatioFixed->True],

Cell[BoxData[
    \(Rn[n_Integer]\  := \ \[IndentingNewLine]With[\ {e\  = \ 
            RET[a, RES[b]]}, \ 
        REP[RES[b], 
          RET[\ RES[b], RES[REPower[\ e, n]], 
            REPower[\ RET[REP[Eps, a], RES[b]], n]]]]\)], "Input"],

Cell[BoxData[{
    \(Rn[16]\), "\[IndentingNewLine]", 
    \(%\  // \ RegularExpressionToFA\), "\[IndentingNewLine]", 
    \(%\  // \ ToDFA\), "\[IndentingNewLine]", 
    \(%\  // \ FullQFA\)}], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["The Semiring of RE Matrices", "Subsection",
  Evaluatable->False,
  AspectRatioFixed->True,
  CellTags->"c:4"],

Cell["\<\
The following code implements multiplication and Kleene star in the \
semiring of matrices of regular expressions.  Some applications follow.\
\>", \
"Text"],

Cell[CellGroupData[{

Cell["Code", "Subsubsection"],

Cell[BoxData[{
    \(\(\(<< \
"\<LinearAlgebra`MatrixManipulation`\>";\)\(\[IndentingNewLine]\)
    \)\), "\n", 
    \(\(\(Clear[MatrixTake, MatrixMultiply, MatrixStar];\)\(\n\)
    \)\), "\[IndentingNewLine]", 
    \(\(\(MatrixTake[A_?MatrixQ, n_Integer, 
          m_Integer] := \((Take[#1, m] &)\) /@ Take[A, n];\)\(\n\)
    \)\), "\[IndentingNewLine]", 
    \(\(\(MatrixMultiply[A_?MatrixQ] := A;\)\(\[IndentingNewLine]\)
    \)\), "\n", 
    \(\(\(MatrixMultiply[A_?MatrixQ, 
          B_?MatrixQ] := \[IndentingNewLine]Module[{n, m, mm, p, 
            BB}, \[IndentingNewLine]{n, m} = 
            Dimensions[A]; \[IndentingNewLine]{mm, p} = 
            Dimensions[B]; \[IndentingNewLine]If[m =!= mm, 
            Return[Null]]; \[IndentingNewLine]BB = 
            Thread[B]; \
\[IndentingNewLine]RESimplify[\[IndentingNewLine]Partition[\ \(Inner[
                    NonCommutativeMultiply, 
                    A\[LeftDoubleBracket]#1\[LeftDoubleBracket]1\
\[RightDoubleBracket]\[RightDoubleBracket], 
                    BB\[LeftDoubleBracket]#1\[LeftDoubleBracket]2\
\[RightDoubleBracket]\[RightDoubleBracket], 
                    REP] &\) /@ \[IndentingNewLine]\t\t\tCartesianProduct[n, 
                  p], p]\[IndentingNewLine]]\[IndentingNewLine]];\)\(\
\[IndentingNewLine]\)
    \)\), "\[IndentingNewLine]", 
    \(\(\(MatrixMultiply[A_?MatrixQ, B_?MatrixQ, CC___?MatrixQ] := 
        MatrixMultiply[MatrixMultiply[A, B], CC];\)\(\[IndentingNewLine]\)
    \)\), "\n", 
    \(\(\(Options[MatrixStar] = {Full \[Rule] True};\)\(\[IndentingNewLine]\)
    \)\), "\[IndentingNewLine]", 
    \(\(MatrixStar[A_?SquareMatrixQ, opts___Rule] := 
        Module[{n, m, n1, n2, A11, A12, A21, A22, A11s, A22s, B11, B12, B21, 
            B22, full}, \[IndentingNewLine]full = \(Full /. {opts}\) /. 
              Options[MatrixStar]; \[IndentingNewLine]{n, m} = 
            Dimensions[A]; \[IndentingNewLine]\[IndentingNewLine]If[n == 1, 
            Return[{{RES[A[\([1, 1]\)]]}}]]; \[IndentingNewLine]If[
            n \[Equal] 2, 
            Return[\[IndentingNewLine]RESimplify[{{RES[
                      REP[A\[LeftDoubleBracket]1, 1\[RightDoubleBracket], 
                        RET[A\[LeftDoubleBracket]1, 2\[RightDoubleBracket], 
                          RES[
                            A\[LeftDoubleBracket]2, 2\[RightDoubleBracket]], 
                          A\[LeftDoubleBracket]2, 1\[RightDoubleBracket]]]], 
                    RET[RES[
                        REP[A\[LeftDoubleBracket]1, 1\[RightDoubleBracket], 
                          RET[A\[LeftDoubleBracket]1, 2\[RightDoubleBracket], 
                            RES[A\[LeftDoubleBracket]2, 
                                2\[RightDoubleBracket]], 
                            A\[LeftDoubleBracket]2, 
                              1\[RightDoubleBracket]]]], 
                      A\[LeftDoubleBracket]1, 2\[RightDoubleBracket], 
                      RES[A\[LeftDoubleBracket]2, 
                          2\[RightDoubleBracket]]]}, {RET[
                      RES[REP[A\[LeftDoubleBracket]2, 2\[RightDoubleBracket], 
                          RET[A\[LeftDoubleBracket]2, 1\[RightDoubleBracket], 
                            RES[A\[LeftDoubleBracket]1, 
                                1\[RightDoubleBracket]], 
                            A\[LeftDoubleBracket]1, 
                              2\[RightDoubleBracket]]]], 
                      A\[LeftDoubleBracket]2, 1\[RightDoubleBracket], 
                      RES[A\[LeftDoubleBracket]1, 1\[RightDoubleBracket]]], 
                    RES[REP[A\[LeftDoubleBracket]2, 2\[RightDoubleBracket], 
                        RET[A\[LeftDoubleBracket]2, 1\[RightDoubleBracket], 
                          RES[A\[LeftDoubleBracket]1, 
                              1\[RightDoubleBracket]], 
                          A\[LeftDoubleBracket]1, 
                            2\[RightDoubleBracket]]]]}}]]]; \
\[IndentingNewLine]\[IndentingNewLine]n1 = 
            Quotient[n, 2]; \[IndentingNewLine]n2 = 
            n - n1; \[IndentingNewLine]{A11, A12, A21, 
              A22} = \[IndentingNewLine]{MatrixTake[A, n1, n1], 
              MatrixTake[A, n1, \(-n2\)], \[IndentingNewLine]MatrixTake[
                A, \(-n2\), n1], 
              MatrixTake[A, \(-n2\), \(-n2\)]}; \[IndentingNewLine]A11s = 
            MatrixStar[A11]; \[IndentingNewLine]A22s = 
            MatrixStar[A22]; \[IndentingNewLine]If[\ 
            full, \[IndentingNewLine]B11 = 
              MatrixStar[ToRegExp[A11 + MatrixMultiply[A12, A22s, A21]]]; 
            B12 = MatrixMultiply[B11, A12, A22s]; 
            B22 = MatrixStar[ToRegExp[A22 + MatrixMultiply[A21, A11s, A12]]]; 
            B21 = MatrixMultiply[B22, A21, A11s], \[IndentingNewLine] (*\ 
              else\ *) \[IndentingNewLine]B11 = 
              MatrixMultiply[
                MatrixStar[MatrixMultiply[A11s, A12, A22s, A21]], A11s]; 
            B12 = MatrixMultiply[B11, A12, A22s]; 
            B22 = MatrixMultiply[
                MatrixStar[MatrixMultiply[A22s, A21, A11s, A12]], A22s]; 
            B21 = MatrixMultiply[B22, A21, 
                A11s]\[IndentingNewLine]]; \[IndentingNewLine]RESimplify[
            AppendRows[AppendColumns[B11, B21], 
              AppendColumns[B12, B22]]]\[IndentingNewLine]];\)\)}], "Input",
  CellTags->"S5.48.1"]
}, Closed]],

Cell[CellGroupData[{

Cell["Some Tests", "Subsubsection",
  Evaluatable->False,
  AspectRatioFixed->True,
  CellTags->"c:5"],

Cell["\<\
Here are some tests of the arithmetic procedures for the RE matrix \
semiring.  Note that since the simplification procedure for regular \
expressions is quite primitive one cannot really deal with large \
matrices.\
\>", "Text"],

Cell[BoxData[
    \(Clear[a, b, c, d, e, f, g, h, i\ ]\)], "Input",
  AspectRatioFixed->True],

Cell[BoxData[
    \(\(MatrixStar[{{Empty, Empty}, {Empty, Empty}}]\  // \ 
        MatrixForm\)\ \  // \ PrintRE\)], "Input",
  AspectRatioFixed->True],

Cell[BoxData[
    \(\(MatrixStar[{{a, b}, {c, d}}]\ \  // \ MatrixForm\)\  // \ 
      PrintRE\)], "Input",
  AspectRatioFixed->True],

Cell[BoxData[
    \(\(MatrixStar[{{a, b}, {c, d}}, Normalize \[Rule] False] // \ 
        MatrixForm\)\  // \ PrintRE\)], "Input",
  AspectRatioFixed->True],

Cell[BoxData[
    \(\(MatrixStar[{{Empty, a}, {a, Empty}}] // \ MatrixForm\)\  // \ 
      PrintRE\)], "Input",
  AspectRatioFixed->True],

Cell[BoxData[
    \(\(MatrixStar[{{Empty, a, Empty}, {Empty, Empty, a}, {a, Empty, 
              b}}] // \ MatrixForm\)\  // \ PrintRE\)], "Input",
  AspectRatioFixed->True],

Cell["\<\
The implementation of the Kleene star operation has an optional \
second form.  The resulting regular expressions differ as terms but are \
equivalent.\
\>", "Text"],

Cell[BoxData[{
    \(\(mat = MatrixStar[{{a, b, c}, {d, e, f}, {g, h, i}}];\)\), "\n", 
    \(exp1 = mat\[LeftDoubleBracket]1, 1\[RightDoubleBracket]\)}], "Input",
  AspectRatioFixed->True],

Cell[BoxData[{
    \(\(mat = 
        MatrixStar[{{a, b, c}, {d, e, f}, {g, h, i}}, 
          Full \[Rule] False];\)\), "\n", 
    \(exp2 = mat\[LeftDoubleBracket]1, 1\[RightDoubleBracket]\)}], "Input",
  AspectRatioFixed->True],

Cell[BoxData[
    \(exp1\  === \ exp2\)], "Input"],

Cell[BoxData[{
    \(RegularExpressionToFA[exp1, 9]\), "\n", 
    \(RegularExpressionToFA[exp2, 9]\), "\n", 
    \(EquivalentQFA[%, %%]\)}], "Input",
  AspectRatioFixed->True]
}, Closed]],

Cell[CellGroupData[{

Cell["Klein'sche Vierergruppe", "Subsubsection",
  Evaluatable->False,
  AspectRatioFixed->True,
  CellTags->"c:6"],

Cell[TextData[{
  "For another example, let us convert a small group, say, the Klein'sche \
Vierergruppe (the symmetries of a rectangle, isomorphic to ",
  Cell[BoxData[
      \(TraditionalForm\`\[DoubleStruckCapitalZ]\_2\[Cross]\ \
\[DoubleStruckCapitalZ]\_2\)]],
  ") into a symbolic multiplication table.   All elements other that 1 have \
order 2 and thus are self-inverse."
}], "Text"],

Cell[BoxData[{
    \(S\  = \ 
      Sort@\(ToList@
          GenerateSG[
            GEN[4, 2, {T[2, 1, 3, 4], 
                T[1, 2, 4, 3]}]]\)\), "\[IndentingNewLine]", 
    \(S\  ** \ S\  // \ Union\)}], "Input",
  AspectRatioFixed->True],

Cell[TextData[{
  "One way to obtain the group table is via ",
  StyleBox["Outer", "MR"],
  ", using a list of elements and their inverses."
}], "Text"],

Cell[BoxData[{
    \(\(mat = Outer[NonCommutativeMultiply, S, S\ ];\)\), "\n", 
    \(mat\  // \ MatrixForm\), "\n", 
    \(\(alph = {e, a, b, c};\)\), "\n", 
    \(\(mtab = mat /. Thread[S \[Rule] alph];\)\), "\n", 
    \(mtab\  // \ MatrixForm\)}], "Input",
  AspectRatioFixed->True],

Cell["\<\
We compute the star of this symbolic matrix. Note the size of the \
expressions, there is no point in trying to print them out.\
\>", "Text"],

Cell[BoxData[{
    \(\(star = MatrixStar[mtab];\)\), "\[IndentingNewLine]", 
    \(star\  // \ ByteCount\)}], "Input",
  AspectRatioFixed->True],

Cell[BoxData[{
    \(Count[star, REP[__], {1, Infinity}]\), "\[IndentingNewLine]", 
    \(Count[star, RET[__], {1, Infinity}]\), "\[IndentingNewLine]", 
    \(Count[star, RES[__], {1, Infinity}]\)}], "Input"],

Cell[TextData[{
  "Extracting the expression in position ",
  Cell[BoxData[
      \(TraditionalForm\`\((1, 1)\)\)]],
  " should produce a machine accepting only words that simplify to 1 the \
group."
}], "Text"],

Cell[BoxData[{
    \(\(exp = star\[LeftDoubleBracket]1, 1\[RightDoubleBracket];\)\), "\n", 
    \(m\  = MinimizeFA@RegularExpressionToFA[exp, alph]\)}], "Input",
  AspectRatioFixed->True],

Cell["A small sanity check.", "Text"],

Cell[BoxData[{
    \(SyntacticHomomorphism[f, m]\), "\[IndentingNewLine]", 
    \(LanguageFA[\ m, \ 4\ ]\), "\[IndentingNewLine]", 
    \(f[%]\  // \ Union\)}], "Input"],

Cell["\<\
On the other hand, if we add the whole first row we should get a \
machine accepting all words.\
\>", "Text"],

Cell[BoxData[{
    \(\(exp = 
        REP @@ star\[LeftDoubleBracket]1\[RightDoubleBracket];\)\), "\
\[IndentingNewLine]", 
    \(RegularExpressionToFA[exp, alph]\  // \ FullQFA\)}], "Input",
  AspectRatioFixed->True],

Cell["Voila.", "Text"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Regularity Preserving Operations", "Section",
  CellTags->"c:1"],

Cell[TextData[{
  "Let ",
  Cell[BoxData[
      \(TraditionalForm\`L\)]],
  " be  some regular language.  We will construct machines for languages of \
the form ",
  Cell[BoxData[
      \(TraditionalForm\`P(L)\)]],
  " for some regularity preserving operations ",
  Cell[BoxData[
      \(TraditionalForm\`P\)]],
  ".\n\nNote: The structured state sets of the machines in this section are \
quite large, make sure to keep formatting turned on."
}], "Text"],

Cell[BoxData[
    \(FormatAutomata[Automatic]\)], "Input"],

Cell[CellGroupData[{

Cell["Length", "Subsection"],

Cell[TextData[{
  "Consider the operation\n\n\t",
  Cell[BoxData[
      \(TraditionalForm\`\(\(T(L, 
          f)\)\(\ \)\(=\)\(\ \)\({\ \ x\ \  | \ \ \[Exists] \ 
              y\ \((\ \ \(\(|\)\(y\)\(|\)\) = 
                      f(\(|\)\(x\)\(|\))\  \[And] \ x\[InvisibleComma]
                    y\  \[Element] \ L\ )\)\ }\)\(\ \)\)\)]],
  "\n\t\t",
  Cell[BoxData[
      \(TraditionalForm\`\(\(=\)\(\ \)\({\ 
          x\  | \ \(x\^\(-1\)\) 
                  L\  \[Intersection] \ \[CapitalSigma]\^\(\(\ \)\(f(\(|\)\(x\
\)\(|\))\)\)\  \[NotEqual] \ \[EmptySet]\ }\)\)\)]],
  "  \n\t\nwhere ",
  Cell[BoxData[
      \(TraditionalForm\`f\  : \ \[DoubleStruckCapitalN]\  \[Rule] \ \
\[DoubleStruckCapitalN]\)]],
  " is a suitable function. \n\nThe construction uses  a cyclic semimodule \
over  ",
  Cell[BoxData[
      \(TraditionalForm\`Q\ \[Cross]\ \[DoubleStruckCapitalB]\^\(\(\ \)\(n\ \
\[Cross]\ n\)\)\)]],
  "  where ",
  Cell[BoxData[
      \(TraditionalForm\`Q\)]],
  " is the state set of the given automaton for ",
  Cell[BoxData[
      \(TraditionalForm\`L\)]],
  " and ",
  Cell[BoxData[
      \(TraditionalForm\`n\)]],
  " its size.  The natural right action is of the form  \n\t",
  Cell[BoxData[
      \(TraditionalForm\`\((p, P)\)\ \[CenterDot]\ 
          a\  = \ \ \((p\[CenterDot]a, \ g(P, a)\ )\)\)]],
  " \nfor some function ",
  Cell[BoxData[
      \(TraditionalForm\`g\)]],
  ", depending on ",
  Cell[BoxData[
      \(TraditionalForm\`f\)]],
  ".  The generator is   ",
  Cell[BoxData[
      \(TraditionalForm\`\((q\_0, M)\)\)]],
  "  where ",
  Cell[BoxData[
      \(TraditionalForm\`M\)]],
  " is the full Boolean transition matrix of the automaton for ",
  Cell[BoxData[
      \(TraditionalForm\`L\)]],
  ".  This yields a DFA for ",
  Cell[BoxData[
      \(TraditionalForm\`T(L, f)\)]],
  " after selecting appropriate final states.\n\nAs an example consider the \
exponential function  ",
  Cell[BoxData[
      \(TraditionalForm\`f(i)\  = \ 2\^\(\(\ \)\(i\)\)\)]],
  ".\nHere  ",
  Cell[BoxData[
      \(TraditionalForm\`g(P, a)\  = \ P\^\(\(\ \)\(2\)\)\)]],
  "  and ",
  Cell[BoxData[
      \(TraditionalForm\`g\)]],
  " is thus independent of ",
  Cell[BoxData[
      \(TraditionalForm\`a\  \[Element] \ \[CapitalSigma]\)]],
  ". \nOn the other hand, for  ",
  Cell[BoxData[
      \(TraditionalForm\`f(i)\  = \ i\)]],
  "  we can use  ",
  Cell[BoxData[
      \(TraditionalForm\`g(P, a)\  = \ P\ M\)]],
  "."
}], "Text"],

Cell[CellGroupData[{

Cell["Example 1", "Subsubsection",
  CellTags->"c:3"],

Cell[TextData[{
  Cell[BoxData[
      FormBox[
        RowBox[{"L", " ", "=", " ", 
          
          RowBox[{\({\ \ x\  | \  | x\( | \_\ \) = \ 0, \ 1\ \ mod\ 4\ \ }\), 
            " ", "\[Intersection]", "  ", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"a", " ", 
                  RowBox[{"(", 
                    RowBox[{"b", "+", 
                      StyleBox["bbb",
                        FontSlant->"Italic"]}], ")"}]}], ")"}], "*"]}]}], 
        TraditionalForm]]],
  "\t\t \n",
  Cell[BoxData[
      \(TraditionalForm\`f(i)\  = \ 2\^\(\(\ \)\(i\)\)\)]],
  "\n\nThe minimal DFA for ",
  Cell[BoxData[
      \(TraditionalForm\`L\)]],
  " is has 9 states."
}], "Text"],

Cell[BoxData[
    \(m = 
      IntersectionFA[\ LengthDFA[\ \(-1\), \ 2, \ Modulus \[Rule] 4], \ 
          RegularExpressionToFA[RES[RET[a, REP[b, bbb]]], 2]\ ]\  // \ 
        MinimizeFA\)], "Input"],

Cell[BoxData[
    \(m = 
      IntersectionFA[\ CounterDFA[\ a, 1, \ 2, \ Modulus \[Rule] 5], \ 
          RegularExpressionToFA[RES[RET[a, REP[b, bbb]]], 2]\ ]\  // \ 
        MinimizeFA\)], "Input"],

Cell[TextData[{
  "The Boolean transition matrix of ",
  Cell[BoxData[
      \(TraditionalForm\`m\)]],
  " is used in the construction of the semimodule, together with the ordinary \
transition function. "
}], "Text"],

Cell[BoxData[{
    \(Clear[n, F, dot, final, Q, W, mm]\), "\n", 
    \(\(n = Size[m];\)\), "\n", 
    \(TransitionFunctionFA[m, delta]\), "\n", 
    \(\(F = ToBitVector[Final[m], Range[n]];\)\), "\n", 
    \(\(M = 
        FullTransitionMatrixFA[m, Type \[Rule] Boolean, 
          Full \[Rule] True]\ ;\)\)}], "Input"],

Cell[TextData[{
  "The actual construction of the new machine uses ",
  ButtonBox["GenerateDFA",
    ButtonStyle->"AddOnsLink"],
  ",  a convenient wrapper for ",
  ButtonBox["GenerateCSM",
    ButtonStyle->"AddOnsLink"],
  ".  We define a right action on   ",
  Cell[BoxData[
      \(TraditionalForm\`Q\ \[Cross]\ \[DoubleStruckCapitalB]\^\(\(\ \)\(n\ \
\[Cross]\ n\)\)\)]],
  "  and a helper function that selects the accepting states in the new \
machine."
}], "Text"],

Cell[BoxData[{
    \(dot[{p_, P_}, s_] := {delta[p, s], BooleanComposition[P, P]}\), "\n", 
    \(\(\(final[{p_, P_}] := ToBitVector[p, Range[n]] . P . F > 0;\)\(\n\)
    \)\), "\[IndentingNewLine]", 
    \(\({mm, W} = 
        GenerateDFA[{{1}, M}, dot, 2, final, Full \[Rule] True, 
          Normalize \[Rule] 1];\)\), "\[IndentingNewLine]", 
    \(mm\  // \ Size\)}], "Input"],

Cell["There are 32 states in all, but only 6 Boolean matrices.", "Text"],

Cell[BoxData[{
    \(\(\(PlotMatrix[#, DisplayFunction \[Rule] Identity] &\) /@ \ 
        Union[Last /@ States[mm]];\)\), "\n", 
    \(\(ShowArray[%];\)\)}], "Input"],

Cell["The minimal DFA is much smaller.", "Text"],

Cell[BoxData[
    \(mmm\  = \ MinimizeFA[mm]\)], "Input"],

Cell[TextData[{
  "A few words in ",
  Cell[BoxData[
      \(TraditionalForm\`T(L, f)\)]],
  "."
}], "Text"],

Cell[BoxData[
    \(LanguageFA[mmm, \(-10\)]\)], "Input"],

Cell[TextData[{
  "A sample quotient   ",
  Cell[BoxData[
      \(TraditionalForm\`\(x\^\(-1\)\) 
          L\  \[Intersection] \ \[CapitalSigma]\^\(\(\ \)\(f(\(|\)\(x\)\(|\))\
\)\)\)]],
  "  for  ",
  Cell[BoxData[
      FormBox[
        RowBox[{"x", " ", "=", " ", 
          StyleBox["abab",
            FontSlant->"Italic"]}], TraditionalForm]]],
  ". "
}], "Text"],

Cell[BoxData[
    \(LanguageFA[\ SetInitialFA[m, delta[1, abab]], \ 16\ ]\)], "Input"],

Cell[TextData[{
  "The growth function of  ",
  Cell[BoxData[
      \(TraditionalForm\`T(L, f)\)]],
  "  involves Fibonacci numbers. It is not hard to give a complete \
description of the language based on this observation."
}], "Text"],

Cell[BoxData[
    \(\(\(LanguageFA[\ mmm, \ \(-20\), \ 
      SizeOnly \[Rule] True\ ]\)\(\ \)\)\)], "Input"],

Cell["We verify the structure of the growth function.", "Text"],

Cell[BoxData[{
    \(Clear[x]\), "\[IndentingNewLine]", 
    \(ser\  = \ GrowthSeriesDFA[mmm, x]\), "\[IndentingNewLine]", 
    \(Series[\ ser, \ {x, \ 0, \ 12}]\)}], "Input"],

Cell[BoxData[
    \(Sum[\ Fibonacci[n + 1]\ x^\((2  n)\), \ {n, 2, 6}]\)], "Input"],

Cell[BoxData[
    \(Sum[\ Fibonacci[n + 1]\ x^\((2  n)\), \ {n, 2, Infinity}\ ]\ \  // \ 
      FullSimplify\)], "Input"],

Cell["Voila.", "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["Example 2", "Subsubsection",
  CellTags->"c:4"],

Cell[TextData[{
  Cell[BoxData[
      \(TraditionalForm\`L\  = \ {\ \ x\  | \ \  | 
            x\( | \_\(\(\ \)\(a\)\)\), \(\(|\)\(x\)\( | \_\(\(\ \)\(b\)\)\)\) \
= \ 0\ \ mod\ \ 4\ \ \ }\)]],
  "\t\t  ",
  Cell[BoxData[
      \(TraditionalForm\`f(i)\  = \ 2\^\(\(\ \)\(i\)\)\)]],
  "\n\nThe minimal DFA for ",
  Cell[BoxData[
      \(TraditionalForm\`L\)]],
  " has size 16."
}], "Text"],

Cell[BoxData[
    \(m = \ 
      IntersectionFA[\ 
        CounterDFA[\ a, 0, \ 2, \ 
          Modulus \[Rule] 4\ ], \[IndentingNewLine]CounterDFA[\ b, 0, \ 2, \ 
          Modulus \[Rule] 4\ ]]\)], "Input"],

Cell[BoxData[{
    \(Clear[n, F, dot, final, Q, W, mm, g]\), "\n", 
    \(\(n = Size[m];\)\), "\n", 
    \(\(F = ToBitVector[Final[m], Range[n]];\)\), "\n", 
    \(\(M = FullTransitionMatrixFA[m, Full \[Rule] True];\)\)}], "Input"],

Cell[BoxData[{
    \(TransitionFunctionFA[m, delta]\), "\n", 
    \(dot[{p_, P_}, s_] := {delta[p, s], BooleanComposition[P, P]}\), "\n", 
    \(\(final[{p_, P_}] := ToBitVector[p, Range[n]] . P . F > 0;\)\), "\n", 
    \(mm = 
      GenerateDFA[{{1}, M}, dot, 2, final, Normalize \[Rule] 1]\)}], "Input"],

Cell["There are 3 associated Boolean matrices.", "Text"],

Cell[BoxData[{
    \(\(\(PlotMatrix[#, DisplayFunction \[Rule] Identity, 
            GridLines \[Rule] True] &\) /@ \ 
        Union[Last /@ States[mm]];\)\), "\n", 
    \(\(ShowArray[%, 3];\)\)}], "Input"],

Cell["We get all words of length a positive multiple of 4.", "Text"],

Cell[BoxData[
    \(LanguageFA[mm, \(-20\), SizeOnly \[Rule] True]\)], "Input"],

Cell[BoxData[
    \(LanguageFA[mm, 4\ ]\)], "Input"],

Cell[BoxData[{
    \(TransitionFunctionFA[mm, g]\), "\[IndentingNewLine]", 
    \(\(m0\  = \ SetInitialFA[mm, g[1, aaab]];\)\), "\[IndentingNewLine]", 
    \(LanguageFA[m0, \(-20\), \ SizeOnly \[Rule] True]\)}], "Input"],

Cell[TextData[{
  "Verify the structure of  ",
  Cell[BoxData[
      \(TraditionalForm\`T(L, f)\)]],
  "."
}], "Text"],

Cell[BoxData[
    \(EquivalentQFA[\ 
      mm, \ \[IndentingNewLine]ComplementFA[\ 
        LengthDFA[0, 2, Modulus \[Rule] 4], 
        BuildDFA[\[Epsilon], 2]]]\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["Example 3", "Subsubsection",
  CellTags->"c:4"],

Cell[TextData[{
  "A single letter alphabet. \n",
  Cell[BoxData[
      \(TraditionalForm\`L\  = \ {\ \ x\  \[Element] \ 
                a\^\(\(\ \)\(*\)\)\  | \ \  | x | \  = \ 0\ mod\ 7\ \ }\)]]
}], "Text"],

Cell[BoxData[
    \(m = LengthDFA[\ 0, \ 1, \ Modulus \[Rule] 7\ ]\)], "Input"],

Cell[BoxData[{
    \(Clear[n, F, dot, final, Q, W, mm]\), "\n", 
    \(\(n = Size[m];\)\), "\n", 
    \(\(F = ToBitVector[Final[m], Range[n]];\)\)}], "Input"],

Cell["The transition matrix is a circulant matrix.", "Text"],

Cell[BoxData[{
    \(TransitionMatrixFA[m, B, Type \[Rule] Boolean, 
      Full \[Rule] True]\), "\n", 
    \(\(M = B[a];\)\), "\[IndentingNewLine]", 
    \(M\  // \ MatrixForm\)}], "Input"],

Cell[BoxData[{
    \(\(TransitionFunctionFA[m, delta];\)\), "\n", 
    \(dot[{p_, P_}, s_] := {delta[p, s], BooleanComposition[P, P]}\), "\n", 
    \(\(final[{p_, P_}] := ToBitVector[p, Range[n]] . P . F > 0;\)\), "\n", 
    \(mm = GenerateDFA[{{1}, M}, dot, 1, final]\)}], "Input"],

Cell[TextData[{
  "The growth function is a strictly periodic sequence,  ",
  Cell[BoxData[
      \(TraditionalForm\`{6, 17, 19}\  + \ 21\ \[DoubleStruckCapitalN]\)]],
  "."
}], "Text"],

Cell[BoxData[{
    \(LanguageFA[mm, \(-40\), SizeOnly \[Rule] True]\), "\n", 
    \(Position[%, 1] - 1\)}], "Input"],

Cell[TextData[{
  "Correct, since we are essentially solving   ",
  Cell[BoxData[
      \(TraditionalForm\`x\  + \ 2\^\(\(\ \)\(x\)\) = \ 0\ \((mod\ 7)\)\)]],
  "  and 2 has order 3 in ",
  Cell[BoxData[
      \(TraditionalForm\`\[DoubleStruckCapitalZ]\_7\%\(\(\ \)\(*\)\)\)]],
  "."
}], "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["Example 4", "Subsubsection",
  CellTags->"c:4"],

Cell[TextData[{
  Cell[BoxData[
      \(TraditionalForm\`L\  = \ {\ \ x\  \[Element] \ 
                a\^\(\(\ \)\(*\)\)\  | \ \  | x | \  = \ 
            1\ \ mod\ 7\ \ }\)]],
  "\t\t  ",
  Cell[BoxData[
      \(TraditionalForm\`f(i)\  = \ i\)]]
}], "Text"],

Cell[BoxData[
    \(m = LengthDFA[\ {1, 3}, \ 1, \ Modulus \[Rule] 7\ ]\)], "Input"],

Cell[BoxData[{
    \(Clear[n, F, dot, final, Q, W, mm]\), "\n", 
    \(\(n = Size[m];\)\), "\n", 
    \(\(F = ToBitVector[Final[m], Range[n]];\)\), "\n", 
    \(TransitionMatrixFA[m, B, Type \[Rule] Boolean, 
      Full \[Rule] True]\), "\n", 
    \(\(M = B[a];\)\), "\n", 
    \(\(TransitionFunctionFA[m, delta];\)\), "\n", 
    \(dot[{p_, P_}, s_] := {delta[p, s], BooleanComposition[P, M]}\), "\n", 
    \(\(final[{p_, P_}] := ToBitVector[p, Range[n]] . P . F > 0;\)\), "\n", 
    \(mm = 
      GenerateDFA[{{1}, IdentityMatrix[n]}, dot, 1, final, 
        Normalize \[Rule] 1]\)}], "Input"],

Cell[TextData[{
  "The growth function is the strictly periodic sequence  ",
  Cell[BoxData[
      \(TraditionalForm\`{4, 5}\  + \ 7\ \[DoubleStruckCapitalN]\)]],
  "."
}], "Text"],

Cell[BoxData[{
    \(LanguageFA[mm, \(-40\), SizeOnly \[Rule] True]\), "\n", 
    \(Position[%, 1] - 1\)}], "Input"],

Cell[TextData[{
  "Correct, since we are solving   ",
  Cell[BoxData[
      \(TraditionalForm\`2\ x\  = \ 1, 3\ \ \((mod\ 7)\)\)]],
  "."
}], "Text"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Square Root", "Subsection",
  CellTags->"c:5"],

Cell[TextData[{
  Cell[BoxData[
      \(TraditionalForm\`\@L\  = \ {\ 
          w\  | \ w\[InvisibleComma]w\  \[Element] \ L\ }\)]],
  "\n\nHere the action is given by  ",
  Cell[BoxData[
      \(TraditionalForm\`g(P, a)\  = \ P\ B\_a\)]],
  "  where ",
  Cell[BoxData[
      \(TraditionalForm\`B\_a\)]],
  " is the transition matrix associated with ",
  Cell[BoxData[
      \(TraditionalForm\`a\  \[Element] \ \[CapitalSigma]\)]],
  ".  \n\nExample:  ",
  Cell[BoxData[
      \(TraditionalForm\`L\  = \ {\ \ x\  | \ \  | 
                x\( | \_\(\(\ \)\(a\)\)\) = \(\(\(1\ \ mod\ \ 5\)\(\ \)\(\
\[And]\)\)\  | x\( | \_\(\(\ \)\(b\)\)\) = 2\), 
            3\ \ mod\ \ 4\ }\  \[Intersection] \ \(\((a\ \
\(b\^+\))\)\^*\)\)]],
  "."
}], "Text"],

Cell[BoxData[
    \(m = 
      IntersectionFA[\ CounterDFA[a, 1, 2, Modulus \[Rule] \ 5], 
          CounterDFA[b, {2, 3}, 2, 
            Modulus \[Rule] \ 4], \[IndentingNewLine]RegularExpressionToFA[\ 
            RES[RET[ab, RES[b]]], 2]\ ]\  // \ MinimizeFA\)], "Input"],

Cell["The requisite transition functions/matrices.", "Text"],

Cell[BoxData[{
    \(Clear[n, F, dot, B, final, Q, W, mm]\), "\n", 
    \(\(n = Size[m];\)\), "\n", 
    \(\(F = ToBitVector[Final[m], Range[n]];\)\), "\n", 
    \(TransitionMatrixFA[m, B, Type \[Rule] Boolean, 
      Full \[Rule] True]\), "\n", 
    \(TransitionFunctionFA[m, delta]\)}], "Input"],

Cell["The right action for the semimodule.", "Text"],

Cell[BoxData[{
    \(dot[{p_, P_}, s_] := {delta[p, s], 
        BooleanComposition[P, B[s]]}\), "\n", 
    \(\(final[{p_, P_}] := ToBitVector[p, Range[n]] . P . F > 0;\)\), "\n", 
    \(mm = 
      GenerateDFA[{{1}, IdentityMatrix[n]}, dot, 2, final, 
        Normalize \[Rule] 1]\)}], "Input"],

Cell["In this case, the Boolean matrices are all distinct.", "Text"],

Cell[BoxData[{
    \(\(\(PlotMatrix[#, DisplayFunction \[Rule] Identity] &\) /@ \ 
        Take[Union[Last /@ States[mm]], 16];\)\), "\n", 
    \(\(ShowArray[%];\)\)}], "Input"],

Cell["The minimal DFA is considerably smaller.", "Text"],

Cell[BoxData[
    \(mmm\  = \ MinimizeFA[mm]\)], "Input"],

Cell[TextData[{
  "The growth function of ",
  Cell[BoxData[
      \(TraditionalForm\`\@L\)]],
  " and some words."
}], "Text"],

Cell[BoxData[
    \(LanguageFA[mmm, \(-14\), \ SizeOnly \[Rule] True\ ]\)], "Input"],

Cell[BoxData[
    \(LanguageFA[mmm, \(-8\)\ ]\)], "Input"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Unpalindromes", "Section",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  "Consider the language of unpalindromes of length ",
  Cell[BoxData[
      \(TraditionalForm\`2  n\)]],
  ":\n\n\t",
  Cell[BoxData[
      \(TraditionalForm\`L\_\(\(\ \)\(n\)\)\  = \ \ {\ 
            u\[VeryThinSpace]v\  | \ \  | \(\(\(u\)\(|\)\) = \
\(\(\(|\)\(v\)\(|\)\)\  = \ 
                  n\ \  \[And] \ \ u \[NotEqual] 
                      v\)\)\ \ }\  \[SubsetEqual] \ \({a, b}\^*\)\)]],
  ".\n\t\n",
  StyleBox["Claim",
    FontWeight->"Bold"],
  ":  The minimal DFA for  ",
  Cell[BoxData[
      \(TraditionalForm\`L\_\(\(\ \)\(n\)\)\)]],
  "  has  ",
  Cell[BoxData[
      \(TraditionalForm\`3\[CenterDot]2\^n + \ n\  - \ 2\)]],
  "  states. \n\nFirst a brute-force construction of the minimal DFA via the \
quotient automaton, see ",
  ButtonBox["QuotientDFA",
    ButtonStyle->"AddOnsLink"],
  ".  Note that the cardinality of ",
  Cell[BoxData[
      \(TraditionalForm\`L\_\(\(\ \)\(n\)\)\)]],
  " is ",
  Cell[BoxData[
      \(TraditionalForm\`2\^\(\(\ \)\(2  n\)\)\  - \ 2\^\(\(\ \)\(n\)\)\)]],
  ", so we have to constrain our experiments to reasonably small values of ",
  Cell[BoxData[
      \(TraditionalForm\`n\)]],
  ". "
}], "Text"],

Cell[BoxData[
    \(\(unpalindrome[n_Integer] := 
        With[{L = Words[n]}, 
          QuotientDFA[
            StringJoin /@ DeleteCases[CartesianProduct[L, L], {x_, x_}], {a, 
              b}, Normalize \[Rule] 1]\[IndentingNewLine]];\)\)], "Input",
  AspectRatioFixed->True],

Cell[TextData[{
  "We verify that the claim holds for ",
  Cell[BoxData[
      \(TraditionalForm\`n\  \[LessEqual] \ 6\)]],
  "."
}], "Text"],

Cell[BoxData[{
    \(Table[\ Size[unpalindrome[n]], \ {n, 6}\ ]\), "\[IndentingNewLine]", 
    \(Table[\ 3\ 2^n\  + \ n\  - 2, \ {n, 6}]\)}], "Input",
  AspectRatioFixed->True],

Cell[TextData[{
  "Here is a closer look at ",
  Cell[BoxData[
      \(TraditionalForm\`n\  = \ 3\)]],
  ".   It is convenient to remove the sink."
}], "Text"],

Cell[BoxData[{
    \(\(m = unpalindrome[3];\)\), "\n", 
    \(TransitionFunctionFA[m, \[Delta]]\), "\[IndentingNewLine]", 
    \(\(mm\  = \ 
        DeleteStateFA[m, TrapStatesFA[m], Normalize \[Rule] 2];\)\), "\n", 
    \(\(Q\  = \ States[mm];\)\)}], "Input",
  AspectRatioFixed->True],

Cell["Here are some of the smaller left quotients.", "Text"],

Cell[BoxData[
    \(Take[\ Q, \ \(-20\)\ ]\  // \ ColumnForm\)], "Input",
  AspectRatioFixed->True],

Cell[BoxData[{
    \(Length\  /@ \ Q\), "\n", 
    \(\(%\  // \ Frequencies\)\  // \ TableForm\)}], "Input"],

Cell[TextData[{
  "Clearly, the states in the quotient automaton for ",
  Cell[BoxData[
      \(TraditionalForm\`L\_n\)]],
  " are organized in layers where states at layer ",
  Cell[BoxData[
      \(TraditionalForm\`k\)]],
  " are of the form ",
  Cell[BoxData[
      \(TraditionalForm\`P\  \[SubsetEqual] \ \[CapitalSigma]\^\(\(\ \)\(2  n \
- k\)\)\)]],
  " and transitions lead from layer ",
  Cell[BoxData[
      \(TraditionalForm\`k\)]],
  " to layer ",
  Cell[BoxData[
      \(TraditionalForm\`k + 1\)]],
  "(except for the sink).  In fact,  the first ",
  Cell[BoxData[
      \(TraditionalForm\`n\)]],
  " layers form full binary tree since the behaviors of these states are \
necessarily distinct.  For example:"
}], "Text"],

Cell[BoxData[{
    \(\(BehaviorFA[m, \[Delta][1, #], 3] &\)\  /@ \ 
      Words[3]\ \), "\[IndentingNewLine]", 
    \(\(%\  // \ Union\)\  // \ Length\)}], "Input"],

Cell[TextData[{
  "In general, there is a leaf ",
  Cell[BoxData[
      FormBox[
        StyleBox["x",
          FontSlant->"Italic"], TraditionalForm]]],
  "  corresponding to ",
  Cell[BoxData[
      FormBox[
        RowBox[{\(\[CapitalSigma]\^n\), "-", 
          RowBox[{"{", 
            StyleBox["x",
              FontSlant->"Italic"], "}"}]}], TraditionalForm]]],
  " for each word ",
  Cell[BoxData[
      \(TraditionalForm\`x\  \[Element] \ \[CapitalSigma]\^\(\(\ \
\)\(n\)\)\)]],
  ".   So the first tree accounts for ",
  Cell[BoxData[
      \(TraditionalForm\`2\^\(n + 1\) - 1\)]],
  " states.  The remainder of the machine has two kinds of states: those \
where a witness for inequality of ",
  Cell[BoxData[
      \(TraditionalForm\`u\)]],
  " and ",
  Cell[BoxData[
      \(TraditionalForm\`v\)]],
  " has already been found, and those where we are still waiting for such a \
witness."
}], "Text"],

Cell[BoxData[{
    \(\(BehaviorFA[m, \[Delta][1, #], 2] &\)\  /@ \ 
      Words[4]\ \), "\[IndentingNewLine]", 
    \(%\  // \ Union\)}], "Input"],

Cell[BoxData[
    \(Select[\ Words[4], \ 
      Length[BehaviorFA[m, \[Delta][1, #], 2]] \[Equal] 4 &]\)], "Input"],

Cell[BoxData[
    \(Select[\ Words[4], \ 
      BehaviorFA[m, \[Delta][1, #], 
            2] \[Equal] {"\<ab\>", "\<ba\>", "\<bb\>"} &]\)], "Input"],

Cell[TextData[{
  "More precisely, the first type is of the form  ",
  Cell[BoxData[
      \(TraditionalForm\`\[Delta](q\_0, u\[VeryThinSpace]x)\)]],
  "  where ",
  Cell[BoxData[
      \(TraditionalForm\`x\)]],
  " is not a prefix of ",
  Cell[BoxData[
      \(TraditionalForm\`u\  \[Element] \ \[CapitalSigma]\^\(\(\ \
\)\(n\)\)\)]],
  " has behavior ",
  Cell[BoxData[
      \(TraditionalForm\`\[CapitalSigma]\^\(\(\(2  n\)\(-\)\) | u\
\[VeryThinSpace]x | \)\)]],
  " and the second type is of the form  ",
  Cell[BoxData[
      \(TraditionalForm\`\[Delta](q\_0, u\[VeryThinSpace]x)\)]],
  "  where ",
  Cell[BoxData[
      \(TraditionalForm\`x\)]],
  " is a prefix of ",
  Cell[BoxData[
      \(TraditionalForm\`u\  \[Element] \ \[CapitalSigma]\^\(\(\ \
\)\(n\)\)\)]],
  " and has behavior ",
  Cell[BoxData[
      \(TraditionalForm\`\[CapitalSigma]\^\(\(\(2  n\)\(-\)\) | u\
\[VeryThinSpace]x | \) - {y}\)]],
  "  where ",
  Cell[BoxData[
      \(TraditionalForm\`u\  = \ x\[VeryThinSpace]y\)]],
  ". But then there are ",
  Cell[BoxData[
      \(TraditionalForm\`n\)]],
  " states of the first type, and ",
  Cell[BoxData[
      \(TraditionalForm\`2\^n - 1\)]],
  " states of the second type (these form another tree which is upside-down \
compared to the first, and has as root the sink of the DFA)."
}], "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["Shuffle", "Section",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  "We construct a machine for   ",
  Cell[BoxData[
      \(TraditionalForm\`{\ 
        x\  | \ \ x\  || \ 
            u\ \  \[Intersection] \ L\ \  \[NotEqual] \ \[EmptySet]\ \ }\)]],
  "  where ",
  Cell[BoxData[
      \(TraditionalForm\`u\)]],
  " is fixed, ",
  Cell[BoxData[
      \(TraditionalForm\`L\)]],
  " regular given by a DFA.\n\nThe machine uses a state set of  ",
  Cell[BoxData[
      \(TraditionalForm\`Q\ \[Cross]\ \([0, m]\)\)]],
  "  where  ",
  Cell[BoxData[
      \(TraditionalForm\`m\)]],
  "  is the length of ",
  Cell[BoxData[
      \(TraditionalForm\`u\)]],
  ".  First,  a walk through the construction. We generate a suitable sample \
machine ",
  Cell[BoxData[
      \(TraditionalForm\`M\)]],
  " for the language  ",
  Cell[BoxData[
      \(TraditionalForm\`L\  = \ \(a\^*\) b\ \(a\^*\) 
          b\ \(c\^+\) \(a\^*\)\)]],
  ". "
}], "Text",
  CellMargins->{{Inherited, 53}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[BoxData[
    \(M\  = \ 
      RegularExpressionToFA[\ RET[\ RES[a], b, RES[a], b, RES[c], c, RES[a]], 
          3]\  // \ MinimizeFA\)], "Input"],

Cell[TextData[{
  "Extract the alphabet of ",
  Cell[BoxData[
      \(TraditionalForm\`M\)]],
  ", its transition function and the initial state."
}], "Text"],

Cell[BoxData[{
    \({k, alph}\  = \ Alphabet[M, Full \[Rule] True]\), "\n", 
    \(TransitionFunctionFA[M, delta]\), "\n", 
    \(q0\  = \ TheInitial[M]\)}], "Input"],

Cell[TextData[{
  "Unpack ",
  Cell[BoxData[
      \(TraditionalForm\`u\)]],
  " into a list of characters and record its length."
}], "Text"],

Cell[BoxData[{
    \(uchar\  = \ ToCharacters[bc]\), "\[IndentingNewLine]", 
    \(lenu\  = \ Length[uchar]\)}], "Input"],

Cell[TextData[{
  "Now for the functions used to generate the new machine.  ",
  StyleBox["next0", "MR"],
  "  is for \[Epsilon]-transitions\n\t",
  Cell[BoxData[
      \(TraditionalForm\`\((p, 
          i)\)\  \[Rule] \ \((p\ \[CenterDot]\ u\_\(i + 1\), \ i + 1)\)\)]],
  "\nand the functional next takes care of transitions\n\t",
  Cell[BoxData[
      \(TraditionalForm\`\((p, i)\)\  \[Rule] \ \((p\[CenterDot]a, i)\)\)]],
  "."
}], "Text"],

Cell[BoxData[{
    \(ClearAll[next, next0]\), "\[IndentingNewLine]", 
    \(\(next0[{p_, i_}]\  := \ \[IndentingNewLine]With[\ {ii\  = \ i + 1}, 
          If[\ ii\  \[LessEqual] \ 
              lenu\ , \ {\ {delta[p, uchar[\([ii]\)]], 
                ii}\ }, \ {}\ ]]\ ;\)\), "\[IndentingNewLine]", 
    \(\(\(next[s_]\)[{p_, i_}]\  := \ {{delta[p, s], i}};\)\)}], "Input"],

Cell["\<\
Now we build the state set and the transition relation, select the \
final states and assemble the machine.\
\>", "Text"],

Cell[BoxData[
    \({Q, tr}\  = \ 
      Closure[\ {{q0, 0}}, \ 
        Join[\ {next0}, \ 
          next\  /@ \ alph], \ \[IndentingNewLine]SetClosure \[Rule] True, \ 
        DoTrace \[Rule] True, \ Index \[Rule] 0\ \ ]\)], "Input"],

Cell[BoxData[
    \(fin\  = \ 
      PositionList[\ Q, \ 
        Select[\ Q, \ 
          MemberQ[Final[M], First[#]] && \ 
              Last[#] \[Equal] lenu &\ ]\ ]\)], "Input"],

Cell[BoxData[
    \(fa\  = \ 
      FA[\ Length[Q], \ k, \ tr, \ {q0}, \ fin, \ alph, \ Q\ ]\)], "Input"],

Cell[BoxData[
    \(LanguageFA[\ fa, \ \(-6\)\ ]\)], "Input"],

Cell["\<\
The whole operation can be packaged into a single function \
thus.\
\>", "Text"],

Cell[BoxData[
    \(shuffleWordFA[\ M_DFA, \ 
        u_?WordQ\ ]\  := \ \[IndentingNewLine]Module[\ {k, alph, q0, delta, 
          uchar, ulen, Q, tr, next0, next, 
          fin}, \[IndentingNewLine]\[IndentingNewLine]\t{k, alph}\  = \ 
          Alphabet[M, Full \[Rule] True]; \[IndentingNewLine]\t
        TransitionFunctionFA[M, delta]; \[IndentingNewLine]\t
        q0\  = \ TheInitial[M]; \[IndentingNewLine]\t
        uchar\  = \ ToCharacters[u]; \[IndentingNewLine]\t
        lenu\  = \ Length[uchar]; \[IndentingNewLine]\[IndentingNewLine]\t
        next0[{p_, 
              i_}]\  := \[IndentingNewLine]\t\tWith[\ {ii\  = \ 
                i + 1}, \[IndentingNewLine]\t\tIf[\ 
              ii\  \[LessEqual] \ 
                lenu\ , \ {\ {delta[p, uchar[\([ii]\)]], 
                  ii}\ }, \ {}\ ]]\ ; \[IndentingNewLine]\t\(next[s_]\)[{p_, 
              i_}]\  := \ {{delta[p, s], 
              i}}; \[IndentingNewLine]\[IndentingNewLine]\t{Q, tr}\  = \ 
          Closure[\ {{q0, 0}}, \ 
            Join[\ {next0}, \ 
              next\  /@ \ alph], \[IndentingNewLine]\t\tSetClosure \[Rule] 
              True, \ DoTrace \[Rule] True, 
            Index \[Rule] 0\ \ ]; \[IndentingNewLine]\[IndentingNewLine]\t
        fin\  = \ 
          PositionList[\ 
            Q, \ \[IndentingNewLine]\t\tSelect[\ Q, \ 
              MemberQ[Final[M], First[#]] && \ 
                  Last[#] \[Equal] 
                    lenu &\ ]\ ]; \[IndentingNewLine]\[IndentingNewLine]\t
        FA[\ Length[Q], \ k, \ tr, \ {q0}, \ fin, \ alph, \ 
          Q\ ]\[IndentingNewLine]]\)], "Input"],

Cell[TextData[{
  "Some more examples.   First ",
  Cell[BoxData[
      FormBox[
        RowBox[{"u", " ", "=", " ", 
          StyleBox["bb",
            FontSlant->"Italic"]}], TraditionalForm]]],
  "."
}], "Text"],

Cell[BoxData[
    \(m\  = \ shuffleWordFA[\ M, \ bb\ ]\  // \ MinimizeFA\)], "Input"],

Cell[BoxData[
    \(LanguageFA[\ m, \ \(-5\)\ ]\)], "Input"],

Cell["\<\
We can verify the construction by computing some shuffles \"by hand\
\".  None of the intersections below should be empty.\
\>", "Text"],

Cell[BoxData[{
    \(\(L7\  = \ LanguageFA[M, 7];\)\), "\n", 
    \(\(LL\  = \ LanguageFA[\ m, \ 5\ ];\)\), "\n", 
    \(\(Intersection[\ L7, \ WordShuffle[\ bb, \ #\ ]] &\)\  /@ \ 
      LL\)}], "Input"],

Cell[TextData[{
  "Now ",
  Cell[BoxData[
      FormBox[
        RowBox[{"u", " ", "=", " ", 
          StyleBox["cc",
            FontSlant->"Italic"]}], TraditionalForm]]],
  "."
}], "Text"],

Cell[BoxData[
    \(m\  = \ shuffleWordFA[\ M, \ cc\ ]\  // \ MinimizeFA\)], "Input"],

Cell[BoxData[
    \(LanguageFA[\ m, \ \(-5\)\ ]\)], "Input"],

Cell[TextData[{
  "But ",
  Cell[BoxData[
      FormBox[
        RowBox[{"u", " ", "=", " ", 
          StyleBox["cb",
            FontSlant->"Italic"]}], TraditionalForm]]],
  " produces a trivial automaton."
}], "Text"],

Cell[BoxData[
    \(shuffleWordFA[\ M, \ cb\ ]\  // \ EmptyQFA\)], "Input"],

Cell[TextData[{
  "It's a good exercise to build a machine for   ",
  Cell[BoxData[
      \(TraditionalForm\`{\ 
        x\  | \ \ x\  || \ u\ \  \[SubsetEqual] \ L\ \ }\)]],
  "  where ",
  Cell[BoxData[
      \(TraditionalForm\`u\)]],
  " is fixed, ",
  Cell[BoxData[
      \(TraditionalForm\`L\)]],
  " regular given by a DFA."
}], "Text",
  CellMargins->{{Inherited, 53}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True]
}, Closed]],

Cell[CellGroupData[{

Cell["Hopcroft & Ullman ", "Section",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["Problem 2.8 b", "Subsubsection"],

Cell[TextData[{
  "Problem 2.8 b  in the classical automata theory text by Hopcroft and \
Ullman uses a 3-point set ",
  Cell[BoxData[
      \(TraditionalForm\`\[CapitalSigma]\  = \ {a, b, c}\)]],
  "  and a non-associative operation on this set described by the following \
table. "
}], "Text"],

Cell[BoxData[{
    \(Clear[a, b, c]\), "\n", 
    \(\(mt\  = \ {{a, a, c}, {c, a, b}, {b, c, a}};\)\), "\n", 
    \(mt\  // \ MatrixForm\)}], "Input"],

Cell[TextData[{
  "One has to show that the collection ",
  Cell[BoxData[
      \(TraditionalForm\`L\)]],
  " of all words over ",
  Cell[BoxData[
      \(TraditionalForm\`\[CapitalSigma]\)]],
  "  that evaluate to the same element of \[CapitalSigma] under both left- \
and right-associative multiplication form a regular language.  "
}], "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["A Solution", "Subsubsection"],

Cell[TextData[{
  "We will construct the minimal DFA for ",
  Cell[BoxData[
      \(TraditionalForm\`L\)]],
  ".   First, we implement left and right multiplication according to the \
given table. For convenience, the operations are defined over ",
  Cell[BoxData[
      \(TraditionalForm\`\[CapitalSigma]\^\(\(\ \)\(+\)\)\)]],
  ". "
}], "Text"],

Cell[BoxData[{
    \(Clear[\[Lambda], \[Rho], mti]\), "\n", 
    \(\(mti\  = \ FlattenOne /@ ToIndex[mt, 3];\)\), "\n", 
    \(SetAttributes[\[Lambda], Listable]\), "\n", 
    \(SetAttributes[\[Rho], Listable]\), "\n", 
    \(\(\[Lambda][i_Integer]\  := \ i;\)\), "\n", 
    \(\(\[Lambda][i_Integer, j_Integer]\  := \ mti[\([i, j]\)];\)\), "\n", 
    \(\(\[Lambda][i_Integer, j_Integer, 
          k___Integer]\  := \ \[Lambda][\ \[Lambda][i, j], k];\)\), "\n", 
    \(\(\[Rho][i_Integer]\  := \ i;\)\), "\n", 
    \(\(\[Rho][i_Integer, j_Integer]\  := \ mti[\([i, j]\)];\)\), "\n", 
    \(\(\[Rho][k___Integer, i_Integer, j_Integer]\  := \ \[Rho][\ 
          k, \[Rho][i, j]];\)\), "\n", 
    \(\(\[Lambda][x_?WordQ]\  := \ 
        ToWord[\[Lambda] @@ ToIndex[x, 3], 3];\)\), "\n", 
    \(\(\[Rho][x_?WordQ]\  := \ 
        ToWord[\[Rho] @@ ToIndex[x, 3], 3];\)\)}], "Input"],

Cell["For words of length 2 there should be no difference.", "Text"],

Cell[BoxData[{
    \(\[Lambda][\ Words[2, 3]]\), "\[IndentingNewLine]", 
    \(\[Rho][\ Words[2, 3]]\)}], "Input"],

Cell["\<\
But for longer words we may get different answers depending on \
grouping.\
\>", "Text"],

Cell[BoxData[
    \(Select[\ Words[3, 3], \ \[Lambda][#] \[Equal] \[Rho][#] &\ ]\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["The Component Machines", "Subsubsection"],

Cell[TextData[{
  "To construct the automaton we first build machines  mr[s]  that accept all \
strings in  ",
  Cell[BoxData[
      \(TraditionalForm\`\[CapitalSigma]\^\(\(\ \)\(+\)\)\)]],
  " that evaluate to ",
  Cell[BoxData[
      \(TraditionalForm\`s\)]],
  " under ",
  Cell[BoxData[
      \(TraditionalForm\`\[Lambda]\)]],
  ".   The appropriate state set is \[CapitalSigma]  augmented by a \"neutral\
\" element."
}], "Text"],

Cell[BoxData[
    \(Q\  = ToString /@ \ {\[Epsilon], a, b, c}\)], "Input"],

Cell[TextData[{
  "These automata are naturally DFAs (The right action of the transition \
function of a DFA translates into left-associative multiplication in this \
context. Right?)  and are easy to construct via ",
  ButtonBox["FunctionToDFA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Text"],

Cell[BoxData[{
    \(ClearAll[del, mr]\), "\n", 
    \(\(del[\ p_, s_\ ]\  := \ \[Lambda][Concatenate[p, s]];\)\), "\n", 
    \(mr[s_]\  := \ 
      FunctionToDFA[Q, 3, del, 1, ToIndex[s, 3] + 1, 
        Normalize \[Rule] 1]\)}], "Input"],

Cell["Here are the machines.", "Text"],

Cell[BoxData[{
    \(mr[a]\), "\[IndentingNewLine]", 
    \(mr[b]\), "\[IndentingNewLine]", 
    \(mr[c]\)}], "Input"],

Cell["Some sanity testing.", "Text"],

Cell[BoxData[{
    \(Select[\ 
      Words[4, 3], \ \[Lambda][#] == "\<a\>" &\ ]\), "\[IndentingNewLine]", 
    \(LanguageFA[mr[a], 4]\)}], "Input"],

Cell[BoxData[
    \(SumFA[\ mr[a], mr[b], mr[c]]\  // \ MinimizeFA\)], "Input"],

Cell[TextData[{
  "Seems to work.  Machines for right-associative multiplication are a bit \
harder to come by.  Here we need transitions  ",
  Cell[BoxData[
      \(TraditionalForm\`p\ \(\[LongRightArrow]\_a\)\ q\)]],
  "  whenever  ",
  Cell[BoxData[
      \(TraditionalForm\`p\  = \ a\ \[CenterDot]\ q\)]],
  ".  The initial state is then ",
  Cell[BoxData[
      \(TraditionalForm\`s\  \[Element] \ \[CapitalSigma]\)]],
  "  and the final state \[Epsilon] (we use the same state set as for the \
other machines).  Since the transitions are the same, we compute them once \
and for all.  This is a bit of a hack."
}], "Text"],

Cell[BoxData[{
    \(tr\  = 
      Sort@Join[\[IndentingNewLine]\({1, 0, 1} + # &\)\  /@ \ 
            FlattenOne@
              MapIndexed[Prepend[#2, #1] &, 
                mti, {\(-1\)}], \[IndentingNewLine]Table[{i + 1, i, 1}, {i, 
              3}]]\), "\[IndentingNewLine]", 
    \(\(ml[s_]\  := \ 
        FA[\ 4, \ 3, \ tr, \ ToIndex[s, 3] + 1, {1}, {}, Q\ ];\)\)}], "Input"],

Cell["Again some modest sanity checking.", "Text"],

Cell[BoxData[{
    \(LanguageFA[ml[b], 4]\), "\[IndentingNewLine]", 
    \(Select[\ Words[4, 3], \ \[Rho][#] == "\<b\>" &\ ]\)}], "Input"],

Cell[BoxData[
    \(SumFA[\ ml[a], ml[b], ml[c]]\  // \ MinimizeFA\)], "Input"],

Cell["Looks good. ", "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["The Principal Machine", "Subsubsection"],

Cell["\<\
Now we construct the main machine, a sum of products of the \
component machines from above.\
\>", "Text"],

Cell[BoxData[
    \(M\  = \ 
      SumFA[\ IntersectionFA[ml[a], mr[a]], \ IntersectionFA[ml[b], mr[b]], \ 
        IntersectionFA[ml[c], mr[c]]]\)], "Input"],

Cell["The minimal DFA has 94 states.", "Text"],

Cell[BoxData[
    \(MM\  = \ MinimizeFA[M]\)], "Input"],

Cell["We can test that it works as advertised.", "Text"],

Cell[BoxData[{
    \(\(L\  = \ LanguageFA[MM, 5];\)\), "\n", 
    \(\[Lambda][L] === \ \[Rho][L]\)}], "Input"],

Cell[TextData[{
  "About one third of the strings in ",
  Cell[BoxData[
      \(TraditionalForm\`\[CapitalSigma]\^\(\(\ \)\(+\)\)\)]],
  "  appear to be in ",
  Cell[BoxData[
      \(TraditionalForm\`L\)]],
  "."
}], "Text"],

Cell[BoxData[{
    \(LanguageFA[MM, \(-12\), 
      SizeOnly \[Rule] True\ ]\), "\[IndentingNewLine]", 
    \(N[\ %\ /\ 3^Range[0, 12]]\)}], "Input"],

Cell[TextData[{
  "Since there is no sink in the minimal DFA for ",
  Cell[BoxData[
      \(TraditionalForm\`L\)]],
  " every word ",
  Cell[BoxData[
      \(TraditionalForm\`u\)]],
  " has an extension ",
  Cell[BoxData[
      \(TraditionalForm\`u\[VeryThinSpace]v\  \[Element] \ L\)]],
  "."
}], "Text"],

Cell[BoxData[
    \(TrapStatesFA[MM]\)], "Input"],

Cell[TextData[{
  "However, in general we can not choose an extension ",
  Cell[BoxData[
      \(TraditionalForm\`u\[VeryThinSpace]v\  \[Element] \ L\)]],
  "  so that, say, ",
  Cell[BoxData[
      \(TraditionalForm\`\[Lambda](
          u\[VeryThinSpace]v)\  = \ \(\[Rho](u\[VeryThinSpace]v)\  = \ 
          a\)\)]],
  ".  To see this, consider the diagram of the minimal DFA. We construct the \
collapse."
}], "Text"],

Cell[BoxData[{
    \(dag\  = \ 
      ToGraphFA[MM]\  // ToDirectedAcyclicGraph\), "\[IndentingNewLine]", 
    \(scc\  = \ VertexSet[dag]\)}], "Input"],

Cell["The components 8, 9, 10 have out-degree 0 in the collapse.", "Text"],

Cell[BoxData[
    \(DegreeGraph[dag, Full \[Rule] True]\  // \ EnumForm\)], "Input"],

Cell[TextData[{
  "There is exactly one final state in each of these three components.  We \
determine the corresponding transfer sequences which evaluate to ",
  Cell[BoxData[
      \(TraditionalForm\`a\)]],
  ", ",
  Cell[BoxData[
      \(TraditionalForm\`c\)]],
  " and ",
  Cell[BoxData[
      \(TraditionalForm\`b\)]],
  ", respectively."
}], "Text"],

Cell[BoxData[
    \(\(Intersection[Final[MM], #] &\)\  /@ \ 
      scc[\([{8, 9, 10}]\)]\)], "Input"],

Cell[BoxData[{
    \(tf\  = \ 
      TransferSequenceFA[MM, {1}, Flatten[%]]\), "\[IndentingNewLine]", 
    \(\[Lambda][tf]\)}], "Input"],

Cell[TextData[{
  "Thus, any extension of ",
  Cell[BoxData[
      FormBox[
        RowBox[{"u", " ", "=", " ", 
          StyleBox["abba",
            FontSlant->"Italic"]}], TraditionalForm]]],
  "  in ",
  Cell[BoxData[
      \(TraditionalForm\`L\)]],
  " will have value ",
  Cell[BoxData[
      \(TraditionalForm\`a\)]],
  ", and likewise for the other components."
}], "Text"],

Cell[BoxData[{
    \(Intersection[\ 
      Concatenate[abba, Words[3, 3]], \[IndentingNewLine]LanguageFA[MM, 
        7]]\), "\[IndentingNewLine]", 
    \(\[Lambda][%]\)}], "Input"],

Cell[BoxData[
    \(Intersection[\ 
        Concatenate[baaca, Words[3, 3]], \[IndentingNewLine]LanguageFA[MM, 
          8]]\  // \ \[Lambda]\)], "Input"],

Cell[BoxData[
    \(Intersection[\ 
        Concatenate[bbaca, Words[3, 3]], \[IndentingNewLine]LanguageFA[MM, 
          8]]\  // \ \[Lambda]\)], "Input"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Minimal NFAs and Blow-Up", "Section",
  CellTags->"c:1"],

Cell[TextData[{
  "Constructions of minimal NFAs that demonstrate exponential blow-up during \
conversion to the minimal DFA. \nNOTE:  Without the external code values of \
",
  Cell[BoxData[
      \(TraditionalForm\`\(\(n\)\(\ \)\(>\)\(13\)\(\ \)\)\)]],
  " or so  are going to result in tediously slow computations.\n\nIt is best \
to set the formatting to a high level."
}], "Text"],

Cell[BoxData[
    \(FormatAutomata[SizeOnly]\)], "Input"],

Cell[BoxData[
    \(ExtAutomata[]\)], "Input"],

Cell[CellGroupData[{

Cell["Iwama, Matsuura, Paterson", "Subsection",
  CellTags->"c:1"],

Cell[TextData[{
  "NFA:   ",
  Cell[BoxData[
      \(TraditionalForm\`n\)]],
  "\tDFA:  ",
  Cell[BoxData[
      \(TraditionalForm\`2\^\(\(\ \)\(n\)\) - \[Alpha]\)]],
  "\nThe order and divisibility condition below for ",
  Cell[BoxData[
      \(TraditionalForm\`n\)]],
  " and ",
  Cell[BoxData[
      \(TraditionalForm\`\[Alpha]\)]],
  " must be satisfied for this to work; the other parameters are just for \
notational convenience."
}], "Text"],

Cell[BoxData[{
    \(\(n\  = \ 12;\)\), "\[IndentingNewLine]", 
    \(\(\[Alpha]\  = \ 8;\)\), "\[IndentingNewLine]", 
    \(\(k\  = \ \[Alpha]\  - \ 1;\)\), "\[IndentingNewLine]", 
    \(\(m\  = \ n\  - \ k;\)\), "\[IndentingNewLine]", 
    \(5\  \[LessEqual] \ \[Alpha]\  < \ n\  && \ 
      GCD[\ n, k] \[Equal] \ 1\)}], "Input"],

Cell["\<\
Construct a set of taps at which non-deterministic transitions \
occur.\
\>", "Text"],

Cell[BoxData[
    \(A\  = \ 
      With[\ {r\  = \ Ceiling[k/4]}, \[IndentingNewLine]If[\ 
          EvenQ[k], \ \[IndentingNewLine]Join[\ Range[3, 2  r, 2], 
            Range[2  r, k - 4, 2]], \[IndentingNewLine]Range[3, k - 4, 
            2]]\ ]\)], "Input"],

Cell[TextData[{
  "Define a structured transition function.  We use  \n\t",
  Cell[BoxData[
      \(TraditionalForm\`t\_\(\(\ \)\(i\)\) \[Rule] \ \ {\ 0, i\ }\)]],
  "\tand\t",
  Cell[BoxData[
      \(TraditionalForm\`s\_\(\(\ \)\(i\)\) \[Rule] \ {\ 1, i\ }\)]],
  "."
}], "Text"],

Cell[BoxData[{
    \(ClearAll[next0, next1]\), "\[IndentingNewLine]", 
    \(\(next0[{0, i_}] := {{0, Mod[i + 1, k]}};\)\), "\n", 
    \(\(\(next0[{1, i_}] := {{1, Mod[i + 1, m]}};\)\(\n\)
    \)\), "\[IndentingNewLine]", 
    \(\(next1[{0, 2}]\ \  := \ {{1, 0}}\ ;\)\), "\[IndentingNewLine]", 
    \(\(\(next1[{0, i_}] := \ 
        If[MemberQ[A, i], {{0, i}, {0, 0}}, \ {{0, i}}\ ] /; 
          i \[NotEqual] 2;\)\(\[IndentingNewLine]\)
    \)\), "\[IndentingNewLine]", 
    \(\(next1[{1, 0}]\ \  := \ {{1, 1}};\)\), "\[IndentingNewLine]", 
    \(\(next1[{1, 1}]\ \  := \ {{0, 0}, {0, 2}};\)\), "\[IndentingNewLine]", 
    \(\(next1[{1, i_}] := \ {{1, i}} /; i \[GreaterEqual] 2;\)\)}], "Input"],

Cell[TextData[{
  "A cheap way to get the state set and the transitions: use ",
  ButtonBox["Closure",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Text"],

Cell[BoxData[{
    \(\({Q, tr}\  = 
        Closure[\ {{0, 0}, {1, 0}}, \ {next0, next1}, \ 
          DoTrace \[Rule] True, \ 
          SetClosure \[Rule] True\ ];\)\), "\[IndentingNewLine]", 
    \(Q\  // \ Length\)}], "Input"],

Cell["The corresponding nondeterministic machine.", "Text"],

Cell[BoxData[
    \(m\  = \ FA[\ Length[Q], 2, \ tr, \ {1}, {1}\ ]\)], "Input"],

Cell["The minimal automaton:", "Text"],

Cell[BoxData[
    \(mm\  = \ ToDFA[m]\)], "Input"],

Cell[BoxData[
    \(mmm\  = \ MinimizeFA[mm]\)], "Input"],

Cell[BoxData[
    \(Size[mmm]\  \[Equal] \ 2^n\  - \ \[Alpha]\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["Moore", "Subsection",
  CellTags->"c:1"],

Cell[TextData[{
  "NFA:   ",
  Cell[BoxData[
      \(TraditionalForm\`n\)]],
  "\tDFA: ",
  Cell[BoxData[
      \(TraditionalForm\`2\^\(\(\ \)\(n\)\)\)]],
  "\nThe classical example by Moore.  Note that full exponential blow-up of \
the minimal DFA automatically guarantees minimality of the NFA:  the number \
of distinct behaviors in any smaller NFA would be insufficient."
}], "Text"],

Cell[BoxData[
    \(MooreFA[
        n_]\  := \ \[IndentingNewLine]With[\ {res\  = \ 
            PermutationSA[\ {Join[{1}, RotateLeft@Range[2, n]], 
                RotateLeft@Range[n]}, \ 
              2\ ]}, \[IndentingNewLine]SetInitialFinalFA[\ 
          SwitchTransitionFA[res, n, 2], {1}, {n}]\ ]\)], "Input"],

Cell[BoxData[
    \(\(\(\ \)\(MooreFA[10]\  // DegreesFA\)\)\)], "Input"],

Cell[BoxData[
    \(MooreFA[10]\  // \ MinimizeFA\)], "Input"],

Cell["\<\
It is easy to see that the corresponding semiautomaton also works. \
\
\>", "Text"],

Cell[BoxData[
    \(\(MooreFA[10]\  // \ ToSA\)\  // \ MinimizeFA\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["Jiraskova", "Subsection",
  CellTags->"c:1"],

Cell[TextData[{
  "NFA:   ",
  Cell[BoxData[
      \(TraditionalForm\`n\)]],
  "\tDFA:  ",
  Cell[BoxData[
      \(TraditionalForm\`\(\(\ \)\(2\^\(\(\ \)\(n\)\)\)\)\)]],
  "\nA machine with a much higher degree on nondeterminism."
}], "Text"],

Cell[BoxData[
    \(JiraskovaFA[
        n_]\  := \ \[IndentingNewLine]Module[\ {next0, 
          next1}, \[IndentingNewLine]\tnext0[n] = \ {}; \[IndentingNewLine]\t
        next1[n] = \ {}; \[IndentingNewLine]\t
        next0[i_] := {1, i + 1}\ \  /; \ i\  < \ n; \[IndentingNewLine]\t
        next1[i_] := {i + 1}\ \  /; \ i\  < \ n; \[IndentingNewLine]\t{Q, 
            tr}\  = Closure[\ {1}, \ {next0, next1}, \ 
            DoTrace \[Rule] True, \ 
            SetClosure \[Rule] True\ ]; \[IndentingNewLine]\t
        FA[\ Length[Q], 2, \ tr, \ {1}, {n}\ ]\[IndentingNewLine]]\)], "Input"],

Cell["All states except the last are nondeterministic.", "Text"],

Cell[BoxData[
    \(\(JiraskovaFA[10]\  // \ DegreesFA\)\  // \ MatrixForm\)], "Input"],

Cell["The NFA has full blow-up and produces a reduced DFA.", "Text"],

Cell[BoxData[
    \(JiraskovaFA[10]\  // \ MinimizeFA\)], "Input"],

Cell["\<\
The corresponding SA fails miserably.  The DFA is still large\
\>", \
"Text"],

Cell[BoxData[
    \(m\  = \ 
      ToDFA[\ ToSA@JiraskovaFA[10], \ Normalize \[Rule] 1\ ]\)], "Input"],

Cell[BoxData[
    \(Complement[\ PowerSet[Range[10]], States[m]]\)], "Input"],

Cell["\<\
\[Ellipsis] but minimization collapses the machine. \
\>", "Text"],

Cell[BoxData[
    \(m\  // \ MinimizeFA\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["One-Permutation Automata", "Subsection"],

Cell[TextData[{
  "NFA:   ",
  Cell[BoxData[
      \(TraditionalForm\`n\)]],
  "\tDFA: ",
  Cell[BoxData[
      \(TraditionalForm\`2\^\(\(\ \)\(n\)\)\)]],
  "\nMoore's example generalizes to 1-permutation automata:  start with a \
transitive permutation automaton and change the label at a self-loop. \n\nAn \
example starting with a circulant graph  ",
  Cell[BoxData[
      \(TraditionalForm\`C(n; 0, 1)\)]],
  ":  label the offset 0 edges by ",
  Cell[BoxData[
      \(TraditionalForm\`a\)]],
  ", and the offset 1 edges by ",
  Cell[BoxData[
      \(TraditionalForm\`b\)]],
  ", producing a permutation automaton.  Then switch the label of the \
self-loop at state 1."
}], "Text"],

Cell[BoxData[
    \(Circulant01FA[
        n_]\  := \ \[IndentingNewLine]With[\ {res\  = \ 
            CirculantSA[n, {0, 1}, Join[Table[1, {n}], Table[2, {n}]], 
              2\ ]}, \[IndentingNewLine]SetInitialFinalFA[\ 
          SwitchTransitionFA[res, 1, 1], {1}, {1}]\ ]\)], "Input"],

Cell[TextData[{
  "We can build the circulant automaton easily via ",
  ButtonBox["CirculantSA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Text"],

Cell[BoxData[
    \(\(Circulant01FA[10]\ \  // \ DegreesFA\)\  // \ TableForm\)], "Input"],

Cell[BoxData[{
    \(Circulant01FA[10]\  // \ ToDFA\ \), "\[IndentingNewLine]", 
    \(\(%\  // \ MinimizeFA\)\ \  // \ Size\)}], "Input"],

Cell[TextData[{
  "A similar class of examples: we use a circulant graph  ",
  Cell[BoxData[
      \(TraditionalForm\`C(n; 1, d)\)]],
  "  where ",
  Cell[BoxData[
      \(TraditionalForm\`1\  < \ d\  < \ n\)]],
  ".  Label as before, then switch the label of the transition ",
  Cell[BoxData[
      \(TraditionalForm\`\((1, a, 2)\)\)]],
  "."
}], "Text"],

Cell[BoxData[
    \(Circulant1dFA[n_, 
        d_]\  := \ \[IndentingNewLine]With[\ {res\  = \ 
            CirculantSA[n, {1, d}, Join[Table[1, {n}], Table[2, {n}]], 
              2\ ]}, \[IndentingNewLine]SetInitialFinalFA[\ 
          SwitchTransitionFA[res, 1, 2], 1, \ 1\ ]\ ]\)], "Input"],

Cell[TextData[{
  "As one might suspect, this time the choice of ",
  Cell[BoxData[
      \(TraditionalForm\`d\)]],
  " matters."
}], "Text"],

Cell[BoxData[
    \(With[\ {n\  = \ 10}, \[IndentingNewLine]\t
        Table[\ {GCD[n, d - 1], Size[ToDFA[Circulant1dFA[n, d]]]}, \ {d, 2, 
            n - 1}]]\  // \ TableForm\)], "Input"],

Cell["Surprisingly, all the power automata are already reduced. ", "Text"],

Cell[BoxData[
    \(With[\ {n\  = \ 10}, \[IndentingNewLine]\t
        Table[\ {GCD[n, d - 1], Size[MinimizeFA[Circulant1dFA[n, d]]]}, \ {d, 
            2, n - 1}]]\  // \ TableForm\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["The Zigzag Decomposition", "Subsection"],

Cell[TextData[{
  "One might wonder how to construct all possible permutation automata on a \
given directed graph ",
  Cell[BoxData[
      \(TraditionalForm\`G\)]],
  ", disregarding isomorphism.   We require ",
  Cell[BoxData[
      \(TraditionalForm\`G\)]],
  " to be strongly connected and ",
  Cell[BoxData[
      \(TraditionalForm\`\((2, 2)\)\)]],
  "-regular and will use circulants as running examples.  In a permutation \
labeling a single label on some edge ",
  Cell[BoxData[
      \(TraditionalForm\`e\)]],
  " determines the labels in the whole zigzag containing ",
  Cell[BoxData[
      \(TraditionalForm\`e\)]],
  ".  A zigzag here is an alternating path starting and ending at the same \
edge (vertices may be repeated on this path, though). "
}], "Text"],

Cell[BoxData[
    \(\(?ZigzagDecompositionGraph\)\)], "Input"],

Cell[TextData[{
  "Some examples.  For ",
  Cell[BoxData[
      \(TraditionalForm\`C(12; 1, 2)\)]],
  " there is only one zigzag."
}], "Text"],

Cell[BoxData[
    \(G\  = \ CirculantGraph[\ 16, \ {1, 2}]\)], "Input"],

Cell[BoxData[
    \(ZigzagDecompositionGraph[G]\  // \ Length\)], "Input"],

Cell[TextData[{
  "But  ",
  Cell[BoxData[
      \(TraditionalForm\`C(12; 1, 5)\)]],
  "  has 4 zigzags (the difference of the strides and the number of nodes are \
not coprime here)."
}], "Text"],

Cell[BoxData[
    \(G\  = \ CirculantGraph[\ 12, \ {1, 5}]\)], "Input"],

Cell[BoxData[{
    \(\(ZZ\  = \ ZigzagDecompositionGraph[G];\)\), "\[IndentingNewLine]", 
    \(ZZ\  // \ Length\)}], "Input"],

Cell[BoxData[
    \(TableForm[ZZ, TableDepth \[Rule] 2, 
      TableSpacing \[Rule] {1, 1}]\)], "Input"],

Cell[TextData[{
  "It is straightforward to define functions that attach labels to the edges \
in a zigzag decomposition. The total number of labelings so obtained is ",
  Cell[BoxData[
      \(TraditionalForm\`2\^\(\(\ \)\(\(|\)\(\ \)\(ZZ\)\(\ \)\(|\)\)\)\)]],
  ". "
}], "Text"],

Cell[BoxData[{
    \(\(\(ClearAll[labelAltPath, labelZZD]\)\(\n\)
    \)\), "\[IndentingNewLine]", 
    \(\(\(labelAltPath[{\ L_List, \ 
            b_Integer}\ ]\  := \ \[IndentingNewLine]With[\ {n\  = \ 
              Length[L]}, \ \[IndentingNewLine]Thread[\ \ {\ First /@ L, \ 
              Mod[Range[0, n - 1] + b, 2] + 1, 
              Last /@ L}\ ]\ ];\)\(\[IndentingNewLine]\)
    \)\), "\n", 
    \(\(labelZZD[\ LL_List, \ 
          r_Integer\ ]\  := \ \[IndentingNewLine]With[\ {m\  = \ 
              Length[LL]}, \[IndentingNewLine]\t
          Sort@FlattenOne[\ 
              labelAltPath\  /@ \ 
                Pairs[LL, 
                  IntegerDigits[r, 2, 
                    m]]\ ]\[IndentingNewLine]];\)\)}], "Input"],

Cell[BoxData[
    \(labelZZD[\ ZZ, \ 0\ ]\)], "Input"],

Cell["\<\
Armed with these labeling functions we can produce all permutation \
automata on a given graph.  In our example, there are 16 labelings.  Again, \
we disregard symmetries here. \
\>", "Text"],

Cell[BoxData[
    \(\(all\  = \ 
        Table[\ SA[\ 12, 2, labelZZD[ZZ, r]], \ {r, 0, 15}\ ];\)\)], "Input"],

Cell[BoxData[
    \(SwitchTransitionFA[\ all[\([1]\)], \ 1, \ 2\ ]\  // \ 
      MinimizeFA\)], "Input"],

Cell[BoxData[
    \(SwitchTransitionFA[\ all[\([1]\)], \ 1, \ 6\ ]\  // \ 
      MinimizeFA\)], "Input"],

Cell["\<\
Without the external code the next computation may take 2-3 \
minutes.   It shows full or nearly full blow-up for all these 1-permutation \
machines.\
\>", "Text"],

Cell[BoxData[
    \(\(Size[MinimizeFA[\ SwitchTransitionFA[\ #, \ 1, \ 2\ ]]] &\)\  /@ \ 
      all\)], "Input"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Surjectivity of Cellular Automata", "Section",
  CellTags->"c:1"],

Cell[CellGroupData[{

Cell["ECA 45", "Subsection"],

Cell["\<\
Consider the elementary CA number 45. Here is the evolution of a \
one-point seed and a random configuration.\
\>", "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[BoxData[
    \(\(C\  = \ CA[45];\)\)], "Input"],

Cell[BoxData[
    \(\(EvolutionCA[\ C, \ 50\ ];\)\)], "Input"],

Cell[BoxData[
    \(\(EvolutionCA[\ C, SeedConfiguration[100, Random \[Rule] 2], \ 
        50\ ];\)\)], "Input"],

Cell["\<\
ECA 45 is surjective on infinite configurations and hence must be \
surjective on infinitely many finite grids (cyclic boundary conditions).  \
\
\>", "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[BoxData[
    \(ClassifyCA[C, Full \[Rule] True]\)], "Input"],

Cell[TextData[{
  "To determine the grid sizes ",
  Cell[BoxData[
      \(TraditionalForm\`n\)]],
  " for which ECA 45 acts like a bijection on ",
  Cell[BoxData[
      \(TraditionalForm\`\(\(\ \)\(\[ScriptCapitalC]\_n = {0, 1}\^n\)\)\)]],
  " we construct the automaton \n\t",
  Cell[BoxData[
      \(TraditionalForm\`\[ScriptCapitalA]\  = \ \(\(\[CapitalSigma]\)\(\ \
\)\)\_\(p \[Element] \ Q\)\ \[ScriptCapitalB]\_\(p, p\)\)]],
  "  \nwhere ",
  Cell[BoxData[
      \(TraditionalForm\`\[ScriptCapitalB]\  = \ \[LeftAngleBracket]Q, {0, 
            1}, \[Delta]\ \[RightAngleBracket]\)]],
  " is the de Bruijn automaton for the CA and ",
  Cell[BoxData[
      \(TraditionalForm\`\[ScriptCapitalB]\_\(p, p\)\)]],
  " indicates the automaton obtained by selecting ",
  Cell[BoxData[
      \(TraditionalForm\`p\)]],
  " as initial and final state."
}], "Text"],

Cell[BoxData[{
    \(\(sa\  = \ ToSA[C];\)\), "\n", 
    \(\(m\  = \ 
        UnionFA @@ \((\(SetInitialFinalFA[sa, {#}, {#}] &\)\  /@ \ 
              Range[Size[sa]])\);\)\), "\n", 
    \(mm\  = \ MinimizeFA[m]\)}], "Input"],

Cell[TextData[{
  "The growth function of the language of ",
  Cell[BoxData[
      \(TraditionalForm\`\[ScriptCapitalA]\)]],
  " suggests that ",
  Cell[BoxData[
      \(TraditionalForm\`C\)]],
  " will be surjevtive exactly for all odd ",
  Cell[BoxData[
      \(TraditionalForm\`n\)]],
  ". "
}], "Text"],

Cell[BoxData[
    \(LanguageFA[mm, \(-12\), SizeOnly \[Rule] True]\)], "Input"],

Cell[TextData[{
  "Here is a proof of this conjecture. We compute the generating function for \
the growth function of ",
  Cell[BoxData[
      \(TraditionalForm\`\[ScriptCapitalA]\)]],
  ". "
}], "Text"],

Cell[BoxData[
    \(g\  = \ GrowthSeriesDFA[mm, x]\)], "Input"],

Cell["Taylor expansion, just to be on the safe side.", "Text"],

Cell[BoxData[
    \(Series[g, {x, 0, 16}]\)], "Input"],

Cell[TextData[{
  "To obtain a closed form we use ",
  ButtonBox["SeriesTerm",
    ButtonStyle->"RefGuideLink"],
  "."
}], "Text"],

Cell["<<DiscreteMath`RSolve`", "Input",
  CellTags->"S5.21.1"],

Cell[BoxData[{
    \(Clear[n]\), "\[IndentingNewLine]", 
    \(ser\  = \ SeriesTerm[g\ , {x, 0, n}]\)}], "Input"],

Cell[TextData[{
  "A better answer is obtained by providing more information about the \
parameter ",
  Cell[BoxData[
      \(TraditionalForm\`n\)]],
  "."
}], "Text"],

Cell[BoxData[
    \(ser = 
      SeriesTerm[g, {x, 0, n}, 
        Assumptions \[Rule] {n \[Element] Integers}]\)], "Input"],

Cell[TextData[{
  "Which answer can be improved slightly by usign the fact that ",
  Cell[BoxData[
      \(TraditionalForm\`n > 0\)]],
  "."
}], "Text"],

Cell[BoxData[
    \(FullSimplify[\ ser, \ {n \[Element] Integers, n > 0}]\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["Grid Size 6", "Subsection"],

Cell[TextData[{
  "For grid size ",
  Cell[BoxData[
      \(TraditionalForm\`n = 6\)]],
  "  we should expect only ",
  Cell[BoxData[
      \(TraditionalForm\`2\^\(\(\ \)\(6\)\) - 2\^\(\(\ \)\(3\)\(\ \)\) = \ 
        56\)]],
  " first generation configurations. We determine the transients and periods \
of all configurations on a small grid."
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[BoxData[{
    \(\(all = Tuples[{0, 1}, 6];\)\), "\n", 
    \(Length[all]\)}], "Input",
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["A brute-force and somewhat inefficient computation using ",
    FontFamily->"Times"],
  ButtonBox["AnalyzeOrbit",
    ButtonStyle->"AddOnsLink"],
  StyleBox[".",
    FontFamily->"Times"]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[BoxData[{
    \(Clear[G, trper]\), "\n", 
    \(GlobalMapCA[G, C]\), "\[IndentingNewLine]", 
    \(\(trper\  = \ \ \(AnalyzeOrbit[G, #] &\)\  /@ \ all;\)\)}], "Input",
  AspectRatioFixed->True],

Cell[TextData[StyleBox["Here is a table of the frequencies of all the \
transient/period combinations. ",
  FontFamily->"Times"]], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[BoxData[
    \(Frequencies[\ trper\ ]\  // \ \(Curry[TableForm]\)[
        TableDepth \[Rule] 2]\)], "Input",
  AspectRatioFixed->True],

Cell[TextData[{
  "Hence there is one limit cycle of length 18, one of length 3, one of \
length 2 and three fixed points.\nHere is ",
  Cell[BoxData[
      \(TraditionalForm\`G\)]],
  "  acting on all configurations.  Moreover, there are exactly ",
  Cell[BoxData[
      \(TraditionalForm\`6\  + \ 2\  = \ 8\)]],
  "  endpoints in the diagram, as predicted."
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[BoxData[
    \(\(PlotFunction[G, all, 1, 
        PlotStyle \[Rule] {{Blue, 0.005}, {Red, 0.006}}, 
        LineStyle \[Rule] {{Black, 0.002}, {GrayLevel[0.5], 
              0.002}}];\)\)], "Input",
  AspectRatioFixed->True],

Cell[TextData[{
  "We follow ",
  Cell[BoxData[
      \(TraditionalForm\`G\)]],
  "  a few steps till the transient part is finished."
}], "Text"],

Cell[BoxData[
    \(\(PlotFunction[G, all, 6, 
        PlotStyle \[Rule] {{Blue, 0.005}, {Red, 0.006}}, 
        LineStyle \[Rule] {{Black, 0.002}, {GrayLevel[0.5], 
              0.002}}];\)\)], "Input",
  AspectRatioFixed->True],

Cell[TextData[{
  "Here is ",
  Cell[BoxData[
      \(TraditionalForm\`G\)]],
  "  acting on the limit cycles.  According to the table above, there are 26 \
points on the limit cycles."
}], "Text"],

Cell[BoxData[{
    \(limcyc\  = \ 
      PositionList[\ all, 
        Nest[\ Union[G /@ #] &, \ all, \ 6\ ]]\), "\[IndentingNewLine]", 
    \(limcyc\  // \ Length\)}], "Input"],

Cell[BoxData[
    \(\(PlotFunction[G, all, 6, 
        DoTrace \[Rule] 
          limcyc, \[IndentingNewLine]\t\ PlotStyle \[Rule] {{Blue, 
              0.005}, {Red, 0.006}}, \[IndentingNewLine]\t
        LineStyle \[Rule] {{Black, 0.002}, {GrayLevel[0.5], 
              0.002}}];\)\)], "Input",
  AspectRatioFixed->True],

Cell["The short cycles, including the fixed points.", "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[BoxData[
    \(\(pos\  = \ 
        Select[limcyc, 
          Nest[G, all[\([#]\)], 6] === all[\([#]\)] &\ ];\)\)], "Input"],

Cell[BoxData[
    \(\(PlotFunction[G, all, 6, DoTrace \[Rule] pos, \[IndentingNewLine]\t
        PlotStyle \[Rule] {{Blue, 0.005}, {Red, 0.01}}, \[IndentingNewLine]\t
        LineStyle \[Rule] {{Black, 0.002}, {GrayLevel[0.3], 
              0.002}}];\)\)], "Input",
  AspectRatioFixed->True],

Cell["The whole orbit of a configuration on the 18-cycle.", "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[BoxData[
    \(\(PlotFunction[G, all, 18, DoTrace \[Rule] {2}, \[IndentingNewLine]\t
        PlotStyle \[Rule] {{Blue, 0.005}, {Red, 0.01}}, \[IndentingNewLine]\t
        LineStyle \[Rule] {{Black, 0.002}, {GrayLevel[0.3], 
              0.002}}];\)\)], "Input",
  AspectRatioFixed->True],

Cell["\<\
Here is a trace of two the evolution of two  \"adjacent\" \
configurations (Hamming distance 1). \
\>", "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[BoxData[
    \(\(PlotFunction[G, all, 10, 
        DoTrace \[Rule] {31, 32}, \[IndentingNewLine]\t
        PlotStyle \[Rule] {{Blue, 0.005}, {Red, 0.006}}, \[IndentingNewLine]\t
        LineStyle \[Rule] {{Black, 0.002}, {GrayLevel[0.3], 
              0.002}}];\)\)], "Input",
  AspectRatioFixed->True]
}, Closed]],

Cell[CellGroupData[{

Cell["Grid Size 7", "Subsection"],

Cell[TextData[{
  "For ",
  Cell[BoxData[
      \(TraditionalForm\`n = 7\)]],
  " the global map is a bijection, which fact we can exploit to slighly speed \
up ",
  ButtonBox["AnalyzeOrbit",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[BoxData[{
    \(\(all = Tuples[{0, 1}, 7];\)\), "\n", 
    \(Length[all]\)}], "Input",
  AspectRatioFixed->True],

Cell[BoxData[{
    \(Clear[G, trper]\), "\n", 
    \(GlobalMapCA[G, C]\), "\[IndentingNewLine]", 
    \(\(trper\  = \ \ \(AnalyzeOrbit[G, #, Periodic \[Rule] True] &\)\  /@ \ 
          all;\)\)}], "Input",
  AspectRatioFixed->True],

Cell[TextData[StyleBox["Again the table of the frequencies of all the \
transient/period combinations. ",
  FontFamily->"Times"]], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[BoxData[
    \(Frequencies[\ trper\ ]\  // \ \(Curry[TableForm]\)[
        TableDepth \[Rule] 2]\)], "Input",
  AspectRatioFixed->True],

Cell["\<\
Hence there is one limit cycle of length 126 and one of length 2. \
The latter consists of all-zero and all-one configurations. \
\>", "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[BoxData[
    \(Select[\ all, \ G[G[#]] === # &\ ]\)], "Input"],

Cell["\<\
 Of course, a better way to compute these cycles is to generate the \
diagram directly and then use Tarjan's algorithm.\
\>", "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[BoxData[
    \(diagram\  = \ 
      Graph[\ 128, \ 128, \ "\<EList\>", \ 
        Pairs[Range[128], PositionList[\ all, \ G /@ all]]\ ]\)], "Input"],

Cell[BoxData[
    \(StronglyConnectedComponents[diagram]\)], "Input"],

Cell[TextData[{
  Cell[BoxData[
      \(TraditionalForm\`G\)]],
  " acting on the whole configuration space."
}], "Text"],

Cell[BoxData[
    \(\(PlotFunction[G, all, 1, 
        PlotStyle \[Rule] {{Blue, 0.005}, {Red, 0.006}}, 
        LineStyle \[Rule] {{Black, 0.002}, {GrayLevel[0.5], 
              0.002}}];\)\)], "Input",
  AspectRatioFixed->True]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Cellular Automata and Fischer Automata", "Section",
  CellTags->"c:1"],

Cell["Back to normal formatting.", "Text"],

Cell[BoxData[
    \(FormatAutomata[Automatic]\)], "Input"],

Cell[CellGroupData[{

Cell["Constructing the Minimal Fischer Automaton", "Subsection"],

Cell[TextData[{
  "Consider a one-dimensional cellular automaton \[Rho] as a transducer on \
bi-infinite words, so ",
  Cell[BoxData[
      \(TraditionalForm\`\[Rho]\  : \ \[CapitalSigma]\^\(\(\ \
\)\(\[Infinity]\)\)\ \[LongRightArrow]\ \[CapitalSigma]\^\(\(\ \
\)\(\[Infinity]\)\)\)]],
  ".  The finite factors of ",
  Cell[BoxData[
      \(TraditionalForm\`\[Rho](X)\)]],
  " where ",
  Cell[BoxData[
      \(TraditionalForm\`X \[Element] \ \[CapitalSigma]\^\(\(\ \
\)\(\[Infinity]\)\)\)]],
  "  form a regular language accepted by the de Bruijn automaton associated \
with \[Rho].  For example, for ECA 73 we get:"
}], "Text"],

Cell[BoxData[
    \(sa\  = \ ToSA[CA[73]]\)], "Input"],

Cell["\<\
The minimal DFA here has maximal size 16.\
\>", "Text"],

Cell[BoxData[
    \(m\  = \ MinimizeFA[sa]\)], "Input"],

Cell["\<\
All the ECAs whose minimal DFA have maximum size are based on \
one-permutation automata. \
\>", "Text"],

Cell[BoxData[
    \(\(\(blowup\)\(\ \)\(=\)\(\ \)\(Select[\ 
        Range[0, 
          127], \ \ Size[MinimizeFA[ToSA[CA[#]]]] \[Equal] 
            16 &\ ]\)\(\ \)\)\)], "Input"],

Cell[BoxData[
    \(TableForm[\(DegreesFA[ToSA[CA[#]]] &\) /@ %, \ 
      TableDepth \[Rule] 2]\)], "Input"],

Cell["\<\
Back to ECA 73. The growth function is rather hopelessly \
complicated:\
\>", "Text"],

Cell[BoxData[
    \(GrowthSeriesDFA[m, x]\)], "Input"],

Cell[BoxData[
    \(Series[\ %, \ {x, 0, 12}\ ]\)], "Input"],

Cell["\<\
The complement forms an ideal, but this ideal fails to have a \
finite base. \
\>", "Text"],

Cell[BoxData[
    \(IdealQFA[\ ComplementFA[m]]\)], "Input"],

Cell[BoxData[
    \(IdealBaseFA[ComplementFA[m]]\)], "Input"],

Cell[BoxData[
    \(InfiniteQFA[%]\)], "Input"],

Cell["\<\
A deterministic reduced semiautomaton for the cover language can be \
determined thus:\
\>", "Text"],

Cell[BoxData[{
    \(mf\  = \ MinimalFischerFA[sa]\), "\n", 
    \(EquivalentQFA[mf, m]\)}], "Input"],

Cell[TextData[{
  "The algorithm for the minimal Fischer automaton extracts a subautomaton of \
the minimal DFA determined by a strongly connected component ",
  Cell[BoxData[
      \(TraditionalForm\`C\)]],
  ".  The component has the property that in the collapse of the transition \
diagram it has edges leading only to the sink. In this case, there is only \
one component other than the sink."
}], "Text"],

Cell[BoxData[
    \(G\  = \ ToGraphFA[m]\)], "Input"],

Cell[BoxData[
    \(ToDirectedAcyclicGraph[G]\)], "Input"],

Cell[BoxData[
    \(scc\  = \ VertexSet[%]\)], "Input"],

Cell["\<\
Here is a more interesting example, a binary width-4 automaton.   \
There are several non-trivial components. \
\>", "Text"],

Cell[BoxData[{
    \(\(m\  = \ \(\(CA[4, \(-2\), 15042]\  // \ ToSA\)\  // \ ToDFA\)\  // \ 
          MinimizeFA;\)\), "\n", 
    \(TrapStatesFA[m]\), "\n", 
    \(ToDirectedAcyclicGraph[ToGraphFA[m]]\), "\n", 
    \(scc\  = \ VertexSet[%]\)}], "Input"],

Cell["\<\
We determine the position of the 6-element strongly connected \
component.\
\>", "Text"],

Cell[BoxData[
    \(p6\  = \ PositionOne[\ Length /@ scc, 6]\)], "Input"],

Cell[BoxData[{
    \(mf = 
      ToSA[SelectFA[m, 
          scc\[LeftDoubleBracket]p6\[RightDoubleBracket]]]\), "\n", 
    \(EquivalentQFA[m, mf]\)}], "Input"],

Cell["\<\
The behaviors of the states in the minimal Fischer automaton agree \
with the behavior of the same states in the minimal DFA.  Thus, the automaton \
is reduced. \
\>", "Text"],

Cell[BoxData[
    \(Table[\ 
      IsomorphicQDFA[\ BehaviorFA[\ mf, \ p\ ], \ 
        BehaviorFA[m, scc[\([p6, p]\)]\ ]], \ {p, 6}\ ]\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["Growth", "Subsection"],

Cell["\<\
Another example, the additive ECA 90.  On biinfinite configurations \
rule 90 is surjective (and even open).\
\>", "Text"],

Cell[BoxData[
    \(CA[90]\  // \ ClassifyCA\)], "Input"],

Cell[TextData[{
  "How about surjectivity on finite grids of length ",
  Cell[BoxData[
      \(TraditionalForm\`n\)]],
  ", say, with cyclic boundary conditions?  It is easy to construct a machine \
from the de Bruijn automaton that accepts only words associated with the \
behavior of \[Rho] on finite grids."
}], "Text"],

Cell[BoxData[{
    \(\(sa\  = \ ToSA[\ CA[90]];\)\), "\[IndentingNewLine]", 
    \(m\  = \ 
      MinimizeFA[
        SumFA @@ Table[\ 
            SetInitialFinalFA[sa, {p}, {p}], \ {p, 4}]]\)}], "Input"],

Cell[BoxData[
    \(LanguageFA[m, \(-10\), \ SizeOnly \[Rule] True]\)], "Input"],

Cell["\<\
A sanity check by computing all configurations at time 1 of length \
up to 10.\
\>", "Text"],

Cell[BoxData[{
    \(GlobalMapCA[f, CA[90]]\), "\n", 
    \(Table[\ 
      Length[Union[\ f\  /@ \ Tuples[{0, 1}, k]]], \ {k, 10}\ ]\)}], "Input"],

Cell["The growth function is rather simple in this case.", "Text"],

Cell[BoxData[{
    \(GrowthSeriesDFA[m, x]\), "\n", 
    \(Series[\ %, \ {x, 0, 12}\ ]\)}], "Input"],

Cell["\<\
Simplifying the corresponding infinite series shows that ECA 90 is \
not surjective on any finite grid.\
\>", "Text"],

Cell[BoxData[
    \(1\  + \ \ \ Sum[\ 
          2^\((i - 1)\) \((x^i + \ x^\((i + 1)\))\), \ {i, 1, Infinity, \ 
            2}\ ]\  // \ Together\)], "Input"],

Cell["\<\
Another well-studied and closely related linear ECA is rule number \
150. \
\>", "Text"],

Cell[BoxData[{
    \(\(sa\  = \ ToSA[\ CA[150]];\)\), "\[IndentingNewLine]", 
    \(m\  = \ 
      MinimizeFA[
        SumFA @@ Table[\ 
            SetInitialFinalFA[sa, {p}, {p}], \ {p, 4}]]\)}], "Input"],

Cell[BoxData[
    \(LanguageFA[m, \(-10\), \ SizeOnly \[Rule] True]\)], "Input"],

Cell[TextData[{
  "It seems that rule 150 is surjective on a grid of length ",
  Cell[BoxData[
      \(TraditionalForm\`n\)]],
  " whenever ",
  Cell[BoxData[
      \(TraditionalForm\`n\)]],
  " is ",
  Cell[BoxData[
      \(TraditionalForm\`n\)]],
  " is not divisible by 3. \nThe growth function is slightly more complicated \
for rule 150, but we can still do a proof by computer algebra."
}], "Text"],

Cell[BoxData[{
    \(GrowthSeriesDFA[m, x]\), "\n", 
    \(Series[\ %, \ {x, 0, 12}\ ]\)}], "Input"],

Cell[BoxData[
    \(1\  + \ \ \ Sum[\ 
          2^i \((x^i + 2\ \ x^\((i + 1)\)\  + \ x^\((i + 2)\))\), \ {i, 1, 
            Infinity, \ 3}\ ]\  // \ Together\)], "Input"],

Cell["\<\
Of course, in general we obtain rather more complicated rational \
functions and it is difficult to solve the corresponding recurrences.\
\>", \
"Text"],

Cell[BoxData[{
    \(\(sa\  = \ ToSA[\ CA[30]];\)\), "\[IndentingNewLine]", 
    \(m\  = \ 
      MinimizeFA[
        SumFA @@ Table[\ 
            SetInitialFinalFA[sa, {p}, {p}], \ {p, 4}]]\)}], "Input"],

Cell[BoxData[
    \(LanguageFA[m, \(-10\), \ SizeOnly \[Rule] True]\)], "Input"],

Cell[BoxData[
    \(GrowthSeriesDFA[m, x]\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["A Counterexample", "Subsection"],

Cell[TextData[{
  "The minimal Fischer automaton for any factorial, extensible and transitive \
language ",
  Cell[BoxData[
      \(TraditionalForm\`L\)]],
  " is uniquely determined. Here is an example due to A. Jonoska that shows \
that transitivity is essential here: two equivalent, deterministic minimal \
semiautomata that fail to be isomorphic."
}], "Text"],

Cell[BoxData[{
    \(\(s1 = 
        SA[4, 3, {{1, 1, 2}, {1, 2, 3}, {2, 1, 1}, {2, 2, 4}, {3, 3, 3}, {3, 
              2, 4}, {4, 1, 3}}];\)\), "\n", 
    \(\(s2 = 
        SA[4, 3, {{1, 2, 4}, {1, 1, 3}, {1, 3, 3}, {2, 1, 2}, {2, 2, 1}, {3, 
              3, 3}, {3, 2, 4}, {4, 1, 3}}];\)\)}], "Input"],

Cell[BoxData[{
    \(DeterministicQFA[s1]\), "\n", 
    \(DeterministicQFA[s2]\), "\n", 
    \(EquivalentQFA[s1, s2]\)}], "Input"],

Cell["The underlying digraphs fail to be isomorphic.", "Text"],

Cell[BoxData[{
    \(\(s1\  // \ ToGraphFA\)\  // \ 
      StronglyConnectedComponents\), "\[IndentingNewLine]", 
    \(\(s2\  // \ ToGraphFA\)\  // \ StronglyConnectedComponents\)}], "Input"],

Cell[TextData[{
  "The machines are reduced, but note that unlike with Fischer automata this \
does not suffice to establish minimality here (it is not hard to see, though, \
that no deterministic machine of size 3 or less can produce ",
  Cell[BoxData[
      \(TraditionalForm\`L\)]],
  "). "
}], "Text"],

Cell[BoxData[{
    \(ToClasses[Range[4], MinimizeFA[SetInitialFA[s1, #]] &, 
      Type \[Rule] Function]\), "\n", 
    \(ToClasses[Range[4], MinimizeFA[SetInitialFA[s2, #]] &, 
      Type \[Rule] Function]\)}], "Input",
  ImageRegion->{{-0, 1}, {0, 1}}],

Cell[TextData[{
  "One of the semiautomata is a subautomaton of the minimal DFA for ",
  Cell[BoxData[
      \(TraditionalForm\`L\)]],
  ", namely the subautomaton determined by the non-transient points in the \
diagram of the minimal DFA.   Here is a brutish computation to establish this \
claim: show that the behaviors of all states in ",
  Cell[BoxData[
      \(TraditionalForm\`s\_2\)]],
  " are behaviors of states in the minimal DFA ",
  Cell[BoxData[
      \(TraditionalForm\`m\)]],
  ". "
}], "Text"],

Cell[BoxData[{
    \(\(m\  = \ MinimizeFA[s1];\)\), "\n", 
    \(\(s2bh\  = \ Table[\ BehaviorFA[s2, p], {p, Size[s2]}];\)\), "\n", 
    \(\(mbh\  = \ Table[\ BehaviorFA[m, p], {p, Size[m]}];\)\), "\n", 
    \(Select[\ CartesianProduct[mbh, s2bh], \ 
      IsomorphicQDFA[#[\([1]\)], #[\([2]\)]] &\ ]\ \), "\[IndentingNewLine]", \

    \(%\  // \ Length\)}], "Input"],

Cell[BoxData[{
    \(\(mm\  = \ DeleteStateFA[\ m, \ TrapStatesFA[m]\ ];\)\), "\n", 
    \(\(\(mm\  // \ ToGraphFA\)\  // \ NonTransientGraph\)\  // \ 
      VertexSet\)}], "Input"],

Cell["\<\
Incidentally, the ideal of forbidden blocks has a finite \
base.\
\>", "Text"],

Cell[BoxData[
    \(mid\  = \ IdealBaseFA[\ ComplementFA[s1]\ ]\)], "Input"],

Cell[BoxData[
    \(LanguageFA[\ mid, \ Full \[Rule] True\ ]\)], "Input"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Multiplicity Semigroups", "Section",
  CellTags->"c:1"],

Cell[CellGroupData[{

Cell["Surjective Global Maps", "Subsection"],

Cell["A surjective binary cellular automaton of width 4.", "Text"],

Cell[BoxData[{
    \(\(C = \ CA[4, 2, 7770];\)\), "\[IndentingNewLine]", 
    \(C\  // \ ClassifyCA\)}], "Input"],

Cell["\<\
The local map is surjective, but fails to be injective or even \
open.\
\>", "Text"],

Cell[BoxData[
    \(\(EvolutionCA[C];\)\)], "Input"],

Cell[BoxData[
    \(\(EvolutionCA[C, 
        SeedConfiguration[\ 100, \ Random \[Rule] 2]\ ];\)\)], "Input"],

Cell[TextData[{
  "Since ",
  Cell[BoxData[
      \(TraditionalForm\`C\)]],
  " is surjective, all words must have the same multiplicity in the de Bruijn \
semiautomaton associated with ",
  Cell[BoxData[
      \(TraditionalForm\`C\)]],
  ". "
}], "Text"],

Cell[BoxData[{
    \(\(sa\  = \ ToSA[C];\)\), "\n", 
    \(LanguageFA[sa, 5, Multiplicity \[Rule] True\ ]\)}], "Input"],

Cell[TextData[{
  "We construct the multiplicity semigroup of the automaton, the image of the \
canonical homomorphism  ",
  Cell[BoxData[
      FormBox[
        RowBox[{\(\[CapitalSigma]\^\(\(\ \)\(*\)\)\), "\[Rule]", " ", 
          SuperscriptBox[
            StyleBox["2",
              FontWeight->"Bold"], \(\(\ \)\(Q\ \[Cross]\ Q\)\)]}], 
        TraditionalForm]]],
  ".  The semigroup has size 754, so this will take some 15 seconds.  Note \
that this semigroup will be infinite if you start with a non-surjective CA."
}], "Text"],

Cell[BoxData[{
    \(TransitionMatrixFA[\ sa, \ f]\), "\n", 
    \(\({S, W, eq} = 
          GenerateSG[\ GEN[\ 4, 2, {f[a], f[b]}], External \[Rule] \ False, 
            Equations \[Rule] \ True];\)\  // \ Timing\)}], "Input"],

Cell[BoxData[
    \(S\  // \ Length\)], "Input"],

Cell["\<\
We extract the 24 matrices of minimal rank, which turns out to be 1 \
in this case.\
\>", "Text"],

Cell[BoxData[{
    \(\(rank[m_?MatrixQ]\  := \ 
        Last@Dimensions[m]\  - \ 
          Length@NullSpace[m];\)\), "\[IndentingNewLine]", 
    \(\(SS\  = \ ToList[S];\)\), "\n", 
    \(\(rk\  = \ rank\  /@ \ SS;\)\), "\n", 
    \(\(rk\  // \ Frequencies\)\  // \ TableForm\)}], "Input"],

Cell["\<\
The structure of the matrices is best represented \
graphically.\
\>", "Text"],

Cell[BoxData[{
    \(\(pos\  = \ Flatten@Position[rk, Min[rk]];\)\), "\n", 
    \(\(minS\  = \ SS[\([pos]\)];\)\), "\n", 
    \(\(\(PlotMatrix[#, DisplayFunction \[Rule] Identity, 
            GridLines \[Rule] True] &\)\  /@ \ minS;\)\), "\n", 
    \(\(ShowArray[\ %, \ 6\ ]\ ;\)\)}], "Input"],

Cell["\<\
There are 6 classes of matrices according to non-zero rows and 4 \
according to columns; all combinations are possible.\
\>", "Text"],

Cell[BoxData[
    \(\(\(\(Union[First /@ Position[#, 1]] &\)\  /@ \ minS\  // \ Union\ \n
    \(Union[First /@ Position[Thread[#], 1]] &\)\  /@ \ minS\  // \ 
      Union\)\(\ \)\)\)], "Input"],

Cell["We extract the witnesses for the minimal-rank matrices.", "Text"],

Cell[BoxData[
    \(wits\  = \ W[\([pos]\)]\)], "Input"],

Cell["Pick one of them at random.", "Text"],

Cell[BoxData[
    \(\(w\  = \ wits[\([Random[Integer, {1, Length[wits]}]]\)];\)\)], "Input"],

Cell[TextData[{
  "Computing   ",
  Cell[BoxData[
      \(TraditionalForm\`\[Delta](Q, w)\)]],
  "  and    ",
  Cell[BoxData[
      \(TraditionalForm\`\(\[Delta]\^\(\(\ \)\(r\)\)\)(Q, 
        w\^\(\(\ \)\(r\)\))\)]],
  "    shows that associated row/column pattern. "
}], "Text"],

Cell[BoxData[{
    \(TransitionFunctionFA[sa, del]\), "\n", 
    \(TransitionFunctionFA[ReverseFA[sa], delr]\), "\[IndentingNewLine]", 
    \(del[Range[8], w\ \ ]\), "\[IndentingNewLine]", 
    \(delr[Range[8], WordReverse[w]\ \ ]\)}], "Input"],

Cell["\<\
The computations of the semiautomaton on one of these witnesses.  \
Notice the bottleneck.\
\>", "Text"],

Cell[BoxData[
    \(\(PlotComputationFA[\ sa, w, Full \[Rule] \ False\ ];\)\)], "Input"],

Cell[TextData[{
  "To check that the global map is not open we use a product machine \
construction (see also ",
  ButtonBox["ClassifyCA",
    ButtonStyle->"AddOnsLink"],
  ").  By preserving structured state and vertex sets we can trace a path \
from the diagonal in the product (which forms a strongly connected component \
since the CA is surjective) to some other component."
}], "Text"],

Cell[BoxData[
    \(s2\  = \ ProductFA[sa, sa, Normalize \[Rule] 1]\)], "Input"],

Cell[TextData[{
  "We extract the non-transient part of the diagram of ",
  Cell[BoxData[
      \(TraditionalForm\`S\^\(\(\ \)\(2\)\)\)]],
  ". "
}], "Text"],

Cell[BoxData[
    \(G\  = \ 
      NonTransientGraph[ToGraphFA[s2, Normalize \[Rule] 1], 
        Normalize \[Rule] 2]\)], "Input"],

Cell[TextData[{
  "The first SCC corresponds to the diagonal, and the second vertex in ",
  Cell[BoxData[
      \(TraditionalForm\`G\)]],
  " is another non-trivial SCC.  All other points lie on paths from component \
1 to component 2."
}], "Text"],

Cell[BoxData[{
    \(\(V\  = \ VertexSet[G];\)\), "\[IndentingNewLine]", 
    \(V[\([1]\)]\), "\[IndentingNewLine]", 
    \(V[\([2]\)]\)}], "Input"],

Cell[BoxData[
    \(Length\  /@ \ V\)], "Input"],

Cell["\<\
Hence we can construct disjoint paths of arbitrary length by \
finding a cycle in the second component. \
\>", "Text"],

Cell[BoxData[{
    \(\(ss\  = \ 
        SelectFA[\ s2, \ PositionList[\ States[s2], \ V[\([2]\)]\ ], 
          Normalize \[Rule] 2\ ];\)\), "\[IndentingNewLine]", 
    \(ss\  = \ 
      SetInitialFinalFA[\ ss, \ \[IndentingNewLine]\t
        Position[\ States[ss], \ {1, 3}\ ], \ 
        Position[\ States[ss], \ {1, 3}\ ]]\)}], "Input"],

Cell[BoxData[
    \(LanguageFA[\ ss, \ \(-10\), \ SizeOnly \[Rule] True\ ]\)], "Input"],

Cell[BoxData[
    \(w2\  = \ WordPower[\ First@LanguageFA[\ ss, \ 7\ ], \ 3\ ]\)], "Input"],

Cell[BoxData[
    \(\(PlotComputationFA[\ sa, w2, Full \[Rule] \ False\ ];\)\)], "Input"],

Cell["And the corresponding matrix.", "Text"],

Cell[BoxData[{
    \(\(M\  = 
        S[\([\(Position[\ W, WordReduce[\ \ w2, \ eq\ ]\ ]\)[\([1, 
                1]\)]]\)];\)\), "\n", 
    \(\(PlotMatrix[\ \ M, \ GridLines \[Rule] True];\)\)}], "Input"],

Cell[TextData[{
  "We can extract the separate strands of the computation from the positions \
of 1 in the matrix ",
  Cell[BoxData[
      \(TraditionalForm\`M\)]],
  ". "
}], "Text"],

Cell[BoxData[{
    \(pos\  = \ Position[M, 1]\), "\[IndentingNewLine]", 
    \(trc\  = \ 
      Map[\ \ Last /@ # &, \ 
          Split[\ Sort[\ 
              Reverse\  /@ \ 
                pos\ ], #1[\([1]\)] \[Equal] #2[\([1]\)] &\ ], \ {1}\ ]\  \
// \ Union\)}], "Input"],

Cell[BoxData[{
    \(\(\(PlotComputationFA[\ sa, w2, \ DoTrace \[Rule] #, 
            DisplayFunction \[Rule] Identity\ ] &\)\  /@ \ 
        trc;\)\), "\[IndentingNewLine]", 
    \(\(ShowArray[\ %, \ 2\ ];\)\)}], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["Open Global Maps", "Subsection"],

Cell["\<\
A binary cellular automaton of width 5 whose local map is \
surjective and open but fails to be injective.\
\>", "Text"],

Cell[BoxData[{
    \(\(C = \ CA[\ 4, 2, 15555];\)\), "\[IndentingNewLine]", 
    \(C\  // \ ClassifyCA\)}], "Input"],

Cell[BoxData[
    \(C\  // \ WelchCA\)], "Input"],

Cell[TextData[{
  "We construct the multiplicity semigroup of the automaton, the image of the \
canonical homomorphism  ",
  Cell[BoxData[
      FormBox[
        RowBox[{\(\[CapitalSigma]\^\(\(\ \)\(*\)\)\), "\[Rule]", " ", 
          SuperscriptBox[
            StyleBox["2",
              FontWeight->"Bold"], \(\(\ \)\(Q\ \[Cross]\ Q\)\)]}], 
        TraditionalForm]]],
  ".  The semigroup has size 24."
}], "Text"],

Cell[BoxData[{
    \(\(sa\  = \ ToSA[C];\)\), "\[IndentingNewLine]", 
    \(TransitionMatrixFA[\ sa, \ f]\), "\n", 
    \(\({S, W, eq} = 
          GenerateSG[\ GEN[\ 4, 2, {f[a], f[b]}], External \[Rule] \ False, 
            Equations \[Rule] \ True];\)\  // \ Timing\)}], "Input"],

Cell[BoxData[
    \(S\  // \ Length\)], "Input"],

Cell["All matrices have the same rank 4.", "Text"],

Cell[BoxData[{
    \(\(rank[m_?MatrixQ]\  := \ 
        Last@Dimensions[m]\  - \ 
          Length@NullSpace[m];\)\), "\[IndentingNewLine]", 
    \(rank\  /@ \ \ ToList[S]\  // \ Union\)}], "Input"],

Cell["The matrices in the semigroup.", "Text"],

Cell[BoxData[{
    \(\(\(PlotMatrix[#, DisplayFunction \[Rule] Identity, 
            GridLines \[Rule] True] &\)\  /@ \ ToList[S];\)\), "\n", 
    \(\(ShowArray[\ %\ ]\ ;\)\)}], "Input"],

Cell["This time there are 2 classes of matrices.", "Text"],

Cell[BoxData[
    \(\(\(\(Union[First /@ Position[#, 1]] &\)\  /@ \ \ ToList[S]\  // \ 
      Union\ \n
    \(Union[First /@ Position[Thread[#], 1]] &\)\  /@ \ \ ToList[S]\  // \ 
      Union\)\(\ \)\)\)], "Input"],

Cell["The computations of the semiautomaton on a random word.", "Text"],

Cell[BoxData[
    \(w\  = \ RandomWord[\ 16, \ 2\ ]\)], "Input"],

Cell[BoxData[
    \(\(PlotComputationFA[\ sa, w, Full \[Rule] \ False\ ];\)\)], "Input"],

Cell["And the corresponding matrix.", "Text"],

Cell[BoxData[{
    \(\(M\  = 
        S[\([\(Position[\ W, WordReduce[\ \ w, \ eq\ ]\ ]\)[\([1, 
                1]\)]]\)];\)\), "\n", 
    \(\(PlotMatrix[\ \ M, \ GridLines \[Rule] True];\)\)}], "Input"],

Cell[TextData[{
  "We can extract the separate strands of the computation from the positions \
of 1 in the matrix ",
  Cell[BoxData[
      \(TraditionalForm\`M\)]],
  ". "
}], "Text"],

Cell[BoxData[{
    \(pos\  = \ Position[M, 1]\), "\[IndentingNewLine]", 
    \(trc\  = \ 
      Map[\ \ Last /@ # &, \ 
          Split[\ Sort[\ 
              Reverse\  /@ \ 
                pos\ ], #1[\([1]\)] \[Equal] #2[\([1]\)] &\ ], \ {1}\ ]\  \
// \ Union\)}], "Input"],

Cell[BoxData[{
    \(\(\(PlotComputationFA[\ sa, w, \ DoTrace \[Rule] #, 
            DisplayFunction \[Rule] Identity\ ] &\)\  /@ \ 
        trc;\)\), "\[IndentingNewLine]", 
    \(\(ShowArray[\ %];\)\)}], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["Injective Global Maps", "Subsection"],

Cell["Lastly, a reversible binary cellular automaton of width 4.", "Text"],

Cell[BoxData[{
    \(\(C = \ CA[\ 4, 2, 21845\ ];\)\), "\[IndentingNewLine]", 
    \(C\  // \ ClassifyCA\)}], "Input"],

Cell[BoxData[
    \(C\  // \ WelchCA\)], "Input"],

Cell[TextData[{
  "We construct the multiplicity semigroup of the automaton, the image of the \
canonical homomorphism  ",
  Cell[BoxData[
      FormBox[
        RowBox[{\(\[CapitalSigma]\^\(\(\ \)\(*\)\)\), "\[Rule]", " ", 
          SuperscriptBox[
            StyleBox["2",
              FontWeight->"Bold"], \(\(\ \)\(Q\ \[Cross]\ Q\)\)]}], 
        TraditionalForm]]],
  ".  The semigroup has size 14"
}], "Text"],

Cell[BoxData[{
    \(\(sa\  = \ ToSA[C];\)\), "\[IndentingNewLine]", 
    \(TransitionMatrixFA[\ sa, \ f]\), "\n", 
    \(\({S, W, eq} = 
          GenerateSG[\ GEN[\ 4, 2, {f[a], f[b]}], External \[Rule] \ False, 
            Equations \[Rule] \ True];\)\  // \ Timing\)}], "Input"],

Cell[BoxData[
    \(S\  // \ Length\)], "Input"],

Cell["We extract the 8 matrices of minimal rank 1.", "Text"],

Cell[BoxData[{
    \(\(rank[m_?MatrixQ]\  := \ 
        Last@Dimensions[m]\  - \ 
          Length@NullSpace[m];\)\), "\[IndentingNewLine]", 
    \(\(SS\  = \ ToList[S];\)\), "\n", 
    \(\(rk\  = \ rank\  /@ \ SS;\)\), "\n", 
    \(\(rk\  // \ Frequencies\)\  // \ TableForm\)}], "Input"],

Cell["\<\
The structure of the matrices is best represented \
graphically.\
\>", "Text"],

Cell[BoxData[{
    \(\(pos\  = \ Flatten@Position[rk, Min[rk]];\)\), "\n", 
    \(\(minS\  = \ SS[\([pos]\)];\)\), "\n", 
    \(\(\(PlotMatrix[#, DisplayFunction \[Rule] Identity, 
            GridLines \[Rule] True] &\)\  /@ \ minS;\)\), "\n", 
    \(\(ShowArray[%]\ ;\)\)}], "Input"],

Cell[BoxData[
    \(w\  = \ RandomWord[\ 16, \ 2\ ]\)], "Input"],

Cell["This time the computation has only one strand.", "Text"],

Cell[BoxData[
    \(\(PlotComputationFA[\ sa, w, Full \[Rule] \ False\ ];\)\)], "Input"],

Cell["And the corresponding matrix.", "Text"],

Cell[BoxData[{
    \(\(M\  = 
        S[\([\(Position[\ W, WordReduce[\ \ w, \ eq\ ]\ ]\)[\([1, 
                1]\)]]\)];\)\), "\n", 
    \(\(PlotMatrix[\ \ M, \ GridLines \[Rule] True];\)\)}], "Input"]
}, Closed]]
}, Closed]]
}, Closed]]
},
FrontEndVersion->"5.0 for X",
ScreenRectangle->{{0, 1280}, {0, 1024}},
WindowSize->{1016, 996},
WindowMargins->{{Automatic, 0}, {Automatic, 0}},
Visible->True,
PrintingStartingPageNumber->76,
PrintingPageRange->{Automatic, Automatic},
PrintingOptions->{"PaperSize"->{612, 792},
"PaperOrientation"->"Portrait",
"PostScriptOutputFile":>FrontEnd`FileName[{$RootDirectory, "home", "sutner", \
"projects", "Package"}, "Examples.nb.ps", CharacterEncoding -> "iso8859-1"],
"Magnification"->1},
ShowSelection->True,
CellLabelAutoDelete->True,
InputAliases->{"intt"->RowBox[ {"\[Integral]", 
        RowBox[ {"\[SelectionPlaceholder]", 
          RowBox[ {"\[DifferentialD]", "\[Placeholder]"}]}]}], 
    "dintt"->RowBox[ {
        SubsuperscriptBox[ 
        "\[Integral]", "\[SelectionPlaceholder]", "\[Placeholder]"], 
        RowBox[ {"\[Placeholder]", 
          RowBox[ {"\[DifferentialD]", "\[Placeholder]"}]}]}], 
    "sumt"->RowBox[ {
        UnderoverscriptBox[ "\[Sum]", 
          RowBox[ {"\[SelectionPlaceholder]", "=", "\[Placeholder]"}], 
          "\[Placeholder]"], "\[Placeholder]"}], "prodt"->RowBox[ {
        UnderoverscriptBox[ "\[Product]", 
          RowBox[ {"\[SelectionPlaceholder]", "=", "\[Placeholder]"}], 
          "\[Placeholder]"], "\[Placeholder]"}], "dt"->RowBox[ {
        SubscriptBox[ "\[PartialD]", "\[Placeholder]"], " ", 
        "\[SelectionPlaceholder]"}], "notation"->RowBox[ {"Notation", "[", 
        RowBox[ {
          TagBox[ 
          "\[Placeholder]", NotationBoxTag, TagStyle -> 
            "NotationTemplateStyle"], " ", "\[DoubleLongLeftRightArrow]", " ",
           
          TagBox[ 
          "\[Placeholder]", NotationBoxTag, TagStyle -> 
            "NotationTemplateStyle"]}], "]"}], 
    "notation>"->RowBox[ {"Notation", "[", 
        RowBox[ {
          TagBox[ 
          "\[Placeholder]", NotationBoxTag, TagStyle -> 
            "NotationTemplateStyle"], " ", "\[DoubleLongRightArrow]", " ", 
          TagBox[ 
          "\[Placeholder]", NotationBoxTag, TagStyle -> 
            "NotationTemplateStyle"]}], "]"}], 
    "notation<"->RowBox[ {"Notation", "[", 
        RowBox[ {
          TagBox[ 
          "\[Placeholder]", NotationBoxTag, TagStyle -> 
            "NotationTemplateStyle"], " ", "\[DoubleLongLeftArrow]", " ", 
          TagBox[ 
          "\[Placeholder]", NotationBoxTag, TagStyle -> 
            "NotationTemplateStyle"]}], "]"}], 
    "symb"->RowBox[ {"Symbolize", "[", 
        TagBox[ 
        "\[Placeholder]", NotationBoxTag, TagStyle -> 
          "NotationTemplateStyle"], "]"}], 
    "infixnotation"->RowBox[ {"InfixNotation", "[", 
        RowBox[ {
          TagBox[ 
          "\[Placeholder]", NotationBoxTag, TagStyle -> 
            "NotationTemplateStyle"], ",", "\[Placeholder]"}], "]"}], 
    "addia"->RowBox[ {"AddInputAlias", "[", 
        RowBox[ {"\"\[Placeholder]\"", "\[Rule]", 
          TagBox[ 
          "\[Placeholder]", NotationBoxTag, TagStyle -> 
            "NotationTemplateStyle"]}], "]"}], 
    "pattwraper"->TagBox[ 
      "\[Placeholder]", NotationPatternTag, TagStyle -> 
        "NotationPatternWrapperStyle"], 
    "madeboxeswraper"->TagBox[ 
      "\[Placeholder]", NotationMadeBoxesTag, TagStyle -> 
        "NotationMadeBoxesWrapperStyle"]},
Magnification->1.5,
StyleDefinitions -> "Default.nb"
]

(*******************************************************************
Cached data follows.  If you edit this Notebook file directly, not
using Mathematica, you must remove the line containing CacheID at
the top of  the file.  The cache data will then be recreated when
you save this file from within Mathematica.
*******************************************************************)

(*CellTagsOutline
CellTagsIndex->{
  "c:4"->{
    Cell[19350, 715, 116, 3, 58, "Subsection",
      Evaluatable->False,
      CellTags->"c:4"],
    Cell[37476, 1246, 53, 1, 39, "Subsubsection",
      CellTags->"c:4"],
    Cell[39710, 1320, 53, 1, 39, "Subsubsection",
      CellTags->"c:4"],
    Cell[41401, 1376, 53, 1, 39, "Subsubsection",
      CellTags->"c:4"]},
  "S5.48.1"->{
    Cell[19693, 730, 5352, 94, 1395, "Input",
      CellTags->"S5.48.1"]},
  "c:5"->{
    Cell[25082, 829, 102, 3, 28, "Subsubsection",
      Evaluatable->False,
      CellTags->"c:5"],
    Cell[42905, 1428, 52, 1, 58, "Subsection",
      CellTags->"c:5"]},
  "c:6"->{
    Cell[27158, 900, 115, 3, 39, "Subsubsection",
      Evaluatable->False,
      CellTags->"c:6"]},
  "c:1"->{
    Cell[29911, 992, 70, 1, 111, "Section",
      CellTags->"c:1"],
    Cell[67727, 2254, 62, 1, 111, "Section",
      CellTags->"c:1"],
    Cell[68311, 2275, 66, 1, 58, "Subsection",
      CellTags->"c:1"],
    Cell[71310, 2372, 46, 1, 58, "Subsection",
      CellTags->"c:1"],
    Cell[72423, 2412, 50, 1, 58, "Subsection",
      CellTags->"c:1"],
    Cell[80423, 2676, 71, 1, 111, "Section",
      CellTags->"c:1"],
    Cell[90722, 3045, 76, 1, 111, "Section",
      CellTags->"c:1"],
    Cell[100854, 3400, 61, 1, 111, "Section",
      CellTags->"c:1"]},
  "c:3"->{
    Cell[33039, 1095, 53, 1, 39, "Subsubsection",
      CellTags->"c:3"]},
  "S5.21.1"->{
    Cell[83267, 2781, 62, 1, 38, "Input",
      CellTags->"S5.21.1"]}
  }
*)

(*CellTagsIndex
CellTagsIndex->{
  {"c:4", 116872, 3896},
  {"S5.48.1", 117207, 3906},
  {"c:5", 117293, 3909},
  {"c:6", 117476, 3915},
  {"c:1", 117588, 3919},
  {"c:3", 118158, 3936},
  {"S5.21.1", 118248, 3939}
  }
*)

(*NotebookFileOutline
Notebook[{
Cell[1754, 51, 48, 0, 41, "SmallText"],

Cell[CellGroupData[{
Cell[1827, 55, 34, 0, 145, "Title"],

Cell[CellGroupData[{
Cell[1886, 59, 89, 1, 39, "Subsubsection"],
Cell[1978, 62, 256, 9, 45, "Text"],
Cell[2237, 73, 196, 3, 84, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[2470, 81, 83, 2, 64, "Section",
  Evaluatable->False],

Cell[CellGroupData[{
Cell[2578, 87, 87, 2, 58, "Subsection",
  Evaluatable->False],
Cell[2668, 91, 837, 28, 71, "Text",
  Evaluatable->False],
Cell[3508, 121, 255, 5, 85, "Input"],
Cell[3766, 128, 70, 0, 46, "Text"],
Cell[3839, 130, 104, 2, 39, "Input"],
Cell[3946, 134, 122, 3, 46, "Text"],
Cell[4071, 139, 107, 2, 62, "Input"],
Cell[4181, 143, 61, 1, 39, "Input"],
Cell[4245, 146, 210, 8, 46, "Text"],
Cell[4458, 156, 177, 3, 62, "Input"],
Cell[4638, 161, 151, 5, 46, "Text"],
Cell[4792, 168, 58, 1, 39, "Input"],
Cell[4853, 171, 65, 0, 46, "Text"],
Cell[4921, 173, 141, 2, 62, "Input"],
Cell[5065, 177, 361, 12, 46, "Text"],
Cell[5429, 191, 98, 2, 39, "Input"],
Cell[5530, 195, 319, 11, 71, "Text"],
Cell[5852, 208, 215, 5, 108, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[6104, 218, 90, 2, 58, "Subsection",
  Evaluatable->False],
Cell[6197, 222, 619, 18, 71, "Text"],
Cell[6819, 242, 255, 5, 108, "Input"],
Cell[7077, 249, 104, 2, 39, "Input"],
Cell[7184, 253, 93, 2, 46, "Text",
  Evaluatable->False],
Cell[7280, 257, 104, 2, 39, "Input"],
Cell[7387, 261, 192, 7, 46, "Text",
  Evaluatable->False],
Cell[7582, 270, 94, 2, 39, "Input"],
Cell[7679, 274, 327, 10, 71, "Text",
  Evaluatable->False],
Cell[8009, 286, 90, 1, 39, "Input"],
Cell[8102, 289, 128, 3, 39, "Input"],
Cell[8233, 294, 303, 10, 46, "Text",
  Evaluatable->False],
Cell[8539, 306, 136, 3, 62, "Input"],
Cell[8678, 311, 250, 9, 46, "Text"],
Cell[8931, 322, 179, 4, 62, "Input"],
Cell[9113, 328, 79, 2, 39, "Input"],
Cell[9195, 332, 102, 2, 39, "Input"],
Cell[9300, 336, 48, 0, 46, "Text"],
Cell[9351, 338, 168, 3, 62, "Input"],
Cell[9522, 343, 42, 1, 39, "Input"],
Cell[9567, 346, 140, 5, 46, "Text"],
Cell[9710, 353, 67, 1, 39, "Input"],
Cell[9780, 356, 183, 4, 62, "Input"],
Cell[9966, 362, 68, 0, 46, "Text"],
Cell[10037, 364, 46, 1, 39, "Input"],
Cell[10086, 367, 323, 8, 121, "Text"],
Cell[10412, 377, 172, 3, 62, "Input"],
Cell[10587, 382, 104, 2, 39, "Input"],
Cell[10694, 386, 280, 8, 46, "Text",
  Evaluatable->False],
Cell[10977, 396, 94, 2, 39, "Input"],
Cell[11074, 400, 199, 4, 62, "Input"],
Cell[11276, 406, 112, 3, 46, "Text"],
Cell[11391, 411, 179, 4, 62, "Input"],
Cell[11573, 417, 79, 2, 39, "Input"],
Cell[11655, 421, 102, 2, 39, "Input"],
Cell[11760, 425, 46, 1, 39, "Input"],
Cell[11809, 428, 484, 14, 96, "Text"],
Cell[12296, 444, 67, 1, 39, "Input"],
Cell[12366, 447, 183, 4, 62, "Input"],
Cell[12552, 453, 136, 3, 39, "Input"],
Cell[12691, 458, 177, 6, 46, "Text"],
Cell[12871, 466, 67, 1, 39, "Input"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[12987, 473, 86, 2, 64, "Section",
  Evaluatable->False],

Cell[CellGroupData[{
Cell[13098, 479, 84, 2, 58, "Subsection",
  Evaluatable->False],
Cell[13185, 483, 413, 14, 71, "Text",
  Evaluatable->False],
Cell[13601, 499, 208, 4, 85, "Input"],
Cell[13812, 505, 89, 1, 39, "Input"],
Cell[13904, 508, 436, 12, 96, "Text",
  Evaluatable->False],
Cell[14343, 522, 103, 2, 39, "Input"],
Cell[14449, 526, 115, 2, 39, "Input"],
Cell[14567, 530, 224, 8, 46, "Text",
  Evaluatable->False],
Cell[14794, 540, 83, 2, 39, "Input"],
Cell[14880, 544, 109, 2, 39, "Input"],
Cell[14992, 548, 336, 10, 71, "Text",
  Evaluatable->False],
Cell[15331, 560, 230, 4, 85, "Input"],
Cell[15564, 566, 265, 10, 46, "Text",
  Evaluatable->False],
Cell[15832, 578, 129, 3, 39, "Input"],
Cell[15964, 583, 155, 5, 46, "Text",
  Evaluatable->False],
Cell[16122, 590, 170, 3, 85, "Input"],
Cell[16295, 595, 54, 1, 39, "Input"],
Cell[16352, 598, 22, 0, 46, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[16411, 603, 90, 2, 58, "Subsection",
  Evaluatable->False],
Cell[16504, 607, 262, 7, 71, "Text"],
Cell[16769, 616, 75, 1, 39, "Input"],
Cell[16847, 619, 287, 6, 108, "Input"],
Cell[17137, 627, 79, 2, 39, "Input"],
Cell[17219, 631, 86, 2, 39, "Input"],
Cell[17308, 635, 86, 2, 39, "Input"],
Cell[17397, 639, 80, 2, 39, "Input"],
Cell[17480, 643, 86, 2, 39, "Input"],
Cell[17569, 647, 80, 2, 39, "Input"],
Cell[17652, 651, 36, 1, 39, "Input"],
Cell[17691, 654, 146, 3, 39, "Input"],
Cell[17840, 659, 68, 1, 39, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[17945, 665, 146, 6, 58, "Subsection",
  Evaluatable->False],
Cell[18094, 673, 496, 16, 71, "Text",
  Evaluatable->False],
Cell[18593, 691, 268, 6, 108, "Input"],
Cell[18864, 699, 243, 5, 108, "Input"],
Cell[19110, 706, 203, 4, 108, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[19350, 715, 116, 3, 58, "Subsection",
  Evaluatable->False,
  CellTags->"c:4"],
Cell[19469, 720, 167, 4, 71, "Text"],

Cell[CellGroupData[{
Cell[19661, 728, 29, 0, 39, "Subsubsection"],
Cell[19693, 730, 5352, 94, 1395, "Input",
  CellTags->"S5.48.1"]
}, Closed]],

Cell[CellGroupData[{
Cell[25082, 829, 102, 3, 28, "Subsubsection",
  Evaluatable->False,
  CellTags->"c:5"],
Cell[25187, 834, 239, 5, 71, "Text"],
Cell[25429, 841, 93, 2, 39, "Input"],
Cell[25525, 845, 151, 3, 62, "Input"],
Cell[25679, 850, 133, 3, 39, "Input"],
Cell[25815, 855, 156, 3, 62, "Input"],
Cell[25974, 860, 137, 3, 39, "Input"],
Cell[26114, 865, 174, 3, 62, "Input"],
Cell[26291, 870, 175, 4, 71, "Text"],
Cell[26469, 876, 189, 3, 62, "Input"],
Cell[26661, 881, 229, 5, 62, "Input"],
Cell[26893, 888, 50, 1, 39, "Input"],
Cell[26946, 891, 175, 4, 85, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[27158, 900, 115, 3, 39, "Subsubsection",
  Evaluatable->False,
  CellTags->"c:6"],
Cell[27276, 905, 390, 8, 96, "Text"],
Cell[27669, 915, 243, 7, 62, "Input"],
Cell[27915, 924, 152, 4, 46, "Text"],
Cell[28070, 930, 285, 6, 131, "Input"],
Cell[28358, 938, 151, 3, 71, "Text"],
Cell[28512, 943, 144, 3, 62, "Input"],
Cell[28659, 948, 208, 3, 85, "Input"],
Cell[28870, 953, 211, 6, 71, "Text"],
Cell[29084, 961, 187, 3, 62, "Input"],
Cell[29274, 966, 37, 0, 46, "Text"],
Cell[29314, 968, 169, 3, 85, "Input"],
Cell[29486, 973, 119, 3, 46, "Text"],
Cell[29608, 978, 217, 5, 62, "Input"],
Cell[29828, 985, 22, 0, 46, "Text"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[29911, 992, 70, 1, 111, "Section",
  CellTags->"c:1"],
Cell[29984, 995, 455, 13, 146, "Text"],
Cell[30442, 1010, 58, 1, 39, "Input"],

Cell[CellGroupData[{
Cell[30525, 1015, 28, 0, 58, "Subsection"],
Cell[30556, 1017, 2458, 74, 421, "Text"],

Cell[CellGroupData[{
Cell[33039, 1095, 53, 1, 39, "Subsubsection",
  CellTags->"c:3"],
Cell[33095, 1098, 720, 22, 118, "Text"],
Cell[33818, 1122, 201, 4, 62, "Input"],
Cell[34022, 1128, 200, 4, 62, "Input"],
Cell[34225, 1134, 217, 6, 71, "Text"],
Cell[34445, 1142, 319, 7, 131, "Input"],
Cell[34767, 1151, 471, 13, 71, "Text"],
Cell[35241, 1166, 380, 7, 131, "Input"],
Cell[35624, 1175, 72, 0, 46, "Text"],
Cell[35699, 1177, 167, 3, 62, "Input"],
Cell[35869, 1182, 48, 0, 46, "Text"],
Cell[35920, 1184, 57, 1, 39, "Input"],
Cell[35980, 1187, 108, 5, 46, "Text"],
Cell[36091, 1194, 57, 1, 39, "Input"],
Cell[36151, 1197, 369, 13, 46, "Text"],
Cell[36523, 1212, 86, 1, 39, "Input"],
Cell[36612, 1215, 236, 6, 71, "Text"],
Cell[36851, 1223, 109, 2, 39, "Input"],
Cell[36963, 1227, 63, 0, 46, "Text"],
Cell[37029, 1229, 175, 3, 85, "Input"],
Cell[37207, 1234, 83, 1, 39, "Input"],
Cell[37293, 1237, 121, 2, 39, "Input"],
Cell[37417, 1241, 22, 0, 46, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[37476, 1246, 53, 1, 39, "Subsubsection",
  CellTags->"c:4"],
Cell[37532, 1249, 389, 12, 96, "Text"],
Cell[37924, 1263, 208, 5, 62, "Input"],
Cell[38135, 1270, 231, 4, 108, "Input"],
Cell[38369, 1276, 305, 5, 108, "Input"],
Cell[38677, 1283, 56, 0, 46, "Text"],
Cell[38736, 1285, 207, 4, 85, "Input"],
Cell[38946, 1291, 68, 0, 46, "Text"],
Cell[39017, 1293, 79, 1, 39, "Input"],
Cell[39099, 1296, 52, 1, 39, "Input"],
Cell[39154, 1299, 220, 3, 85, "Input"],
Cell[39377, 1304, 118, 5, 46, "Text"],
Cell[39498, 1311, 175, 4, 62, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[39710, 1320, 53, 1, 39, "Subsubsection",
  CellTags->"c:4"],
Cell[39766, 1323, 209, 5, 65, "Text"],
Cell[39978, 1330, 79, 1, 39, "Input"],
Cell[40060, 1333, 158, 3, 85, "Input"],
Cell[40221, 1338, 60, 0, 46, "Text"],
Cell[40284, 1340, 190, 4, 85, "Input"],
Cell[40477, 1346, 282, 4, 108, "Input"],
Cell[40762, 1352, 185, 5, 46, "Text"],
Cell[40950, 1359, 116, 2, 62, "Input"],
Cell[41069, 1363, 295, 8, 46, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[41401, 1376, 53, 1, 39, "Subsubsection",
  CellTags->"c:4"],
Cell[41457, 1379, 261, 8, 46, "Text"],
Cell[41721, 1389, 84, 1, 39, "Input"],
Cell[41808, 1392, 594, 12, 223, "Input"],
Cell[42405, 1406, 180, 5, 46, "Text"],
Cell[42588, 1413, 116, 2, 62, "Input"],
Cell[42707, 1417, 149, 5, 46, "Text"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[42905, 1428, 52, 1, 58, "Subsection",
  CellTags->"c:5"],
Cell[42960, 1431, 746, 21, 143, "Text"],
Cell[43709, 1454, 275, 5, 85, "Input"],
Cell[43987, 1461, 60, 0, 46, "Text"],
Cell[44050, 1463, 297, 6, 131, "Input"],
Cell[44350, 1471, 52, 0, 46, "Text"],
Cell[44405, 1473, 295, 6, 85, "Input"],
Cell[44703, 1481, 68, 0, 46, "Text"],
Cell[44774, 1483, 177, 3, 85, "Input"],
Cell[44954, 1488, 56, 0, 46, "Text"],
Cell[45013, 1490, 57, 1, 39, "Input"],
Cell[45073, 1493, 127, 5, 46, "Text"],
Cell[45203, 1500, 84, 1, 39, "Input"],
Cell[45290, 1503, 58, 1, 39, "Input"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[45397, 1510, 80, 2, 64, "Section",
  Evaluatable->False],
Cell[45480, 1514, 1179, 34, 220, "Text"],
Cell[46662, 1550, 281, 6, 130, "Input"],
Cell[46946, 1558, 141, 5, 45, "Text"],
Cell[47090, 1565, 176, 3, 61, "Input"],
Cell[47269, 1570, 159, 5, 45, "Text"],
Cell[47431, 1577, 286, 6, 107, "Input"],
Cell[47720, 1585, 60, 0, 45, "Text"],
Cell[47783, 1587, 99, 2, 38, "Input"],
Cell[47885, 1591, 108, 2, 61, "Input"],
Cell[47996, 1595, 732, 22, 95, "Text"],
Cell[48731, 1619, 164, 3, 61, "Input"],
Cell[48898, 1624, 913, 29, 95, "Text"],
Cell[49814, 1655, 146, 3, 61, "Input"],
Cell[49963, 1660, 115, 2, 38, "Input"],
Cell[50081, 1664, 149, 3, 38, "Input"],
Cell[50233, 1669, 1319, 40, 120, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[51589, 1714, 74, 2, 64, "Section",
  Evaluatable->False],
Cell[51666, 1718, 996, 33, 121, "Text",
  Evaluatable->False],
Cell[52665, 1753, 151, 3, 62, "Input"],
Cell[52819, 1758, 158, 5, 46, "Text"],
Cell[52980, 1765, 167, 3, 85, "Input"],
Cell[53150, 1770, 142, 5, 46, "Text"],
Cell[53295, 1777, 121, 2, 62, "Input"],
Cell[53419, 1781, 443, 11, 121, "Text"],
Cell[53865, 1794, 376, 6, 108, "Input"],
Cell[54244, 1802, 131, 3, 46, "Text"],
Cell[54378, 1807, 235, 5, 62, "Input"],
Cell[54616, 1814, 181, 5, 62, "Input"],
Cell[54800, 1821, 105, 2, 39, "Input"],
Cell[54908, 1825, 61, 1, 39, "Input"],
Cell[54972, 1828, 90, 3, 46, "Text"],
Cell[55065, 1833, 1607, 30, 522, "Input"],
Cell[56675, 1865, 216, 8, 46, "Text"],
Cell[56894, 1875, 85, 1, 39, "Input"],
Cell[56982, 1878, 60, 1, 39, "Input"],
Cell[57045, 1881, 146, 3, 71, "Text"],
Cell[57194, 1886, 204, 4, 85, "Input"],
Cell[57401, 1892, 192, 8, 46, "Text"],
Cell[57596, 1902, 85, 1, 39, "Input"],
Cell[57684, 1905, 60, 1, 39, "Input"],
Cell[57747, 1908, 221, 8, 46, "Text"],
Cell[57971, 1918, 75, 1, 39, "Input"],
Cell[58049, 1921, 448, 15, 46, "Text",
  Evaluatable->False]
}, Closed]],

Cell[CellGroupData[{
Cell[58534, 1941, 85, 2, 111, "Section",
  Evaluatable->False],

Cell[CellGroupData[{
Cell[58644, 1947, 38, 0, 39, "Subsubsection"],
Cell[58685, 1949, 295, 7, 71, "Text"],
Cell[58983, 1958, 150, 3, 85, "Input"],
Cell[59136, 1963, 346, 9, 71, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[59519, 1977, 35, 0, 39, "Subsubsection"],
Cell[59557, 1979, 346, 9, 71, "Text"],
Cell[59906, 1990, 878, 16, 292, "Input"],
Cell[60787, 2008, 68, 0, 46, "Text"],
Cell[60858, 2010, 114, 2, 62, "Input"],
Cell[60975, 2014, 98, 3, 46, "Text"],
Cell[61076, 2019, 93, 1, 39, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[61206, 2025, 47, 0, 39, "Subsubsection"],
Cell[61256, 2027, 434, 13, 71, "Text"],
Cell[61693, 2042, 74, 1, 39, "Input"],
Cell[61770, 2045, 296, 7, 71, "Text"],
Cell[62069, 2054, 239, 5, 85, "Input"],
Cell[62311, 2061, 38, 0, 46, "Text"],
Cell[62352, 2063, 118, 3, 85, "Input"],
Cell[62473, 2068, 36, 0, 46, "Text"],
Cell[62512, 2070, 148, 3, 62, "Input"],
Cell[62663, 2075, 79, 1, 39, "Input"],
Cell[62745, 2078, 628, 14, 96, "Text"],
Cell[63376, 2094, 385, 8, 108, "Input"],
Cell[63764, 2104, 50, 0, 46, "Text"],
Cell[63817, 2106, 138, 2, 62, "Input"],
Cell[63958, 2110, 79, 1, 39, "Input"],
Cell[64040, 2113, 28, 0, 46, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[64105, 2118, 46, 0, 39, "Subsubsection"],
Cell[64154, 2120, 116, 3, 46, "Text"],
Cell[64273, 2125, 158, 3, 62, "Input"],
Cell[64434, 2130, 46, 0, 46, "Text"],
Cell[64483, 2132, 55, 1, 39, "Input"],
Cell[64541, 2135, 56, 0, 46, "Text"],
Cell[64600, 2137, 110, 2, 62, "Input"],
Cell[64713, 2141, 224, 8, 46, "Text"],
Cell[64940, 2151, 149, 3, 62, "Input"],
Cell[65092, 2156, 305, 11, 46, "Text"],
Cell[65400, 2169, 49, 1, 39, "Input"],
Cell[65452, 2172, 421, 11, 71, "Text"],
Cell[65876, 2185, 151, 3, 62, "Input"],
Cell[66030, 2190, 74, 0, 46, "Text"],
Cell[66107, 2192, 84, 1, 39, "Input"],
Cell[66194, 2195, 355, 12, 71, "Text"],
Cell[66552, 2209, 101, 2, 39, "Input"],
Cell[66656, 2213, 137, 3, 62, "Input"],
Cell[66796, 2218, 382, 14, 46, "Text"],
Cell[67181, 2234, 181, 4, 85, "Input"],
Cell[67365, 2240, 155, 3, 62, "Input"],
Cell[67523, 2245, 155, 3, 62, "Input"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[67727, 2254, 62, 1, 111, "Section",
  CellTags->"c:1"],
Cell[67792, 2257, 385, 8, 121, "Text"],
Cell[68180, 2267, 57, 1, 39, "Input"],
Cell[68240, 2270, 46, 1, 39, "Input"],

Cell[CellGroupData[{
Cell[68311, 2275, 66, 1, 58, "Subsection",
  CellTags->"c:1"],
Cell[68380, 2278, 448, 15, 96, "Text"],
Cell[68831, 2295, 332, 6, 131, "Input"],
Cell[69166, 2303, 95, 3, 46, "Text"],
Cell[69264, 2308, 262, 5, 108, "Input"],
Cell[69529, 2315, 280, 8, 71, "Text"],
Cell[69812, 2325, 698, 12, 246, "Input"],
Cell[70513, 2339, 152, 5, 46, "Text"],
Cell[70668, 2346, 230, 5, 85, "Input"],
Cell[70901, 2353, 59, 0, 46, "Text"],
Cell[70963, 2355, 79, 1, 39, "Input"],
Cell[71045, 2358, 38, 0, 46, "Text"],
Cell[71086, 2360, 50, 1, 39, "Input"],
Cell[71139, 2363, 57, 1, 39, "Input"],
Cell[71199, 2366, 74, 1, 39, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[71310, 2372, 46, 1, 58, "Subsection",
  CellTags->"c:1"],
Cell[71359, 2375, 387, 10, 96, "Text"],
Cell[71749, 2387, 319, 6, 131, "Input"],
Cell[72071, 2395, 73, 1, 39, "Input"],
Cell[72147, 2398, 62, 1, 39, "Input"],
Cell[72212, 2401, 93, 3, 46, "Text"],
Cell[72308, 2406, 78, 1, 39, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[72423, 2412, 50, 1, 58, "Subsection",
  CellTags->"c:1"],
Cell[72476, 2415, 242, 8, 71, "Text"],
Cell[72721, 2425, 597, 10, 246, "Input"],
Cell[73321, 2437, 64, 0, 46, "Text"],
Cell[73388, 2439, 87, 1, 39, "Input"],
Cell[73478, 2442, 68, 0, 46, "Text"],
Cell[73549, 2444, 66, 1, 39, "Input"],
Cell[73618, 2447, 87, 3, 46, "Text"],
Cell[73708, 2452, 102, 2, 39, "Input"],
Cell[73813, 2456, 77, 1, 39, "Input"],
Cell[73893, 2459, 76, 2, 46, "Text"],
Cell[73972, 2463, 52, 1, 39, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[74061, 2469, 46, 0, 58, "Subsection"],
Cell[74110, 2471, 684, 20, 171, "Text"],
Cell[74797, 2493, 291, 5, 85, "Input"],
Cell[75091, 2500, 146, 5, 46, "Text"],
Cell[75240, 2507, 90, 1, 39, "Input"],
Cell[75333, 2510, 138, 2, 62, "Input"],
Cell[75474, 2514, 355, 11, 71, "Text"],
Cell[75832, 2527, 295, 5, 85, "Input"],
Cell[76130, 2534, 141, 5, 46, "Text"],
Cell[76274, 2541, 190, 3, 85, "Input"],
Cell[76467, 2546, 74, 0, 46, "Text"],
Cell[76544, 2548, 195, 3, 85, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[76776, 2556, 46, 0, 58, "Subsection"],
Cell[76825, 2558, 771, 20, 146, "Text"],
Cell[77599, 2580, 62, 1, 39, "Input"],
Cell[77664, 2583, 142, 5, 46, "Text"],
Cell[77809, 2590, 71, 1, 39, "Input"],
Cell[77883, 2593, 74, 1, 39, "Input"],
Cell[77960, 2596, 196, 6, 46, "Text"],
Cell[78159, 2604, 71, 1, 39, "Input"],
Cell[78233, 2607, 126, 2, 62, "Input"],
Cell[78362, 2611, 104, 2, 39, "Input"],
Cell[78469, 2615, 280, 6, 71, "Text"],
Cell[78752, 2623, 743, 16, 246, "Input"],
Cell[79498, 2641, 54, 1, 39, "Input"],
Cell[79555, 2644, 201, 4, 71, "Text"],
Cell[79759, 2650, 110, 2, 39, "Input"],
Cell[79872, 2654, 104, 2, 39, "Input"],
Cell[79979, 2658, 104, 2, 39, "Input"],
Cell[80086, 2662, 173, 4, 71, "Text"],
Cell[80262, 2668, 112, 2, 39, "Input"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[80423, 2676, 71, 1, 111, "Section",
  CellTags->"c:1"],

Cell[CellGroupData[{
Cell[80519, 2681, 28, 0, 58, "Subsection"],
Cell[80550, 2683, 181, 5, 46, "Text",
  Evaluatable->False],
Cell[80734, 2690, 52, 1, 39, "Input"],
Cell[80789, 2693, 62, 1, 39, "Input"],
Cell[80854, 2696, 113, 2, 39, "Input"],
Cell[80970, 2700, 216, 6, 71, "Text",
  Evaluatable->False],
Cell[81189, 2708, 65, 1, 39, "Input"],
Cell[81257, 2711, 859, 22, 121, "Text"],
Cell[82119, 2735, 226, 5, 85, "Input"],
Cell[82348, 2742, 306, 11, 46, "Text"],
Cell[82657, 2755, 79, 1, 39, "Input"],
Cell[82739, 2758, 204, 6, 46, "Text"],
Cell[82946, 2766, 63, 1, 39, "Input"],
Cell[83012, 2769, 62, 0, 46, "Text"],
Cell[83077, 2771, 54, 1, 39, "Input"],
Cell[83134, 2774, 130, 5, 46, "Text"],
Cell[83267, 2781, 62, 1, 38, "Input",
  CellTags->"S5.21.1"],
Cell[83332, 2784, 113, 2, 62, "Input"],
Cell[83448, 2788, 167, 6, 46, "Text"],
Cell[83618, 2796, 124, 3, 39, "Input"],
Cell[83745, 2801, 152, 5, 46, "Text"],
Cell[83900, 2808, 86, 1, 39, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[84023, 2814, 33, 0, 58, "Subsection"],
Cell[84059, 2816, 404, 12, 71, "Text",
  Evaluatable->False],
Cell[84466, 2830, 117, 3, 62, "Input"],
Cell[84586, 2835, 274, 9, 46, "Text",
  Evaluatable->False],
Cell[84863, 2846, 198, 4, 85, "Input"],
Cell[85064, 2852, 186, 4, 46, "Text",
  Evaluatable->False],
Cell[85253, 2858, 140, 3, 39, "Input"],
Cell[85396, 2863, 419, 11, 96, "Text",
  Evaluatable->False],
Cell[85818, 2876, 230, 5, 62, "Input"],
Cell[86051, 2883, 146, 5, 46, "Text"],
Cell[86200, 2890, 230, 5, 62, "Input"],
Cell[86433, 2897, 197, 6, 46, "Text"],
Cell[86633, 2905, 176, 4, 62, "Input"],
Cell[86812, 2911, 324, 7, 85, "Input"],
Cell[87139, 2920, 109, 2, 46, "Text",
  Evaluatable->False],
Cell[87251, 2924, 129, 3, 39, "Input"],
Cell[87383, 2929, 292, 5, 85, "Input"],
Cell[87678, 2936, 115, 2, 46, "Text",
  Evaluatable->False],
Cell[87796, 2940, 293, 5, 85, "Input"],
Cell[88092, 2947, 169, 5, 46, "Text",
  Evaluatable->False],
Cell[88264, 2954, 308, 6, 85, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[88609, 2965, 33, 0, 58, "Subsection"],
Cell[88645, 2967, 289, 11, 46, "Text",
  Evaluatable->False],
Cell[88937, 2980, 117, 3, 62, "Input"],
Cell[89057, 2985, 232, 5, 85, "Input"],
Cell[89292, 2992, 186, 4, 46, "Text",
  Evaluatable->False],
Cell[89481, 2998, 140, 3, 39, "Input"],
Cell[89624, 3003, 200, 5, 71, "Text",
  Evaluatable->False],
Cell[89827, 3010, 67, 1, 39, "Input"],
Cell[89897, 3013, 191, 5, 46, "Text",
  Evaluatable->False],
Cell[90091, 3020, 153, 3, 62, "Input"],
Cell[90247, 3025, 69, 1, 39, "Input"],
Cell[90319, 3028, 121, 4, 46, "Text"],
Cell[90443, 3034, 230, 5, 62, "Input"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[90722, 3045, 76, 1, 111, "Section",
  CellTags->"c:1"],
Cell[90801, 3048, 42, 0, 46, "Text"],
Cell[90846, 3050, 58, 1, 39, "Input"],

Cell[CellGroupData[{
Cell[90929, 3055, 64, 0, 58, "Subsection"],
Cell[90996, 3057, 629, 16, 96, "Text"],
Cell[91628, 3075, 54, 1, 39, "Input"],
Cell[91685, 3078, 65, 2, 46, "Text"],
Cell[91753, 3082, 55, 1, 39, "Input"],
Cell[91811, 3085, 114, 3, 46, "Text"],
Cell[91928, 3090, 181, 4, 39, "Input"],
Cell[92112, 3096, 108, 2, 39, "Input"],
Cell[92223, 3100, 95, 3, 46, "Text"],
Cell[92321, 3105, 54, 1, 39, "Input"],
Cell[92378, 3108, 60, 1, 39, "Input"],
Cell[92441, 3111, 101, 3, 46, "Text"],
Cell[92545, 3116, 60, 1, 39, "Input"],
Cell[92608, 3119, 61, 1, 39, "Input"],
Cell[92672, 3122, 47, 1, 39, "Input"],
Cell[92722, 3125, 110, 3, 46, "Text"],
Cell[92835, 3130, 101, 2, 62, "Input"],
Cell[92939, 3134, 410, 8, 96, "Text"],
Cell[93352, 3144, 53, 1, 39, "Input"],
Cell[93408, 3147, 58, 1, 39, "Input"],
Cell[93469, 3150, 55, 1, 39, "Input"],
Cell[93527, 3153, 134, 3, 46, "Text"],
Cell[93664, 3158, 254, 5, 108, "Input"],
Cell[93921, 3165, 98, 3, 46, "Text"],
Cell[94022, 3170, 73, 1, 39, "Input"],
Cell[94098, 3173, 160, 4, 62, "Input"],
Cell[94261, 3179, 185, 4, 71, "Text"],
Cell[94449, 3185, 148, 3, 62, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[94634, 3193, 28, 0, 58, "Subsection"],
Cell[94665, 3195, 132, 3, 46, "Text"],
Cell[94800, 3200, 57, 1, 39, "Input"],
Cell[94860, 3203, 322, 7, 71, "Text"],
Cell[95185, 3212, 205, 5, 62, "Input"],
Cell[95393, 3219, 80, 1, 39, "Input"],
Cell[95476, 3222, 102, 3, 46, "Text"],
Cell[95581, 3227, 146, 3, 62, "Input"],
Cell[95730, 3232, 66, 0, 46, "Text"],
Cell[95799, 3234, 100, 2, 62, "Input"],
Cell[95902, 3238, 127, 3, 46, "Text"],
Cell[96032, 3243, 160, 3, 39, "Input"],
Cell[96195, 3248, 98, 3, 46, "Text"],
Cell[96296, 3253, 206, 5, 62, "Input"],
Cell[96505, 3260, 80, 1, 39, "Input"],
Cell[96588, 3263, 404, 12, 71, "Text"],
Cell[96995, 3277, 100, 2, 62, "Input"],
Cell[97098, 3281, 173, 3, 62, "Input"],
Cell[97274, 3286, 162, 4, 71, "Text"],
Cell[97439, 3292, 205, 5, 62, "Input"],
Cell[97647, 3299, 80, 1, 39, "Input"],
Cell[97730, 3302, 54, 1, 39, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[97821, 3308, 38, 0, 58, "Subsection"],
Cell[97862, 3310, 364, 8, 96, "Text"],
Cell[98229, 3320, 305, 6, 108, "Input"],
Cell[98537, 3328, 130, 3, 85, "Input"],
Cell[98670, 3333, 62, 0, 46, "Text"],
Cell[98735, 3335, 192, 3, 62, "Input"],
Cell[98930, 3340, 305, 7, 71, "Text"],
Cell[99238, 3349, 254, 5, 62, "Input"],
Cell[99495, 3356, 510, 13, 96, "Text"],
Cell[100008, 3371, 367, 7, 131, "Input"],
Cell[100378, 3380, 181, 3, 62, "Input"],
Cell[100562, 3385, 88, 3, 46, "Text"],
Cell[100653, 3390, 76, 1, 39, "Input"],
Cell[100732, 3393, 73, 1, 39, "Input"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[100854, 3400, 61, 1, 111, "Section",
  CellTags->"c:1"],

Cell[CellGroupData[{
Cell[100940, 3405, 44, 0, 58, "Subsection"],
Cell[100987, 3407, 66, 0, 46, "Text"],
Cell[101056, 3409, 113, 2, 62, "Input"],
Cell[101172, 3413, 94, 3, 46, "Text"],
Cell[101269, 3418, 52, 1, 39, "Input"],
Cell[101324, 3421, 109, 2, 39, "Input"],
Cell[101436, 3425, 255, 9, 46, "Text"],
Cell[101694, 3436, 119, 2, 62, "Input"],
Cell[101816, 3440, 539, 12, 96, "Text"],
Cell[102358, 3454, 228, 4, 85, "Input"],
Cell[102589, 3460, 48, 1, 39, "Input"],
Cell[102640, 3463, 107, 3, 46, "Text"],
Cell[102750, 3468, 289, 6, 108, "Input"],
Cell[103042, 3476, 88, 3, 46, "Text"],
Cell[103133, 3481, 294, 5, 108, "Input"],
Cell[103430, 3488, 143, 3, 71, "Text"],
Cell[103576, 3493, 193, 3, 62, "Input"],
Cell[103772, 3498, 71, 0, 46, "Text"],
Cell[103846, 3500, 56, 1, 39, "Input"],
Cell[103905, 3503, 43, 0, 46, "Text"],
Cell[103951, 3505, 92, 1, 39, "Input"],
Cell[104046, 3508, 280, 9, 46, "Text"],
Cell[104329, 3519, 244, 4, 108, "Input"],
Cell[104576, 3525, 114, 3, 46, "Text"],
Cell[104693, 3530, 88, 1, 39, "Input"],
Cell[104784, 3533, 391, 8, 96, "Text"],
Cell[105178, 3543, 80, 1, 39, "Input"],
Cell[105261, 3546, 157, 5, 46, "Text"],
Cell[105421, 3553, 131, 3, 39, "Input"],
Cell[105555, 3558, 248, 6, 71, "Text"],
Cell[105806, 3566, 148, 3, 85, "Input"],
Cell[105957, 3571, 48, 1, 39, "Input"],
Cell[106008, 3574, 128, 3, 46, "Text"],
Cell[106139, 3579, 340, 7, 85, "Input"],
Cell[106482, 3588, 87, 1, 39, "Input"],
Cell[106572, 3591, 91, 1, 39, "Input"],
Cell[106666, 3594, 89, 1, 39, "Input"],
Cell[106758, 3597, 45, 0, 46, "Text"],
Cell[106806, 3599, 205, 4, 62, "Input"],
Cell[107014, 3605, 183, 6, 46, "Text"],
Cell[107200, 3613, 276, 7, 108, "Input"],
Cell[107479, 3622, 223, 4, 62, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[107739, 3631, 38, 0, 58, "Subsection"],
Cell[107780, 3633, 130, 3, 46, "Text"],
Cell[107913, 3638, 116, 2, 62, "Input"],
Cell[108032, 3642, 49, 1, 39, "Input"],
Cell[108084, 3645, 419, 11, 71, "Text"],
Cell[108506, 3658, 283, 5, 108, "Input"],
Cell[108792, 3665, 48, 1, 39, "Input"],
Cell[108843, 3668, 50, 0, 46, "Text"],
Cell[108896, 3670, 198, 4, 62, "Input"],
Cell[109097, 3676, 46, 0, 46, "Text"],
Cell[109146, 3678, 187, 3, 62, "Input"],
Cell[109336, 3683, 58, 0, 46, "Text"],
Cell[109397, 3685, 214, 4, 62, "Input"],
Cell[109614, 3691, 71, 0, 46, "Text"],
Cell[109688, 3693, 64, 1, 39, "Input"],
Cell[109755, 3696, 88, 1, 39, "Input"],
Cell[109846, 3699, 45, 0, 46, "Text"],
Cell[109894, 3701, 204, 4, 62, "Input"],
Cell[110101, 3707, 183, 6, 46, "Text"],
Cell[110287, 3715, 276, 7, 108, "Input"],
Cell[110566, 3724, 215, 4, 62, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[110818, 3733, 43, 0, 58, "Subsection"],
Cell[110864, 3735, 74, 0, 46, "Text"],
Cell[110941, 3737, 118, 2, 62, "Input"],
Cell[111062, 3741, 49, 1, 39, "Input"],
Cell[111114, 3744, 418, 11, 71, "Text"],
Cell[111535, 3757, 283, 5, 108, "Input"],
Cell[111821, 3764, 48, 1, 39, "Input"],
Cell[111872, 3767, 60, 0, 46, "Text"],
Cell[111935, 3769, 289, 6, 108, "Input"],
Cell[112227, 3777, 88, 3, 46, "Text"],
Cell[112318, 3782, 285, 5, 108, "Input"],
Cell[112606, 3789, 64, 1, 39, "Input"],
Cell[112673, 3792, 62, 0, 46, "Text"],
Cell[112738, 3794, 88, 1, 39, "Input"],
Cell[112829, 3797, 45, 0, 46, "Text"],
Cell[112877, 3799, 204, 4, 62, "Input"]
}, Closed]]
}, Closed]]
}, Closed]]
}
]
*)



(*******************************************************************
End of Mathematica Notebook file.
*******************************************************************)

