(************** Content-type: application/mathematica **************
                     CreatedBy='Mathematica 5.0'

                    Mathematica-Compatible Notebook

This notebook can be used with any Mathematica-compatible
application, such as Mathematica, MathReader or Publicon. The data
for the notebook starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do
one of the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the
  application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing
the word CacheID, otherwise Mathematica-compatible applications may
try to use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
*******************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     80448,       2581]*)
(*NotebookOutlinePosition[     81751,       2622]*)
(*  CellTagsIndexPosition[     81707,       2618]*)
(*WindowFrame->Normal*)



Notebook[{
Cell["\[Copyright] 2004 K. Sutner", "SmallText"],

Cell[CellGroupData[{

Cell["Chessboards, Pebbles, Kronecker, and Fibonacci", "Title",
  Evaluatable->False,
  AspectRatioFixed->True,
  FontFamily->"Charter"],

Cell[TextData[{
  "Klaus Sutner\n",
  ButtonBox["www.cs.cmu.edu/~sutner",
    ButtonData:>{
      URL[ "http://www.cs.cmu.edu/~sutner"], None},
    ButtonStyle->"Hyperlink"]
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["Load Packages", "Subsubsection"],

Cell[TextData[{
  "You have to install the  ",
  StyleBox["Automata", "MR"],
  "  package in order to run the commands in this notebook. You will also \
need the data files ",
  StyleBox["rho-factors.dat", "MR"],
  " and ",
  StyleBox["corank-400.dat", "MR"],
  ". Place them into the same directory as this notebook."
}], "Text"],

Cell[BoxData[{
    \(Get["\<Automata`fibocode`\>"]\), "\n", 
    \(FiboLoadData[]\)}], "Input",
  CellLabel->"In[900]:="]
}, Open  ]],

Cell[CellGroupData[{

Cell["Chessboards and Pebbles", "Section",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell[" A Pebbling Game", "Subsection"],

Cell["\<\
Suppose we have a chessboard and a collection of two-colored \
pebbles, black on one side and white on the other.  We place a single pebble \
black side up on each square of the board.  The goal is to flip all the \
pebbles over so that they wind up white side up.  The catch is that whenever \
we flip a pebble, we also have to flip all the pebbles on the four \
neighboring squares. A little experimentation shows that this problem is \
rather easily solved for smaller boards of size up to 4 or 5.  For the \
original 8 by 8 chessboard, however, it is by no means clear that there even \
is a solution to this problem. A hint for those who have not yet computed the \
solution using the machinery below: for an 8 by 8 chessboard there is only \
one solution (disregarding order), and the chessboard has lots of symmetries. \
\
\>", "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["\<\
Of course, this game is just a thinly veiled excuse to study a \
class of additive cellular automata.  See  
\t\[FilledSmallCircle] K. Sutner, \"Decomposition of Additive CA\", Complex \
Systems 13 (2001) 2, 245--270  and 
\t\[FilledSmallCircle] K. Sutner, \"\[Sigma]-Automata and \
Chebyshev-Polynomials\",  Theoretical Computer Science, 230 (2000) 49--73
for more background information, references and proofs.  Here we only present \
computations. \
\>", "Text",
  Evaluatable->False,
  AspectRatioFixed->True]
}, Closed]],

Cell[CellGroupData[{

Cell[" Pattern Spaces", "Subsection"],

Cell[TextData[{
  "\[DoubleStruckCapitalF] For the analysis of the pebbling game it is \
convenient to think of a configuration of pebbles as a map from the squares \
of the  board to ",
  Cell[BoxData[
      \(TraditionalForm\`{0, 1}\)]],
  ", construed as the two-element field,  ",
  Cell[BoxData[
      \(TraditionalForm\`\[DoubleStruckCapitalF]\_2\)]],
  ".  Addition here amounts to addition modulo 2, or exclusive or if we \
identify 0 with false and 1 with true. We can then interpret the collection \
of all configurations as a vector space over  ",
  Cell[BoxData[
      \(TraditionalForm\`\[DoubleStruckCapitalF]\_2\)]],
  ". A chessboard is just a special type of undirected graph, so we can think \
more generally of any finite graph ",
  Cell[BoxData[
      \(TraditionalForm\`G\  = \ \[LeftAngleBracket]\ V, 
          E\ \[RightAngleBracket]\)]],
  " and the vector space  \[ScriptCapitalC]  of all configurations  ",
  Cell[BoxData[
      \(TraditionalForm\`X\  : \ V\ \ \[LongRightArrow]\ \ {0, 1}\)]],
  ". We will call any such space a pattern space.   There is a natural linear \
operator on \[ScriptCapitalC]  defined by \n\n\t\t",
  Cell[BoxData[
      \(TraditionalForm\`\(\[Sigma](
            X)\) \((v)\)\  = \ \[Sum]\_\(u\  \[Element] \ N(v)\)\ X(u)\)]],
  "\n\nwhere addition is over ",
  Cell[BoxData[
      \(TraditionalForm\`\[DoubleStruckCapitalF]\_2\)]],
  ".   Here ",
  Cell[BoxData[
      \(TraditionalForm\`N(v)\)]],
  "  denotes the closed neighborhood of ",
  Cell[BoxData[
      \(TraditionalForm\`v\)]],
  " in graph ",
  Cell[BoxData[
      \(TraditionalForm\`G\)]],
  "  (which includes the vertex ",
  Cell[BoxData[
      \(TraditionalForm\`v\)]],
  " itself).",
  Cell[BoxData[
      FormBox[
        RowBox[{Cell[""], Cell[""]}], TraditionalForm]]],
  " Thus, operator ",
  Cell[BoxData[
      \(TraditionalForm\`\[Sigma]\)]],
  " adds up the values of all neighbors of a vertex plus the vertex itself, \
modulo 2, to obtain a new value for ",
  Cell[BoxData[
      \(TraditionalForm\`v\)]],
  ".  A closely related operator is ",
  Cell[BoxData[
      \(TraditionalForm\`\(\[Sigma]\^-\)\  = \ \[Sigma]\  + \ I\)]],
  ",  which excludes the center vertex from the summation (since we are \
dealing with a field of characteristic 2, summation is the same as \
subtraction). While this operator makes sense for any locally finite graph, \
we will focus on paths, cycles, and Cartesian products thereof, grids, \
cylinders, and tori. For clarity, we may indicate the underlying graph as a \
subscript. Thus, ",
  Cell[BoxData[
      \(TraditionalForm\`\[Sigma]\_m\)]],
  " is operator \[Sigma] on a path of length ",
  Cell[BoxData[
      \(TraditionalForm\`m\)]],
  ", and ",
  Cell[BoxData[
      \(TraditionalForm\`\[Sigma]\_\(m, n\)\)]],
  " is the same operator on an ",
  Cell[BoxData[
      \(TraditionalForm\`m\)]],
  " by ",
  Cell[BoxData[
      \(TraditionalForm\`n\)]],
  " grid. "
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  "A moment's thought reveals the connection to the pebbling game.  The graph \
here is an 8 by 8 undirected grid, a product graph with vertex set ",
  Cell[BoxData[
      \(TraditionalForm\`\([8]\)\[Cross]\([8]\)\)]],
  ".  Flipping the pebbles in all those places ",
  Cell[BoxData[
      \(TraditionalForm\`v\)]],
  " where  ",
  Cell[BoxData[
      \(TraditionalForm\`X(v) = 1\)]],
  " will produce an all-white board if, and only if, ",
  Cell[BoxData[
      FormBox[
        RowBox[{\(\[Sigma](X)\), "=", 
          StyleBox["1",
            FontWeight->"Bold"]}], TraditionalForm]]],
  ".  Here ",
  Cell[BoxData[
      FormBox[
        StyleBox["1",
          FontWeight->"Bold"], TraditionalForm]]],
  "  denotes the all-ones configuration, ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          RowBox[{
            StyleBox["1",
              FontWeight->"Bold"], \((v)\)}], "=", "1"}], TraditionalForm]]],
  ".  To see that such a configuration ",
  Cell[BoxData[
      \(TraditionalForm\`X\)]],
  " always exists one can exploit the fact there is an inner product on \
pattern spaces:  ",
  Cell[BoxData[
      \(TraditionalForm\`\[LeftAngleBracket]X, Y\[RightAngleBracket]\)]],
  "  is the cardinality of  ",
  Cell[BoxData[
      \(TraditionalForm\`X\  \[Intersection] \ Y\)]],
  ", modulo 2. With respect to this inner product, ",
  Cell[BoxData[
      \(TraditionalForm\`\[Sigma]\)]],
  " is self-adjoint. Hence, from algebra, the range of ",
  Cell[BoxData[
      \(TraditionalForm\`\[Sigma]\)]],
  " is perpendicular to its kernel.  From the handshaking lemma one can see \
that the all-ones pattern ",
  Cell[BoxData[
      FormBox[
        RowBox[{" ", 
          StyleBox["1",
            FontWeight->"Bold"], " "}], TraditionalForm]]],
  " is perpendicular to all kernel patterns, and must therefore lie in the \
range. Unfortunately, the outlined proof of the theorem is non-constructive: \
the all-ones pattern must have a predecessor since it is perpendicular to a \
certain collection of patterns, but there is no immediate way to actually \
construct the predecessor.  On an ordinary chessboard, there are  ",
  Cell[BoxData[
      \(TraditionalForm\`2\^64\)]],
  "  patterns, so exhaustive search is out of the question. "
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Kronecker Matrices", "Section",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  "For experimental computation it is best to represent the \[Sigma] operator \
by matrices over ",
  Cell[BoxData[
      \(TraditionalForm\`\[DoubleStruckCapitalF]\_2\)]],
  ".  When the underlying graph is a simple path of length ",
  Cell[BoxData[
      \(TraditionalForm\`n\)]],
  ",  the matrix representation of ",
  Cell[BoxData[
      \(TraditionalForm\`\[Sigma]\)]],
  " is a tri-diagonal ",
  Cell[BoxData[
      \(TraditionalForm\`n\ \[Cross]\ n\)]],
  "  matrix."
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[BoxData[
    \(\(PlotMatrix[\ SigmaMatrix[8], GridLines \[Rule] True];\)\)], "Input",
  CellLabel->"In[7]:=",
  AspectRatioFixed->True],

Cell[TextData[{
  "For higher dimensional grids the situation becomes quite a bit more \
complicated and we need Kronecker products to give a reasonable description \
of the matrices that describe  ",
  Cell[BoxData[
      \(TraditionalForm\`\[Sigma]\)]],
  "  on these grids. Below are the matrices for the ",
  Cell[BoxData[
      \(TraditionalForm\`8\ \[Cross]\ 8\)]],
  " chessboard, and for a ",
  Cell[BoxData[
      \(TraditionalForm\`2\^6\)]],
  " hypercube."
}], "Text"],

Cell[BoxData[
    \(\(PlotMatrix[KroneckerMatrix[8, 8\ ]];\)\)], "Input",
  CellLabel->"In[8]:="],

Cell[BoxData[
    \(\(PlotMatrix[
        Inverse[KroneckerMatrix[8, 8\ ], Modulus \[Rule] 2]];\)\)], "Input",
  CellLabel->"In[9]:="],

Cell[BoxData[
    \(\(PlotMatrix[KroneckerMatrix[2, 2, 2, 2, 2, 2]];\)\)], "Input",
  CellLabel->"In[10]:="],

Cell[TextData[{
  "The collection of solutions to our pebbling game forms an affine subspace \
of our pattern space, hence the number of solutions is always 0 or ",
  Cell[BoxData[
      \(TraditionalForm\`2\^\(\(\ \)\(d\)\)\)]],
  ", where ",
  Cell[BoxData[
      \(TraditionalForm\`d\)]],
  " is the dimension of the kernel of ",
  Cell[BoxData[
      \(TraditionalForm\`\[Sigma]\)]],
  ".  The same comment applies to all preimages ",
  Cell[BoxData[
      \(TraditionalForm\`\(\[Sigma]\^\(-1\)\)(X)\)]],
  ". Uniqueness of the solution can now be expressed in terms of the \
reversibility of these matrices. More generally, we can compute the rank of \
the \[Sigma] matrices. Here are the results for some paths graphs."
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[BoxData[
    \(TableForm[\ Table[\ Rank2[\ SigmaMatrix[i]], {i, 16}], 
      TableHeadings \[Rule] {Automatic}]\)], "Input",
  CellLabel->"In[11]:="],

Cell["\<\
It is perhaps easier to see what is going on if one considers the \
corank instead.\
\>", "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[BoxData[
    \(TableForm[\ Table[\ CoRank2[\ SigmaMatrix[i]], {i, 16}], 
      TableHeadings \[Rule] {Automatic}]\)], "Input",
  CellLabel->"In[12]:="],

Cell[TextData[{
  "Thus, on a path of length ",
  Cell[BoxData[
      \(TraditionalForm\`n\)]],
  " operation \[Sigma] is reversible if, and only if, ",
  Cell[BoxData[
      \(TraditionalForm\`n\  \[NotEqual] \ 2\ \((mod\ 3)\)\)]],
  ". For the other values of ",
  Cell[BoxData[
      \(TraditionalForm\`n\)]],
  " the kernel of \[Sigma] consists of only one non-trivial pattern."
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[BoxData[
    \(NullSpace2[\ SigmaMatrix[11]]\)], "Input",
  CellLabel->"In[13]:="],

Cell[TextData[{
  "How about cycles? A similar picture emerges. Disregarding the degenerate \
cases ",
  Cell[BoxData[
      \(TraditionalForm\`n = 1, 2\)]],
  " we have invertibility for ",
  Cell[BoxData[
      \(TraditionalForm\`n\  \[NotEqual] \ 0\ \((mod\ 3)\)\)]],
  ". In the other cases the kernel has dimension 2. The whole kernel space \
consists of ",
  Cell[BoxData[
      FormBox[
        StyleBox["0",
          FontWeight->"Bold"], TraditionalForm]]],
  " and the cyclic rotations of just one configuration."
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[BoxData[
    \(Table[\ 
      CoRank2[\ SigmaMatrix[i, Cyclic \[Rule] True]], {i, 20}]\)], "Input",
  CellLabel->"In[14]:="],

Cell[BoxData[
    \(NullSpace2[\ SigmaMatrix[12, Cyclic \[Rule] True]]\)], "Input",
  CellLabel->"In[15]:="],

Cell[TextData[{
  "Likewise one can easily handle ",
  Cell[BoxData[
      \(TraditionalForm\`2\ \[Cross]\ n\)]],
  " rectangles,  ",
  Cell[BoxData[
      \(TraditionalForm\`3\ \[Cross]\ n\)]],
  " rectangles and corresponding cyclic bands. "
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[BoxData[
    \(Table[\ CoRank2[\ KroneckerMatrix[2, i]], {i, 20}]\)], "Input",
  CellLabel->"In[16]:="],

Cell[BoxData[
    \(Table[\ CoRank2[\ KroneckerMatrix[3, i]], {i, 20}]\)], "Input",
  CellLabel->"In[17]:="],

Cell[TextData[{
  "But how about squares, or more generally, all rectangles of size ",
  Cell[BoxData[
      \(TraditionalForm\`m\ \[Cross]\ n\)]],
  "? The situation there turns out to be much more complicated. Note the 4 in \
position ",
  Cell[BoxData[
      \(TraditionalForm\`\((4, 4)\)\)]],
  ":  on the 4 by 4 square the kernel of \[Sigma] has full dimension 4. One \
can show that ",
  Cell[BoxData[
      \(TraditionalForm\`m = 4\)]],
  " is the only case where this happens, but the argument is rather \
complicated. "
}], "Text"],

Cell[BoxData[{
    \(\(tab\  = \ 
        Table[\ CoRank2[\ KroneckerMatrix[i, j]], {i, 12}, {j, 
            12}];\)\), "\n", 
    \(TableForm[\ tab, \ TableSpacing \[Rule] {1, 1}\ ]\)}], "Input",
  CellLabel->"In[18]:="],

Cell["A plot of irreversible grid sizes.", "Text"],

Cell[BoxData[
    \(\(\(tab\  // \ Sign\)\  // \ PlotMatrix;\)\)], "Input",
  CellLabel->"In[20]:="],

Cell[TextData[{
  "For the operator  ",
  Cell[BoxData[
      \(TraditionalForm\`\(\[Sigma]\^-\)\)]],
  " the situation turns out to be considerably less complicated. "
}], "Text"],

Cell[BoxData[{
    \(\(tab\  = \ 
        Table[\ CoRank2[\ KroneckerMatrix[i, j, Full \[Rule] False]], {i, 
            12}, {j, 12}];\)\), "\n", 
    \(TableForm[\ tab, \ TableSpacing \[Rule] {1, 1}\ ]\)}], "Input",
  CellLabel->"In[21]:="],

Cell[TextData[{
  "Note the full coranks along the diagonal.  This observation is the key to \
the analysis of ",
  Cell[BoxData[
      \(TraditionalForm\`\(\[Sigma]\^-\)\)]],
  " automata on grids.  Again, a plot of irreversible grid sizes."
}], "Text"],

Cell[BoxData[
    \(\(\(tab\  // \ Sign\)\  // \ PlotMatrix;\)\)], "Input",
  CellLabel->"In[23]:="],

Cell[TextData[{
  "The computational effort here is rather significant: for an ",
  Cell[BoxData[
      \(TraditionalForm\`m\ \[Cross]\ n\)]],
  " grid, the corresponding Kronecker matrix has size ",
  Cell[BoxData[
      \(TraditionalForm\`m\[VeryThinSpace]n\ \[Cross]\ 
          m\[VeryThinSpace]n\)]],
  ". Thus there is no hope to push this approach much further. "
}], "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["Fibonacci and \[Pi] Polynomials", "Section",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["Extending Patterns", "Subsubsection"],

Cell[TextData[{
  "We need to avoid having to compute the Kronecker matrix representing ",
  Cell[BoxData[
      \(TraditionalForm\`\[Sigma]\)]],
  " directly. As a first step, let us try to construct a pattern ",
  Cell[BoxData[
      \(TraditionalForm\`Z\)]],
  " in the kernel of ",
  Cell[BoxData[
      \(TraditionalForm\`\[Sigma]\)]],
  ",  rather than a preimage of ",
  Cell[BoxData[
      FormBox[
        StyleBox["1",
          FontWeight->"Bold"], TraditionalForm]]],
  ".  In other words, we would like to solve the matrix equation ",
  Cell[BoxData[
      FormBox[
        RowBox[{\(S\ \[CenterDot]\ X\), " ", "=", " ", 
          StyleBox["0",
            FontWeight->"Bold"]}], TraditionalForm]]],
  " without having to compute the Kronecker matrix ",
  Cell[BoxData[
      \(TraditionalForm\`S\)]],
  ". This may appear to be not particularly helpful vis-a-vis our pebbling \
problem, but as it turns out the problems are very similar indeed.  Suppose \
",
  Cell[BoxData[
      \(TraditionalForm\`Z\)]],
  " is an ",
  Cell[BoxData[
      \(TraditionalForm\`n\[Cross]m\)]],
  " pattern in the kernel of ",
  Cell[BoxData[
      \(TraditionalForm\`\[Sigma]\)]],
  ". We claim that ",
  Cell[BoxData[
      \(TraditionalForm\`Z\)]],
  " is entirely determined by its first row ",
  Cell[BoxData[
      \(TraditionalForm\`Z\_\(\(\ \)\(1\)\)\)]],
  ".  To see this, write ",
  Cell[BoxData[
      \(TraditionalForm\`Z\_\(\(\ \ \)\(i\)\)\)]],
  " for the ",
  Cell[BoxData[
      \(TraditionalForm\`i\)]],
  "-th row in ",
  Cell[BoxData[
      \(TraditionalForm\`Z\)]],
  ".  Since ",
  Cell[BoxData[
      \(TraditionalForm\`Z\)]],
  " is in the kernel, we must have a Fibonacci type recurrence\n\n\t",
  Cell[BoxData[
      \(TraditionalForm\`Z\_\(\(\ \)\(n + 1\)\)\  = \[Sigma]\_m\ \((Z\_\(\(\ \
\)\(n\)\))\)\  + \ Z\_\(\(\ \)\(n - 1\)\)\)]],
  ",\n\t\nwhere ",
  Cell[BoxData[
      \(TraditionalForm\`\[Sigma]\_m\)]],
  " is our linear operator for the smaller pattern space  ",
  Cell[BoxData[
      \(TraditionalForm\`\([m]\)\ \[LongRightArrow]\ {0, 1}\)]],
  ". Here we tacitly assume that ",
  Cell[BoxData[
      FormBox[
        RowBox[{\(Z\_\(\(\ \)\(0\)\)\), "=", " ", 
          StyleBox["0",
            FontWeight->"Bold"]}], TraditionalForm]]],
  ".  On the other hand, suppose we start with a row ",
  Cell[BoxData[
      \(TraditionalForm\`Y\)]],
  ".  We can define a linear extension operator  ",
  Cell[BoxData[
      \(TraditionalForm\`ext(Y, i)\)]],
  "  as follows: \n\n\t",
  Cell[BoxData[
      \(TraditionalForm\`ext(Y, 0)\  = \ 0\)]],
  ",\n\t",
  Cell[BoxData[
      \(TraditionalForm\`ext(Y, 1)\  = \ Y\)]],
  ", \n\t",
  Cell[BoxData[
      \(TraditionalForm\`ext(Y, 
          n)\  = \ \[Sigma](\ ext(Y, n - 1))\ \  + \ \ ext(Y, n - 2)\)]],
  ".\n\t\nfor all",
  Cell[BoxData[
      \(TraditionalForm\`\(\(\ \)\(n\  \[GreaterEqual] \ 2\)\)\)]],
  ".  More precisely, we can think of the extension operator as a linear map \
",
  Cell[BoxData[
      \(TraditionalForm\`ext\  : \ \[ScriptCapitalC]\_m\[LongRightArrow]\ \
\[ScriptCapitalC]\_\(m, \[Infinity]\)\)]],
  " which associates to each pattern of size ",
  Cell[BoxData[
      \(TraditionalForm\`m\)]],
  " an infinite pattern on the grid ",
  Cell[BoxData[
      \(TraditionalForm\`\([m]\)\ \[Cross]\ \[DoubleStruckCapitalN]\)]],
  ".  Here is an example, the extension of a one-point seed on ",
  Cell[BoxData[
      \(TraditionalForm\`\[ScriptCapitalC]\_11\)]],
  ", out to 48 steps. "
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[BoxData[{
    \(\(pat\  = \ 
        ExtendPatternZero[\ SeedConfiguration[{1}, 11], 48\ ];\)\), "\n", 
    \(\(PlotMatrix[\ pat\ ];\)\)}], "Input",
  CellLabel->"In[24]:="],

Cell["\<\
One can immediately observe various symmetries, as well as several \
zero rows. But if we truncate at a zero row, we get a kernel pattern for that \
particular finite grid. \
\>", "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[BoxData[{
    \(\(X\  = \ Take[pat, 23];\)\), "\[IndentingNewLine]", 
    \(\(Sigma[X] // \ Flatten\)\  // \ Union\)}], "Input",
  CellLabel->"In[26]:="],

Cell[TextData[{
  "Alternatively, we can check that the pattern is a fixed point under ",
  Cell[BoxData[
      \(TraditionalForm\`\(\[Sigma]\^-\)\)]],
  "."
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[BoxData[
    \(Sigma[\ X, Full \[Rule] False] === X\)], "Input",
  CellLabel->"In[28]:="],

Cell[TextData[{
  "More generally, the pattern  ",
  Cell[BoxData[
      \(TraditionalForm\`Z\)]],
  "  defined by  ",
  Cell[BoxData[
      \(TraditionalForm\`Z\_\(\(\ \)\(i\)\) = \ ext(Y, i), \ 
      1\  \[LessEqual] \ i \[LessEqual] \ n\)]],
  ",  always lies almost in the kernel of ",
  Cell[BoxData[
      \(TraditionalForm\`\[Sigma]\)]],
  ":   ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          FormBox[\(\[Sigma](Z)\),
            "TraditionalForm"], " ", "=", " ", 
          StyleBox["0",
            FontWeight->"Bold"]}], TraditionalForm]]],
  ", except perhaps in the last row. The last row will work out  iff ",
  Cell[BoxData[
      \(TraditionalForm\`ext(Y, n + 1)\  = \ 0\)]],
  ". "
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  "It follows that the dimension of the kernel of ",
  Cell[BoxData[
      \(TraditionalForm\`\[Sigma]\)]],
  " is at most ",
  Cell[BoxData[
      \(TraditionalForm\`min(m, n)\)]],
  ". A similar extension trick can be used for any target pattern ",
  Cell[BoxData[
      \(TraditionalForm\`X\)]],
  ", not just ",
  Cell[BoxData[
      FormBox[
        StyleBox["0",
          FontWeight->"Bold"], TraditionalForm]]],
  " or ",
  Cell[BoxData[
      FormBox[
        StyleBox["1",
          FontWeight->"Bold"], TraditionalForm]]],
  ".  Of course, if we start with the wrong pattern in the first row, the \
extension will ultimately fail, but at least we have to search only over ",
  Cell[BoxData[
      \(TraditionalForm\`2\^\(\(\ \)\(min(m, n)\)\)\)]],
  "  starting patterns. "
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  "As we have seen, for the ",
  Cell[BoxData[
      \(TraditionalForm\`4\ \[Cross]\ 4\)]],
  " square it does not matter where we start, the extension is always \
successful.  Here are all the kernel patterns (no symmetries are exploited)."
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[BoxData[{
    \(\(\(PlotMatrix[#, DisplayFunction \[Rule] Identity, 
            GridLines \[Rule] True] &\)\  /@ 
        Table[ExtendPatternOne[IntegerDigits[i, 2, 4\ ], 4], \ {i, 0, 
            15}];\)\), "\[IndentingNewLine]", 
    \(\(ShowArray[%];\)\)}], "Input",
  CellLabel->"In[29]:="]
}, Closed]],

Cell[CellGroupData[{

Cell["Binary Fibonacci Polynomials", "Subsubsection"],

Cell[TextData[{
  "Let us take a closer look at the recurrence  ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          FormBox[\(Z\_n\),
            "TraditionalForm"], " ", "=", " ", 
          RowBox[{
            RowBox[{
              FormBox["\[Sigma]",
                "TraditionalForm"], "(", \(Z\_\(n - 1\)\), ")"}], "  ", "+", 
            "   ", \(Z\_\(n - 2\)\)}]}], TraditionalForm]]],
  " from above. Actually, it is best to ignore the ",
  Cell[BoxData[
      \(TraditionalForm\`\[Sigma]\)]],
  " operator here entirely and study a general Fibonacci type recurrence over \
",
  Cell[BoxData[
      \(TraditionalForm\`\[DoubleStruckCapitalF]\_2[x]\)]],
  ", the ring of polynomials over ",
  Cell[BoxData[
      \(TraditionalForm\`\[DoubleStruckCapitalF]\_2\)]],
  ".  We will refer to these polynomials as  ",
  Cell[BoxData[
      \(TraditionalForm\`\[Pi]\)]],
  "-polynomials.  Here is the definition.\n\n\t",
  Cell[BoxData[
      FormBox[
        RowBox[{
          FormBox[\(\[Pi]\_0\),
            "TraditionalForm"], " ", "=", " ", "0"}], TraditionalForm]]],
  ",\n\t",
  Cell[BoxData[
      FormBox[
        RowBox[{
          FormBox[\(\[Pi]\_\(\(1\)\(\ \)\)\),
            "TraditionalForm"], "=", " ", "1"}], TraditionalForm]]],
  ",\n\t",
  Cell[BoxData[
      \(TraditionalForm\`\[Pi]\_n\  = \ 
        x\ \[CenterDot]\ \[Pi]\_\(n - 1\)\  + \ \(\(\[Pi]\_\(n - 
                  2\)\)\(.\)\)\)]],
  "\n\t\nReplacing the variable ",
  Cell[BoxData[
      \(TraditionalForm\`x\)]],
  " by ",
  Cell[BoxData[
      \(TraditionalForm\`\[Sigma]\)]],
  ", we get back our extension operator for kernel patterns: ",
  Cell[BoxData[
      \(TraditionalForm\`ext(Y, 
          n)\  = \ \(\(\[Pi]\_\(\(\ \)\(n\)\)\)(\[Sigma])\) \((Y)\)\)]],
  ".  For the time being, though, we will consider  ",
  Cell[BoxData[
      \(TraditionalForm\`\[Pi]\_n\)]],
  "  just as a polynomial with coefficients in ",
  Cell[BoxData[
      \(TraditionalForm\`\[DoubleStruckCapitalF]\_2\)]],
  ".  The integer version of these polynomials is a built-in function in ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "."
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[BoxData[{
    \(\(Fibonacci2[n_]\  := \ 
        mod2@Fibonacci[n, x];\)\), "\[IndentingNewLine]", 
    \(Fibonacci2[33]\)}], "Input",
  CellLabel->"In[31]:="],

Cell[TextData[{
  "Here are the first few ",
  Cell[BoxData[
      \(TraditionalForm\`\[Pi]\)]],
  "-polynomials: "
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[BoxData[
    \(EnumForm0[\ Fibonacci2[Range[0, 16]]]\)], "Input",
  CellLabel->"In[33]:=",
  AspectRatioFixed->True],

Cell[TextData[{
  "And here is an immediate conjecture, motivated by these first few values:  \
",
  Cell[BoxData[
      FormBox[
        RowBox[{
          FormBox[\(\[Pi]\_\(2\^k\)\),
            "TraditionalForm"], " ", "=", 
          " ", \(\(\(x\)\(\ \)\)\^\(2\^k - 1\)\)}], TraditionalForm]]],
  ".  We can verify our conjecture for a few powers of 2. "
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["Fibonacci2[2^Range[8]]", "Input",
  CellLabel->"In[34]:=",
  AspectRatioFixed->True],

Cell[TextData[{
  "There are few other observations:  the monomials in  ",
  Cell[BoxData[
      \(TraditionalForm\`\[Pi]\_\(2  n\)\)]],
  "  are always odd powers of ",
  Cell[BoxData[
      \(TraditionalForm\`x\)]],
  ", whereas the terms in  ",
  Cell[BoxData[
      \(TraditionalForm\`\[Pi]\_\(2  n + 1\)\)]],
  "  are always even powers of ",
  Cell[BoxData[
      \(TraditionalForm\`x\)]],
  ".  Of course, the degree of  ",
  Cell[BoxData[
      \(TraditionalForm\`\[Pi]\_n\)]],
  "  is  ",
  Cell[BoxData[
      \(TraditionalForm\`n - 1\)]],
  ".  ",
  StyleBox["Our recurrence is linear and homogeneous, but the ring of \
coefficients is more complicated than usual. Among other things, the \
characteristic is 2, which has the peculiar consequence that out recurrence \
can be run backwards:  ",
    FontFamily->"Times"],
  Cell[BoxData[
      \(TraditionalForm\`\[Pi]\_\(n - 2\)\  = \ 
        x\ \[CenterDot]\ \[Pi]\_\(n - 1\)\  + \ \[Pi]\_n\)]],
  "."
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[BoxData[
    \(mod2[\ x\ \ Fibonacci2[20] + Fibonacci2[21]] === \ 
      Fibonacci2[19]\)], "Input",
  CellLabel->"In[35]:=",
  AspectRatioFixed->True],

Cell[TextData[{
  "The extension operator for kernel patterns is linear, but for other target \
patterns such as ",
  Cell[BoxData[
      FormBox[
        StyleBox["1",
          FontWeight->"Bold"], TraditionalForm]]],
  " we only have affine operations. The linear part is just ",
  Cell[BoxData[
      \(TraditionalForm\`\(\[Pi]\_n\)(\[Sigma])\)]],
  ", but the affine part is obtained by applying a polynomial similar to ",
  Cell[BoxData[
      \(TraditionalForm\`\(\[Pi]\_n\)(\[Sigma])\)]],
  " to ",
  Cell[BoxData[
      FormBox[
        SubscriptBox[
          FormBox[
            StyleBox["1",
              FontWeight->"Bold"],
            "TraditionalForm"], "m"], TraditionalForm]]],
  ", the all-ones configuration on the path of length ",
  Cell[BoxData[
      \(TraditionalForm\`m\)]],
  ".  The recursive definition for these other polynomials is as follows. \n\n\
\t",
  Cell[BoxData[
      FormBox[
        RowBox[{
          FormBox[\(\[Tau]\_\(\(\ \)\(0\)\)\),
            "TraditionalForm"], " ", "=", " ", "0"}], TraditionalForm]]],
  ",\n\t",
  Cell[BoxData[
      FormBox[
        RowBox[{
          FormBox[\(\[Tau]\_\(\(\ \)\(1\)\)\),
            "TraditionalForm"], "=", " ", "1"}], TraditionalForm]]],
  ",\n\t",
  Cell[BoxData[
      \(TraditionalForm\`\[Tau]\_n\  = \ 
        x\ \[CenterDot]\ \[Tau]\_\(n - 1\)\  + \ \[Tau]\_\(n - 2\)\  + \ 
          1. \)]],
  "\n\t\nThus, the \[Tau] polynomials are a non-homogeneous version of the \
\[Pi] polynomials. If we compare the coefficient patterns for the two \
families, it becomes clear that they are closely related."
}], "Text"],

Cell[BoxData[{
    \(ClearAll[tau]\), "\[IndentingNewLine]", 
    \(\(tau[0]\  = \ 0;\)\), "\[IndentingNewLine]", 
    \(\(tau[1]\  = \ 1;\)\), "\[IndentingNewLine]", 
    \(\(tau[n_]\  := \ \(tau[n]\  = \ 
          mod2[x\ tau[n - 1]\  + \ tau[n - 2]\  + \ 1]\);\)\)}], "Input",
  CellLabel->"In[36]:="],

Cell[BoxData[{
    \(\(m1\  = \ \(PadRight[CoefficientList[\ pi[#], x], 50] &\)\  /@ \ 
          Range[50];\)\), "\[IndentingNewLine]", 
    \(m2\  = \ \(PadRight[CoefficientList[\ tau[#], x], 50] &\)\  /@ \ 
        Range[50]; 
    ShowArray[\ {PlotMatrix[m1, DisplayFunction \[Rule] Identity], 
        PlotMatrix[m2, DisplayFunction \[Rule] Identity]}];\)}], "Input",
  CellLabel->"In[40]:="],

Cell[TextData[{
  "A little more work shows that indeed\n\n\t",
  Cell[BoxData[
      \(TraditionalForm\`\[Tau]\_\(\(\ \)\(n\)\)\  = \ \ \ \((\[Pi]\_n + \
\[Pi]\_\(n + 1\) + 1)\)/x\)]],
  ".\n\nThus, it all comes down to computing \[Pi] polynomials. The all-ones \
extension operator now takes the form \n\n\t",
  Cell[BoxData[
      FormBox[
        RowBox[{
          RowBox[{
            SubscriptBox["ext", 
              StyleBox["1",
                FontWeight->"Bold"]], "(", \(Y, n\), ")"}], " ", "=", " ", 
          
          RowBox[{\(\(\(\[Pi]\_\(\(\ \)\(n\)\)\)(\[Sigma])\) \((Y)\)\), " ", 
            "+", " ", 
            RowBox[{\(\(\[Tau]\_\(\(\ \)\(n - 1\)\)\)(\[Sigma])\), 
              RowBox[{"(", 
                SubscriptBox[
                  StyleBox["1",
                    FontWeight->"Bold"], "m"], 
                StyleBox[")",
                  FontWeight->"Plain"]}]}]}]}], TraditionalForm]]],
  "   \n\nand, as before, the extension is successful if, and only if,  ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          RowBox[{
            SubscriptBox["ext", 
              StyleBox["1",
                FontWeight->"Bold"]], "(", \(Y, n + 1\), ")"}], " ", "=", " ",
           
          RowBox[{
            
            RowBox[{\(\(\(\[Pi]\_\(\(\ \)\(n + 1\)\)\)(\[Sigma])\) \((Y)\)\), 
              " ", "+", " ", 
              RowBox[{\(\(\[Tau]\_\(\(\ \)\(n\)\)\)(\[Sigma])\), 
                RowBox[{"(", 
                  StyleBox["1",
                    FontWeight->"Bold"], 
                  StyleBox[")",
                    FontWeight->"Plain"]}]}]}], 
            StyleBox[" ",
              FontWeight->"Plain"], 
            StyleBox["=",
              FontWeight->"Plain"], 
            StyleBox[" ",
              FontWeight->"Plain"], 
            StyleBox["0",
              FontWeight->"Bold"]}]}], TraditionalForm]]],
  ".    Here are the matrices for some of these operators. "
}], "Text"],

Cell[BoxData[{
    \(\(\(PlotMatrix[PolynomialToMatrix[pi[#], 10, \ Full \[Rule] True], 
            DisplayFunction \[Rule] Identity] &\)\  /@ \ 
        Range[0, 35];\)\), "\[IndentingNewLine]", 
    \(\(ShowArray[%];\)\)}], "Input",
  CellLabel->"In[42]:=",
  AspectRatioFixed->True],

Cell[BoxData[{
    \(\(\(PlotMatrix[PolynomialToMatrix[tau[#], 10], 
            DisplayFunction \[Rule] Identity] &\)\  /@ \ 
        Range[0, 24];\)\), "\[IndentingNewLine]", 
    \(\(ShowArray[%];\)\)}], "Input",
  CellLabel->"In[44]:=",
  AspectRatioFixed->True],

Cell["Note the symmetries in the sequence.", "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["Computing Fibonacci Polynomials", "Subsubsection",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  "If one writes the ",
  Cell[BoxData[
      \(TraditionalForm\`\[Pi]\)]],
  "-polynomials as  ",
  Cell[BoxData[
      \(TraditionalForm\`\[Pi]\_n\  = \ \[Sum]\+\(i < n\)c\_\(n, i\)\ \
x\^i\)]],
  "  then the coefficients also obey a Fibonacci type recurrence relation, \
with the appropriate ranges for ",
  Cell[BoxData[
      \(TraditionalForm\`n\)]],
  " and ",
  Cell[BoxData[
      \(TraditionalForm\`i\)]],
  ": \n\n\t",
  Cell[BoxData[
      \(TraditionalForm\`c\_\(0, 0\)\  = \ \ 0\)]],
  ",\n\t",
  Cell[BoxData[
      \(TraditionalForm\`c\_\(1, 0\)\  = \ \ 1\)]],
  ",\n\t",
  Cell[BoxData[
      \(TraditionalForm\`c\_\(n, 0\)\  = \ \ c\_\(n - 2, 0\)\)]],
  ",\n\t",
  Cell[BoxData[
      \(TraditionalForm\`c\_\(n, i\)\  = \ \ c\_\(n - 1, i - 1\)\  + \ 
          c\_\(n - 2, i\)\)]],
  ".\n\nOf course, addition here is understood to be mod 2 since we are \
operating in ",
  Cell[BoxData[
      \(TraditionalForm\`\[DoubleStruckCapitalF]\_2[x]\)]],
  ". It is not hard to show that the coefficients have the following closed \
form description\n\n\t",
  Cell[BoxData[
      FormBox[
        FormBox[
          RowBox[{\(c\_\(\(\ \)\(n, \ i\)\)\), "  ", "=", " ", 
            RowBox[{
              FormBox[
                RowBox[{"(", "\[NegativeThinSpace]", GridBox[{
                      {\(n + i\)},
                      {\(2  i + 1\)}
                      }], "\[NegativeThinSpace]", ")"}],
                "TraditionalForm"], "  ", "mod", " ", "2"}]}],
          "TraditionalForm"], TraditionalForm]]],
  "\n \nAs it turns out, one does not have to compute the binomial \
coefficients to determine ",
  Cell[BoxData[
      \(TraditionalForm\`c\)]],
  " coefficients.  Let us say that a natural number ",
  Cell[BoxData[
      \(TraditionalForm\`n\)]],
  " precedes ",
  Cell[BoxData[
      \(TraditionalForm\`m\)]],
  " if the ",
  Cell[BoxData[
      \(TraditionalForm\`i\)]],
  "th bit in the binary expansion of ",
  Cell[BoxData[
      \(TraditionalForm\`n\)]],
  " is less than or equal to the ",
  Cell[BoxData[
      \(TraditionalForm\`i\)]],
  "th bit in the expansion of ",
  Cell[BoxData[
      \(TraditionalForm\`m\)]],
  ", for all ",
  Cell[BoxData[
      \(TraditionalForm\`i\)]],
  " (pad with 0's if necessary).    "
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[BoxData[
    \(Select[Range[100], BinLess[#1, 100] == 1 &]\)], "Input",
  CellLabel->"In[46]:=",
  AspectRatioFixed->True],

Cell[TextData[{
  "An observation due to E. Lucas is that the binomial coefficient ",
  Cell[BoxData[
      FormBox[
        RowBox[{" ", 
          FormBox[
            RowBox[{"(", "\[NegativeThinSpace]", GridBox[{
                  {"n"},
                  {"m"}
                  }], "\[NegativeThinSpace]", ")"}],
            "TraditionalForm"]}], TraditionalForm]]],
  "  is odd if and only if ",
  Cell[BoxData[
      \(TraditionalForm\`n\)]],
  " precedes",
  Cell[BoxData[
      \(TraditionalForm\`\(\(\ \)\(m\)\)\)]],
  ". "
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[BoxData[
    \(Select[\ Range[100], \ OddQ@Binomial[100, #] &\ ]\)], "Input",
  CellLabel->"In[47]:=",
  AspectRatioFixed->True],

Cell["\<\
Perhaps one should check that our description of the coefficients \
is really correct. \
\>", "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[BoxData[
    \(CoefficientList[\ Fibonacci2[20], x] === 
      Table[\ BinLess[2  i + 1, 20 + i], {i, 0, 19}]\)], "Input",
  CellLabel->"In[48]:="],

Cell[TextData[{
  "Another way to compute the \[Pi] polynomials rapidly is to find a \
recurrence that expresses ",
  Cell[BoxData[
      \(TraditionalForm\`\[Pi]\_n\)]],
  " essentially in terms of ",
  Cell[BoxData[
      \(TraditionalForm\`\[Pi]\_\(n/2\)\)]],
  ".\n\n\t",
  Cell[BoxData[
      FormBox[
        FormBox[
          RowBox[{
            FormBox[\(\(\(\[Pi]\)\(\ \)\)\_\(2  n\)\),
              "TraditionalForm"], " ", "=", " ", \(x\ \ \[Pi]\_n\^2\)}],
          "TraditionalForm"], TraditionalForm]]],
  ",\n\t",
  Cell[BoxData[
      FormBox[
        FormBox[
          RowBox[{\(\[Pi]\_\(2  n + 1\)\), " ", "=", 
            RowBox[{
              
              FormBox[\(\[Pi]\_\(n + 1\)\^2\  + \[Pi]\_\(\(\ \)\(n\)\)\%\(\(\ \
\)\(2\)\)\),
                "TraditionalForm"], " ", "=", 
              FormBox[\(\((\[Pi]\_\(n + 1\)\  + \ \[Pi]\_n)\)\^2\),
                "TraditionalForm"]}]}],
          "TraditionalForm"], TraditionalForm]]],
  ".\n\nThe square can be pushed through since we are working over a field \
with characteristic 2. The depth of the recursion is only ",
  Cell[BoxData[
      \(TraditionalForm\`log\ n\)]],
  ", and it seems that in ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " this is the fastest way to compute the polynomials.  The function ",
  StyleBox["pi[n]", "MR"],
  " in this notebook uses this method."
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True]
}, Closed]],

Cell[CellGroupData[{

Cell["Reversibility", "Subsubsection",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  "So far, the ",
  Cell[BoxData[
      \(TraditionalForm\`\[Pi]\)]],
  "-polynomials are only mildly useful as a tool to compute the kernel of \
\[Sigma] on an ",
  Cell[BoxData[
      \(TraditionalForm\`m\)]],
  " by ",
  Cell[BoxData[
      \(TraditionalForm\`n\)]],
  " grid by computing the nullspace of ",
  Cell[BoxData[
      \(TraditionalForm\`\(\[Pi]\_\(n + 1\)\)(\[Sigma])\)]],
  ". But we can determine the dimension of the kernel from the polynomials \
alone, without ever substituting the \[Sigma] operator. To see why this \
should be possible, first consider a somewhat easier problem, computing the \
corank of ",
  Cell[BoxData[
      \(TraditionalForm\`\(\[Sigma]\^-\)\)]],
  " on an ",
  Cell[BoxData[
      \(TraditionalForm\`m\)]],
  " by ",
  Cell[BoxData[
      \(TraditionalForm\`n\)]],
  " grid. It is not at all clear that ",
  Cell[BoxData[
      \(TraditionalForm\`\(\[Sigma]\^-\) = \[Sigma]\  + I\)]],
  "  should be any easier to handle, but it turns out that there is a very \
simply description of the corank in this case.  First off, ",
  Cell[BoxData[
      FormBox[
        RowBox[{\(\(\[Pi]\_\(m + 1\)\)(\[Sigma]\_m)\), " ", "=", " ", 
          StyleBox["0",
            FontWeight->"Bold"]}], TraditionalForm]]],
  ", for all ",
  Cell[BoxData[
      \(TraditionalForm\`n\)]],
  ". "
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[BoxData[
    \(\(Table[\ 
          PolynomialToMatrix[\ m + 1, m, Full \[Rule] False\ ], {m, 
            20}\ ]\  // \ Flatten\)\  // \ Union\)], "Input",
  CellLabel->"In[49]:="],

Cell[TextData[{
  "We can understand why this is the case by plotting the matrices ",
  Cell[BoxData[
      \(TraditionalForm\`\(\[Pi]\_\(\(\ \)\(k\)\)\)(\[Sigma]\_m)\)]],
  "  for ",
  Cell[BoxData[
      \(TraditionalForm\`k \[LessEqual] m + 1\)]],
  ". Note that the matrices below represent some linear maps from the pattern \
space of a ten-point path to itself. "
}], "Text"],

Cell[BoxData[
    \(\(\(\(\(PlotMatrix[PolynomialToMatrix[\ #, 10, Full \[Rule] False\ ], 
            DisplayFunction \[Rule] Identity] &\)\  /@ \ 
        Range[0, 11];\)\[IndentingNewLine]
    \(ShowArray[%];\)\)\(\ \)\)\)], "Input",
  CellLabel->"In[50]:="],

Cell[TextData[{
  "Note that if we misconstrue these patterns as belonging to the pattern \
space of the 10 by 10 square, we get a basis for the kernel of ",
  Cell[BoxData[
      \(TraditionalForm\`\[Sigma]\_\(10, 10\)\)]],
  ". The checkerboard patterns from this example are easily established in \
general by a combinatorial argument. Alternatively, one can invoke the \
Cayley-Hamilton theorem:  ",
  Cell[BoxData[
      \(TraditionalForm\`\[Pi]\_\(m + 1\)\)]],
  " is is the characteristic polynomial of ",
  Cell[BoxData[
      \(TraditionalForm\`\[Sigma]\_\(\(\ \)\(m\)\)\)]],
  " and therefore an annihilator of this linear map.  In fact, ",
  Cell[BoxData[
      \(TraditionalForm\`\[Pi]\_\(m + 1\)\)]],
  "  turns out to be the minimal polynomial of ",
  Cell[BoxData[
      \(TraditionalForm\`\[Sigma]\_m\)]],
  ". "
}], "Text"],

Cell[BoxData[
    \(pi[11] === 
      mod2@Det[\ 
          SigmaMatrix[10, Full \[Rule] False]\  + \ 
            x\ IdentityMatrix[10]]\)], "Input",
  CellLabel->"In[52]:="],

Cell[TextData[{
  "It follows that on any ",
  Cell[BoxData[
      \(TraditionalForm\`m\)]],
  " by ",
  Cell[BoxData[
      \(TraditionalForm\`m\)]],
  " square,  ",
  Cell[BoxData[
      \(TraditionalForm\`\(\[Sigma]\^-\)\)]],
  " has corank ",
  Cell[BoxData[
      \(TraditionalForm\`m\)]],
  ". More generally, the corank of ",
  Cell[BoxData[
      \(TraditionalForm\`\(\[Sigma]\^-\)\)]],
  " on an ",
  Cell[BoxData[
      \(TraditionalForm\`m\)]],
  " by ",
  Cell[BoxData[
      \(TraditionalForm\`n\)]],
  " grid is ",
  Cell[BoxData[
      \(TraditionalForm\`k\  = \ gcd(m + 1, n + 1) - 1\)]],
  ":  the kernel patterns here are properly positioned copies of kernel \
patterns for the ",
  Cell[BoxData[
      \(TraditionalForm\`k\)]],
  " by ",
  Cell[BoxData[
      \(TraditionalForm\`k\)]],
  " square. An example will make this clear geometrically. "
}], "Text"],

Cell[BoxData[{
    \(\(A\  = \ Zero[15];\)\), "\n", 
    \(\(A[\([3]\)]\  = \(A[\([\(-3\)]\)]\  = \ 1\);\)\), "\n", 
    \(\(ExtendPatternZero[\ A, 23, Full \[Rule] False\ ] // \ 
        PlotMatrix;\)\)}], "Input",
  CellLabel->"In[53]:="],

Cell[TextData[{
  "This pattern in the kernel of ",
  Cell[BoxData[
      \(TraditionalForm\`\[Sigma]\_\(15, 23\)\)]],
  " is composed of 6 kernel patterns for ",
  Cell[BoxData[
      \(TraditionalForm\`\[Sigma]\_\(8, 8\)\)]],
  ". More importantly, algebraically ",
  Cell[BoxData[
      \(TraditionalForm\`gcd(m + 1, n + 1) - 1\)]],
  " is none other than the degree of ",
  Cell[BoxData[
      \(TraditionalForm\`gcd(\ \[Pi]\_\(m + 1\), \ \[Pi]\_\(n + 1\))\)]],
  ". Hence we can test invertibility of ",
  Cell[BoxData[
      \(TraditionalForm\`\[Sigma]\_\(m, n\)\)]],
  " solely by performing polynomial arithmetic in ",
  Cell[BoxData[
      \(TraditionalForm\`\[DoubleStruckCapitalF]\_2[x]\)]],
  ". \nBut how about our original problem, which is based on \[Sigma] rather \
than ",
  Cell[BoxData[
      \(TraditionalForm\`\(\[Sigma]\^-\)\)]],
  " ? As it turns out, the corank of ",
  Cell[BoxData[
      \(TraditionalForm\`\[Sigma]\_\(m, n\)\)]],
  " is the degree of ",
  Cell[BoxData[
      \(TraditionalForm\`gcd(\(\[Pi]\_\(m + 1\)\)(
          x + 1), \(\[Pi]\_\(n + 1\)\)(x))\)]],
  ". Recall that  ",
  Cell[BoxData[
      \(TraditionalForm\`\[Sigma]\  = \ \(\[Sigma]\^-\) + I\)]],
  ", so that  ",
  Cell[BoxData[
      \(TraditionalForm\`\(\[Pi]\_\(\(\ \)\(k\)\)\)(\[Sigma])\  = \ \(\(\[Pi]\
\_\(\(\ \)\(k\)\)\)(1 + x)\) \((\(\[Sigma]\^-\))\)\)]],
  ".  As an example, consider the 11 by 11 square, a case where we can still \
compute the corank by brute force using Kronecker matrices. The corank of \
\[Sigma] turns out to be 6 in this case."
}], "Text"],

Cell[BoxData[
    \(KroneckerMatrix[11, 11]\  // \ CoRank2\)], "Input",
  CellLabel->"In[56]:="],

Cell[TextData[{
  "To determine the GCD of the two polynomials we need to perform the \
substitution ",
  Cell[BoxData[
      \(TraditionalForm\`x\  \[Rule] \ x + 1\)]],
  ", which induces an involution of the polynomial ring. "
}], "Text"],

Cell[BoxData[
    \(PolynomialGCD[\ Phi[pi[12]], pi[12], \ Modulus \[Rule] 2]\)], "Input",
  CellLabel->"In[57]:="],

Cell["\<\
We can now easily determine coranks for fairly large grids, without \
ever having to resort to linear algebra. \
\>", "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[BoxData[
    \(PolynomialGCD[\ Phi[pi[1000]], pi[1000], \ 
      Modulus \[Rule] 2]\)], "Input",
  CellLabel->"In[58]:="]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Analyzing \[Pi] Polynomials", "Section",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["Division Properties and Depth", "Subsection",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  "The question arises if we can push things a bit further. We have seen that \
one can reduce the problem of computing the corank of ",
  Cell[BoxData[
      \(TraditionalForm\`\(\[Sigma]\^-\)\)]],
  " on an ",
  Cell[BoxData[
      \(TraditionalForm\`m\)]],
  " by ",
  Cell[BoxData[
      \(TraditionalForm\`n\)]],
  " grid to integer arithmetic, i.e., the computation of ",
  Cell[BoxData[
      \(TraditionalForm\`gcd(m + 1, n + 1)\)]],
  ".  The reason ultimately is the fact that \[Pi] polynomials have very \
simple division properties, in particular ",
  Cell[BoxData[
      \(TraditionalForm\`gcd(\[Pi]\_s, \[Pi]\_r)\  = \ \[Pi]\_\(gcd(s, \
r)\)\)]],
  ". But for \[Sigma] we are stranded we polynomial arithmetic, we have to \
compute  ",
  Cell[BoxData[
      \(TraditionalForm\`gcd(\(\[Pi]\_s\)(x + 1), \(\[Pi]\_r\)(x))\)]],
  ". Since  ",
  Cell[BoxData[
      \(TraditionalForm\`\(\[Pi]\_s\)(x + 1)\)]],
  " is but the image of  ",
  Cell[BoxData[
      \(TraditionalForm\`\[Pi]\_s\)]],
  " under the involution ",
  Cell[BoxData[
      \(TraditionalForm\`x\  \[RightTeeArrow] x + 1\)]],
  " it is tempting to use some structural properties of the \[Pi] polynomials \
to obtain the degree of the GCD more directly, without explicit computation. \
"
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  "In particular one would like to understand the irreducible factors of ",
  Cell[BoxData[
      \(TraditionalForm\`\[Pi]\_n\)]],
  ". Because of the \"fast\" recurrence\n\n\t",
  Cell[BoxData[
      FormBox[
        RowBox[{
          FormBox[\(\(\(\[Pi]\)\(\ \)\)\_\(2  n\)\),
            "TraditionalForm"], " ", "=", " ", \(x\ \ \[Pi]\_n\^2\)}], 
        TraditionalForm]]],
  ",\n\t",
  Cell[BoxData[
      FormBox[
        RowBox[{\(\[Pi]\_\(2  n + 1\)\), " ", "=", 
          RowBox[{
            
            FormBox[\(\[Pi]\_\(n + 1\)\^2\  + \[Pi]\_\(\(\ \)\(n\)\)\%\(\(\ \
\)\(2\)\)\),
              "TraditionalForm"], " ", "=", 
            FormBox[\(\((\[Pi]\_\(n + 1\)\  + \ \[Pi]\_n)\)\^2\),
              "TraditionalForm"]}]}], TraditionalForm]]],
  "\n\t\nwe can essentially ignore even levels. At odd levels, we are always \
dealing with squares. "
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[BoxData[{
    \(fac2[pi[9]]\), "\n", 
    \(fac2[pi[11]]\), "\n", 
    \(fac2[pi[13]]\), "\n", 
    \(fac2[pi[15]]\)}], "Input",
  CellLabel->"In[59]:=",
  AspectRatioFixed->True],

Cell[TextData[{
  "One can show that  ",
  Cell[BoxData[
      \(TraditionalForm\`\[Pi]\_\(n + 1\)\  + \ \[Pi]\_n\)]],
  " is square-free, so that no higher exponents can occur.  However, the \
next, and rather tempting, thought is wrong: multiple factors do not occur \
only at non-primes.  "
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[BoxData[
    \(fac2[pi[17]]\)], "Input",
  CellLabel->"In[63]:=",
  AspectRatioFixed->True],

Cell[BoxData[
    \(fac2[pi[125]]\)], "Input",
  CellLabel->"In[64]:="],

Cell[TextData[{
  "Quite ugly, at first sight. Let's take a closer look at the irreducible \
factors. If some irreducible polynomial \[Tau] divides any of the \[Pi] \
polynomials there has to be a minimal ",
  Cell[BoxData[
      \(TraditionalForm\`n\)]],
  " such that \[Tau] divides ",
  Cell[BoxData[
      \(TraditionalForm\`\[Pi]\_\(\(\ \)\(n\)\)\)]],
  ". We will call this number ",
  Cell[BoxData[
      \(TraditionalForm\`n\)]],
  " the depth of \[Tau]. Other than 2, the depth can never assume an even \
value.  The product over the square of all irreducible factors of depth ",
  Cell[BoxData[
      \(TraditionalForm\`n\)]],
  " is the critical part of ",
  Cell[BoxData[
      \(TraditionalForm\`\[Pi]\_\(\(\ \)\(n\)\)\)]],
  " and will be written ",
  Cell[BoxData[
      \(TraditionalForm\`\[Rho]\_\(\(\ \)\(n\)\)\)]],
  ". In the absence of more information it is rather cumbersome to compute  \
",
  Cell[BoxData[
      \(TraditionalForm\`\[Rho]\_n\)]],
  ":  we have to factor ",
  Cell[BoxData[
      \(TraditionalForm\`\[Pi]\_\(\(\ \)\(n\)\)\)]],
  " and then check for all these factors whether they already divide some ",
  Cell[BoxData[
      \(TraditionalForm\`\[Pi]\_\(\(\ \)\(k\)\)\)]],
  " for ",
  Cell[BoxData[
      \(TraditionalForm\`k < n\)]],
  ".  In order to reduce the search space, one can show that the critical \
part  ",
  Cell[BoxData[
      \(TraditionalForm\`\[Rho]\_m\)]],
  "  divides  ",
  Cell[BoxData[
      \(TraditionalForm\`\[Pi]\_n\)]],
  "  iff  ",
  Cell[BoxData[
      \(TraditionalForm\`\[Pi]\_m\)]],
  "  divides ",
  Cell[BoxData[
      \(TraditionalForm\`\[Pi]\_n\)]],
  "  iff  ",
  Cell[BoxData[
      \(TraditionalForm\`\(\(m\)\(\ \)\)\)]],
  " divides ",
  Cell[BoxData[
      \(TraditionalForm\`n\)]],
  ". Hence, it suffices to consider only all divisors  ",
  Cell[BoxData[
      \(TraditionalForm\`m\)]],
  " of ",
  Cell[BoxData[
      \(TraditionalForm\`n\)]],
  " when we try to eliminate irreducible factors that occur earlier on. A \
little more exploration finally produces the decomposition lemma:  let ",
  Cell[BoxData[
      \(TraditionalForm\`n\  = \ \(2\^k\) p\)]],
  "  where ",
  Cell[BoxData[
      \(TraditionalForm\`p\)]],
  " is odd. Then \n\n\t",
  Cell[BoxData[
      \(TraditionalForm\`\[Pi]\_\(\(\ \)\(n\)\) = \ 
        x\^\(\(\ \)\(2\^k - 1\)\)\ \[CenterDot]\ \(\[Product]\_\(\(\ \)\(d | \
p\)\)\(\[Rho]\_d\)(x\^\(2\^k\))\)\)]],
  "\n\t\nwhere the product is taken over all divisors of ",
  Cell[BoxData[
      \(TraditionalForm\`p\)]],
  ".  Hence we can use Moebius inversion to express the critical factors ",
  Cell[BoxData[
      \(TraditionalForm\`\[Rho]\_n\)]],
  " in terms of \[Pi]-polynomials:\n\n\t",
  Cell[BoxData[
      \(TraditionalForm\`\[Rho]\_n\%2\  = \ \ \[Product]\_\(d | n\)\ \
\[Pi]\_\(n/d\)\%\(\[Mu](d)\)\)]],
  "\n\nHere is a test:  "
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[BoxData[
    \(rho[n = 45]\)], "Input",
  CellLabel->"In[65]:=",
  AspectRatioFixed->True],

Cell["\<\
L = piFac[n/#]^MoebiusMu[#]& /@ Divisors[n]
Times@@L\
\>", "Input",
  CellLabel->"In[66]:="],

Cell[TextData[{
  "Note that the intermediate results are actually rational functions over ",
  Cell[BoxData[
      \(TraditionalForm\`\[DoubleStruckCapitalF]\_2\)]],
  ", rather than polynomials. In the end, though, everything cancels out and \
we are left with a polynomial. \n\nWe can also compute the degrees of the \
critical factors using the M\[ODoubleDot]bius function since the degree of ",
  Cell[BoxData[
      \(TraditionalForm\`\[Pi]\_n\)]],
  " is simply ",
  Cell[BoxData[
      \(TraditionalForm\`n - 1\)]],
  ". "
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[BoxData[{
    \(\(degrho[nn_Integer] := 
        Plus @@ \(\((\((nn/# - 1)\)\ MoebiusMu[#1] &)\) /@ 
            Divisors[nn]\);\)\), "\n", 
    \(L = degrho /@ Range[100]\)}], "Input",
  CellLabel->"In[68]:=",
  AspectRatioFixed->True],

Cell["\<\
A plot of these values should generate a feeling of d\[EAcute]j\
\[AGrave] vu for anyone with some exposure to elementary number theory. \
\
\>", "Text"],

Cell[BoxData[
    \(\(ListPlot[L, PlotStyle \[Rule] {Blue, PointSize[0.02]}];\)\)], "Input",\

  CellLabel->"In[70]:=",
  AspectRatioFixed->True],

Cell["\<\
Euler's totient function is hiding here, except for the value at 1.\
\
\>", "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[BoxData[
    \(Rest[L]\  === \ EulerPhi[Range[2, 100]]\)], "Input",
  CellLabel->"In[71]:=",
  AspectRatioFixed->True],

Cell[TextData[{
  "One can see that the degrees of the irreducible factors of ",
  Cell[BoxData[
      \(TraditionalForm\`\[Rho]\_n\)]],
  " are all the same.  It turns out that this degree is the suborder of 2 in \
",
  Cell[BoxData[
      \(TraditionalForm\`\[DoubleStruckCapitalZ]\_n\)]],
  ", i.e., the least exponent ",
  Cell[BoxData[
      \(TraditionalForm\`k\)]],
  " such that ",
  Cell[BoxData[
      \(TraditionalForm\`2\^\(\(\ \)\(k\)\) = \ \(\[PlusMinus]\ 
            1\)\ \((mod\ n)\)\)]],
  ".  Since the degree of the whole critical factor is ",
  Cell[BoxData[
      \(TraditionalForm\`\[Phi](n)\)]],
  ", the number of distinct irreducible factors is ",
  Cell[BoxData[
      \(TraditionalForm\`\(\[Phi](n)\)/2\)]],
  " divided by the suborder of 2.\nFor example, for ",
  Cell[BoxData[
      \(TraditionalForm\`n = 51\)]],
  " there are two irreducible components in the critical factor, each of \
degree 8."
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[BoxData[{
    \(pi[51] // fac2\), "\[IndentingNewLine]", 
    \(rho[51]\), "\[IndentingNewLine]", 
    \(EulerPhi[51]\), "\[IndentingNewLine]", 
    \(suborder2[51]\)}], "Input",
  CellLabel->"In[72]:="],

Cell["The other factors are due to the divisors 3 and 17. ", "Text"],

Cell[BoxData[{
    \(pi[3]\  // \ fac2\), "\[IndentingNewLine]", 
    \(pi[17] // fac2\)}], "Input",
  CellLabel->"In[76]:="],

Cell[TextData[{
  "Likewise, for the prime ",
  Cell[BoxData[
      \(TraditionalForm\`n = 61\)]],
  " there is only one factor of degree 30 in the critical part. "
}], "Text"],

Cell[BoxData[{
    \(pi[61] // fac2\), "\[IndentingNewLine]", 
    \(rho[61]\), "\[IndentingNewLine]", 
    \(EulerPhi[61]\), "\[IndentingNewLine]", 
    \(suborder2[61]\)}], "Input",
  CellLabel->"In[78]:="]
}, Closed]],

Cell[CellGroupData[{

Cell["Computing Depth", "Subsection",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  "So far, depth is only defined for those irreducible factors that divide \
some ",
  Cell[BoxData[
      \(TraditionalForm\`\[Pi]\)]],
  " polynomial.  As it turns out, all irreducible factors divide some ",
  Cell[BoxData[
      \(TraditionalForm\`\[Pi]\)]],
  " polynomial. The proof makes use of the fact that the quotient ring ",
  Cell[BoxData[
      \(TraditionalForm\`\[DoubleStruckCapitalF]\_2[x]/\((p)\)\)]],
  "  is a field for every irreducible polynomial ",
  Cell[BoxData[
      \(TraditionalForm\`p\)]],
  ". In fact, this field is isomorphic to  ",
  Cell[BoxData[
      \(TraditionalForm\`\[DoubleStruckCapitalF]\_\(2\^k\)\)]],
  " where ",
  Cell[BoxData[
      \(TraditionalForm\`k\)]],
  " is the degree of ",
  Cell[BoxData[
      \(TraditionalForm\`p\)]],
  ".  If we think of the recursion as taking place over this field , the \
sequence of ",
  Cell[BoxData[
      \(TraditionalForm\`\[Pi]\)]],
  " polynomials is by necessity periodic since there are only finitely many \
possible values. But because of the fact that the recursion can be run \
backwards, the sequence cannot have a transient part.  Hence, it is cyclic \
and for some ",
  Cell[BoxData[
      \(TraditionalForm\`n\  > \ 0\)]],
  " we have ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          FormBox[\(\[Pi]\_n\),
            "TraditionalForm"], " ", "=", " ", \(0\ mod\ p\)}], 
        TraditionalForm]]],
  ".  In other words, ",
  Cell[BoxData[
      \(TraditionalForm\`p\)]],
  " divides ",
  Cell[BoxData[
      \(TraditionalForm\`\[Pi]\_n\)]],
  ". \n\nHere is an example: "
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[BoxData[
    \(p = 1 + x + x\^4\)], "Input",
  CellLabel->"In[82]:=",
  AspectRatioFixed->True],

Cell[BoxData[
    \(With[\ {res\  = \ recurrence[p, DoTrace \[Rule] True]}, 
      TableForm[\ \ res, \ 
        TableHeadings \[Rule] {Range[0, Length[res] - 1]}]]\)], "Input",
  CellLabel->"In[83]:=",
  AspectRatioFixed->True],

Cell[TextData[{
  "Note that in the last argument, ",
  Cell[BoxData[
      \(TraditionalForm\`x\)]],
  " plays the role of a root of ",
  Cell[BoxData[
      \(TraditionalForm\`p\)]],
  ".  Using a little more algebra this fact can be exploited to show the the \
depth ",
  Cell[BoxData[
      \(TraditionalForm\`d\)]],
  " of an irreducible polynomial ",
  Cell[BoxData[
      \(TraditionalForm\`p\)]],
  " divides ",
  Cell[BoxData[
      \(TraditionalForm\`\(\(2\^k\)\(+\)\(\ \)\(1\)\(\ \)\)\)]],
  "or ",
  Cell[BoxData[
      \(TraditionalForm\`2\^k - \ 1\)]],
  ", where ",
  Cell[BoxData[
      \(TraditionalForm\`k\)]],
  " is the degree of ",
  Cell[BoxData[
      \(TraditionalForm\`p\)]],
  ".  More precisely, ",
  Cell[BoxData[
      \(TraditionalForm\`k\)]],
  " is the suborder of ",
  Cell[BoxData[
      \(TraditionalForm\`2\)]],
  " in the multiplicative group ",
  Cell[BoxData[
      \(TraditionalForm\`\[DoubleStruckCapitalZ]\_\(\(\ \)\(d\)\)\)]],
  ".  For example, for one of the critical factors of ",
  Cell[BoxData[
      \(TraditionalForm\`\[Pi]\_51\)]],
  " we have"
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[BoxData[{
    \(\(p = 1 + x\^2 + x\^3 + x\^4 + x\^8;\)\), "\n", 
    \(Mod[\ 2^suborder2[51]\  - \ 1, \ depth[p]]\)}], "Input",
  CellLabel->"In[84]:=",
  AspectRatioFixed->True],

Cell[TextData[{
  "Here is a more elaborate example. Let us consider the field  ",
  Cell[BoxData[
      \(TraditionalForm\`\[DoubleStruckCapitalF]\_256\)]],
  ", the splitting field of irreducible polynomials of degree 8. Using \
cyclotomic polynomials (which, incidentally have many properties similar to \
the ",
  Cell[BoxData[
      \(TraditionalForm\`\[Pi]\)]],
  "-polynomials) we can decompose  ",
  Cell[BoxData[
      \(TraditionalForm\`x\^255 + \ 1\)]],
  "  into irreducible factors as follows. "
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["\<\
k = 8; 
q = 2^k - 1; 
pols = (fac2[Cyclotomic[#,x]] & ) /@ Divisors[q]; 
TableForm[Pairs[Divisors[q], pols]]\
\>", "Input",
  CellLabel->"In[86]:=",
  AspectRatioFixed->True],

Cell["Here are all the degree 8 factors in one list:", "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[BoxData[
    \(pols8 = Flatten[toList /@ Drop[pols, 4]]\)], "Input",
  CellLabel->"In[90]:=",
  AspectRatioFixed->True],

Cell["\<\
Check that all 30 irreducible polynomials of degree 8 occur in this \
way: \
\>", "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[BoxData[
    \(NumberIrr[8] === \ Length[pols8]\)], "Input",
  CellLabel->"In[91]:=",
  AspectRatioFixed->True],

Cell[TextData[{
  "The depths are all divisors of  q ",
  Cell[BoxData[
      \(TraditionalForm\`\(\[PlusMinus]\ 1\)\)]],
  ":"
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[BoxData[{
    \(Divisors[q]\), "\n", 
    \(Divisors[q + 2]\)}], "Input",
  CellLabel->"In[92]:=",
  AspectRatioFixed->True],

Cell[BoxData[
    \(dd = depth /@ pols8\)], "Input",
  CellLabel->"In[94]:=",
  AspectRatioFixed->True],

Cell[TextData[{
  "And, the suborder of 2 in ",
  Cell[BoxData[
      \(TraditionalForm\`\[DoubleStruckCapitalZ]\_d\)]],
  " is 8, for all these depths ",
  Cell[BoxData[
      \(TraditionalForm\`d\)]],
  ". "
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[BoxData[
    \(suborder2 /@ dd\)], "Input",
  CellLabel->"In[95]:=",
  AspectRatioFixed->True],

Cell[TextData[{
  "If we associate the polynomials with their depths, on can see that all \
polynomials of the form  ",
  Cell[BoxData[
      \(TraditionalForm\`1\  + \ x\  + \ \[Ellipsis]\  + \ x\^8\)]],
  "  have depth dividing  ",
  Cell[BoxData[
      \(TraditionalForm\`2\^8 + \ 1\)]],
  ", whereas all others missing the linear term have depth dividing  ",
  Cell[BoxData[
      \(TraditionalForm\`2\^8 - \ 1\)]],
  ". The next table shows all the 30 irreducible polynomials of degree 8, \
indicates the permutation induced by the involution ",
  Cell[BoxData[
      \(TraditionalForm\`x\  \[RightTeeArrow] \ x\  + 1\)]],
  ", and shows their depths. "
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[BoxData[{
    \(\(perm = PositionList[pols8, Phi /@ pols8];\)\), "\n", 
    \(TableForm[Thread[{Range[30], perm, dd, pols8}]]\)}], "Input",
  CellLabel->"In[96]:=",
  AspectRatioFixed->True]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Additive Cellular Automata", "Section",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  "The linear map  ",
  Cell[BoxData[
      \(TraditionalForm\`\[Sigma]\  : \ \[ScriptCapitalC]\ \[LongRightArrow]\ \
\[ScriptCapitalC]\)]],
  "  has a locality property: in order to compute ",
  Cell[BoxData[
      FormBox[
        RowBox[{\(\[Sigma](X)\), \((v)\), Cell[""]}], TraditionalForm]]],
  ", one only needs to know the values of ",
  Cell[BoxData[
      \(TraditionalForm\`X\)]],
  " in the neighborhood of ",
  Cell[BoxData[
      \(TraditionalForm\`v\)]],
  ". The same locality property is crucial in cellular automata, and \
responsible for the continuity of the global maps. Hence, we can think of \
\[Sigma] on a graph ",
  Cell[BoxData[
      \(TraditionalForm\`G\)]],
  " as a kind of generalized cellular automaton, a discrete dynamical system \
",
  Cell[BoxData[
      \(TraditionalForm\`\[LeftAngleBracket]G, \
\[Sigma]\[RightAngleBracket]\)]],
  ". Indeed, in the special case where the graph is a path (finite or \
infinite) or cycle, we get an elementary cellular automaton that has been \
studied extensively.  The operator \[Sigma] here corresponds to Wolfram's \
rule 150, and ",
  Cell[BoxData[
      \(TraditionalForm\`\(\[Sigma]\^-\)\)]],
  " corresponds to rule 90. "
}], "Text"],

Cell[TextData[{
  "From the dynamical systems point of view, it is interesting to analyze the \
functional digraph of \[Sigma], i.e., the graph whose vertices are all \
patterns, and whose edges have the form ",
  Cell[BoxData[
      \(TraditionalForm\`\((X, \[Sigma](X))\)\)]],
  ".  In the finite cases, it is clear that the connected components of this \
graph are unicyclic, and the nontrivial strongly connected components are but \
the limit cycles of the system. The linearity of \[Sigma] makes the analysis \
of these systems particularly easy, at least compared to non-linear ones. As \
is standard in the cellular automata literature, linearity is referred to as \
additivity, and the fact that ",
  Cell[BoxData[
      \(TraditionalForm\`\[Sigma](X + Y)\  = \ \[Sigma](X) + \[Sigma](Y)\)]],
  " is often called the superposition principle. "
}], "Text"],

Cell[TextData[{
  "At any rate, in the finite case it follows from algebra that we can split \
\[Sigma] into a nilpotent and a automorphism part. More precisely, there is \
an iteration factor ",
  Cell[BoxData[
      \(TraditionalForm\`m\)]],
  " such that  ",
  Cell[BoxData[
      \(TraditionalForm\`\[ScriptCapitalC]\  = \ \[ScriptCapitalC]\_n\ \
\[CirclePlus]\ \[ScriptCapitalC]\_a\)]],
  "  where ",
  Cell[BoxData[
      \(TraditionalForm\`\[ScriptCapitalC]\_n\  = \ 
        ker\ \[Sigma]\^\(\(\ \)\(m\)\)\)]],
  "  and  ",
  Cell[BoxData[
      \(TraditionalForm\`\[ScriptCapitalC]\_\(\(\ \)\(a\)\)\  = \ 
        im\ \[Sigma]\^\(\(\ \)\(m\)\)\)]],
  ".  The restrictions ",
  Cell[BoxData[
      \(TraditionalForm\`\[Sigma]\_n\)]],
  " and ",
  Cell[BoxData[
      \(TraditionalForm\`\[Sigma]\_a\)]],
  " of \[Sigma] to ",
  Cell[BoxData[
      \(TraditionalForm\`\[ScriptCapitalC]\_n\)]],
  " and ",
  Cell[BoxData[
      \(TraditionalForm\`\[ScriptCapitalC]\_\(\(\ \)\(a\)\)\)]],
  " are nilpotent and an automorphism, respectively. The functional digraphs \
of these two restrictions are particularly simple:  for ",
  Cell[BoxData[
      \(TraditionalForm\`\[Sigma]\_n\)]],
  " we have a unicyclic tree (an ordinary tree whose root has a self-loop), \
and for ",
  Cell[BoxData[
      \(TraditionalForm\`\[Sigma]\_a\)]],
  " we have a collection of cycles. The digraph for \[Sigma] is then just the \
direct product of these two graphs, where in effect every node on the limit \
cycles has been replaced by a copy of the kernel tree. "
}], "Text"],

Cell[TextData[{
  "As an example, let us consider the path of length 11, with fixed boundary \
conditions. The corank of the matrix turns out to be 10, and for ",
  Cell[BoxData[
      \(TraditionalForm\`\[Sigma]\^8\)]],
  " we have maximal corank 8."
}], "Text"],

Cell[BoxData[{
    \(ClearAll[S, M]\), "\n", 
    \(\(M = SigmaMatrix[11];\)\), "\n", 
    \(\(Scan[\((S[#1] = mod2[MatrixPower[M, #1]])\) &, 
        Range[10]];\)\), "\n", 
    \(CoRank2 /@ \(S /@ Range[10]\)\)}], "Input",
  CellLabel->"In[98]:="],

Cell[TextData[{
  "We can compute the null-space of ",
  Cell[BoxData[
      \(TraditionalForm\`\[Sigma]\_11\%8\)]],
  ", and verify that these patterns do in fact have the right properties."
}], "Text"],

Cell[BoxData[
    \(Cn\  = \ NullSpace2[S[8]]\)], "Input",
  CellLabel->"In[102]:="],

Cell[TextData[{
  "The orbits of the patterns in ",
  Cell[BoxData[
      \(TraditionalForm\`\[ScriptCapitalC]\_n\)]],
  " end in a fixed point, though the transients vary."
}], "Text"],

Cell[BoxData[
    \(\(AnalyzeOrbit[\ Sigma, #\ ] &\)\  /@ \ Cn\)], "Input",
  CellLabel->"In[103]:="],

Cell[BoxData[{
    \(\(Table[\ 
        PlotMatrix[NestList[\ Sigma, Cn[\([i]\)], \ 8\ ], 
          DisplayFunction \[Rule] Identity, GridLines \[Rule] True], {i, 
          8}];\)\), "\[IndentingNewLine]", 
    \(\(ShowArray[%, 4];\)\)}], "Input",
  CellLabel->"In[176]:="],

Cell[TextData[{
  "For the other direct summand we can either determine the image of ",
  Cell[BoxData[
      \(TraditionalForm\`\[Sigma]\_11\%8\)]],
  " directly from the matrix, or, alternatively, find an independent set ",
  Cell[BoxData[
      \(TraditionalForm\`B\)]],
  "  that extends the basis of ",
  Cell[BoxData[
      \(TraditionalForm\`\[ScriptCapitalC]\_n\)]],
  " to a basis for the whole space. Applying ",
  Cell[BoxData[
      \(TraditionalForm\`\[Sigma]\_11\%8\)]],
  " to ",
  Cell[BoxData[
      \(TraditionalForm\`B\)]],
  " produces a basis for ",
  Cell[BoxData[
      \(TraditionalForm\`\[ScriptCapitalC]\_a\)]],
  "."
}], "Text"],

Cell[BoxData[{
    \(\(B\  = \ SteinitzComplement[Cn];\)\), "\n", 
    \(Ca\  = \ Thread[S[k] . Thread[B]]\)}], "Input",
  CellLabel->"In[106]:="],

Cell[TextData[{
  "The patterns in ",
  Cell[BoxData[
      \(TraditionalForm\`\[ScriptCapitalC]\_a\)]],
  " behave properly, their orbits have only a periodic part."
}], "Text"],

Cell[BoxData[
    \(AnalyzeOrbit[\ Sigma, mod2[\ Ca[\([1]\)] + Ca[\([2]\)]]\ ]\)], "Input",
  CellLabel->"In[108]:="],

Cell["\<\
Since \[Sigma] is self-adjoint, the two spaces ought to be \
orthogonal, another property that is easily verified. \
\>", "Text"],

Cell[BoxData[
    \(\(Apply[\ Dot, \ CartesianProduct[Ca, Cn], \ {1}] // \ mod2\)\  // \ 
      Union\)], "Input",
  CellLabel->"In[109]:="],

Cell["\<\
General patterns are just a linear combination of patterns from \
these subspaces.\
\>", "Text"],

Cell[BoxData[
    \(\(NestList[\ Sigma, mod2[Ca[\([1]\)] + Cn[\([1]\)]], \ 12\ ] // 
        PlotMatrix;\)\)], "Input",
  CellLabel->"In[110]:="],

Cell[TextData[{
  "To obtain a complete description of the diagram of \[Sigma] we only need \
to determine the limit cycles, and the tree corresponding to co-orbit of ",
  Cell[BoxData[
      FormBox[
        StyleBox["0",
          FontWeight->"Bold"], TraditionalForm]]],
  ", the kernel of ",
  Cell[BoxData[
      \(TraditionalForm\`\[Sigma]\^8\)]],
  ". The limit cycles are easy."
}], "Text"],

Cell[BoxData[{
    \(\(A\  = \(\((Thread[Ca] . #)\) &\)\  /@ \ 
          Tuples[{0, 1}, 3];\)\), "\n", 
    \(\(\(AnalyzeOrbit[\ 
              Function[x, Sigma[x, Full \[Rule] True]], #] &\)\  /@ \ 
          A\  // \ Frequencies\)\  // \ \(Curry[TableForm]\)[
        TableDepth \[Rule] 2]\)}], "Input",
  CellLabel->"In[111]:="],

Cell[TextData[{
  "Thus, there are 2 fixed points, a 2-cycle, and a 4-cycle. The tree is \
rather large in this example. Since the corank of \[Sigma] is 1, the tree is \
binary, and we have seen that it has depth ",
  Cell[BoxData[
      \(TraditionalForm\`8\)]],
  ". In fact, it must be a complete binary tree of depth 8 for cardinality \
reasons.  Here is the analysis of the functional digraph."
}], "Text"],

Cell[BoxData[{
    \(\(n\  = \ 11;\)\), "\n", 
    \(\(V\  = \ Tuples[\ {0, 1}, n];\)\), "\n", 
    \(\(VV\  = \ \(Sigma[#, Full \[Rule] True] &\)\  /@ \ V;\)\), "\n", 
    \(\(ed\  = \ Pairs[Range[2^n], PositionList[\ V, \ VV\ ]];\)\), "\n", 
    \(\(G\  = \ Graph[\ 2^n, 2^n, "\<EList\>", ed];\)\), "\n", 
    \(\(scc\  = \ 
        StronglyConnectedComponents[G, 
          NonTrivial \[Rule] True];\)\)}], "Input",
  CellLabel->"In[113]:="],

Cell["\<\
There are 4 non-trivial strongly connected components, \
corresponding to the 4 limit cycles.\
\>", "Text"],

Cell[BoxData[
    \(scc\)], "Input",
  CellLabel->"In[119]:="],

Cell["The longest transient in the graph has length 8.", "Text"],

Cell[BoxData[
    \(LongestPath[G]\)], "Input",
  CellLabel->"In[120]:="],

Cell["\<\
A backwards traversal from one of the fixed points produces 256 \
nodes, the same holds for the other fixed point.\
\>", "Text"],

Cell[BoxData[
    \(WeaklyConnectedComponents[\ ReverseGraph[G], scc[\([1]\)]] // 
      Length\)], "Input",
  CellLabel->"In[121]:="],

Cell["\<\
The 2-cycle is weakly connected to 512 points and the 4-cycle to \
1024 points.\
\>", "Text"],

Cell[BoxData[
    \(WeaklyConnectedComponents[\ ReverseGraph[G], scc[\([\(-2\)]\)]] // 
      Length\)], "Input",
  CellLabel->"In[122]:="],

Cell[BoxData[
    \(\(PlotMatrix[
        V\[LeftDoubleBracket]
          scc\[LeftDoubleBracket]2\[RightDoubleBracket]\[RightDoubleBracket], 
        GridLines \[Rule] True];\)\)], "Input",
  CellLabel->"In[123]:="],

Cell[BoxData[
    \(\(PlotMatrix[
        V\[LeftDoubleBracket]
          scc\[LeftDoubleBracket]3\[RightDoubleBracket]\[RightDoubleBracket], 
        GridLines \[Rule] True];\)\)], "Input",
  CellLabel->"In[124]:="]
}, Closed]],

Cell[CellGroupData[{

Cell["Back To Pebbles", "Section",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["The Chessboard", "Subsubsection",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["\<\
We have not yet explained how to actually compute the solution to \
our pebbling problem, without resorting to searches. In the case of an \
ordinary chessboard, symmetry would reduce the search space to size 16, but \
for larger examples, and situations where \[Sigma] is not invertible we need \
better tools. Here is the general method, applied to the original 8 by 8 \
chessboard.  \
\>", "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  "Recall from above that the affine extension operator for the all-ones \
pattern, starting at first row ",
  Cell[BoxData[
      \(TraditionalForm\`X\)]],
  ", is given by\n\n\t",
  Cell[BoxData[
      FormBox[
        RowBox[{
          RowBox[{
            SubscriptBox["ext", 
              StyleBox["1",
                FontWeight->"Bold"]], "(", \(X, n\), ")"}], " ", "=", " ", 
          
          RowBox[{\(\(\(\[Pi]\_\(\(\ \)\(n\)\)\)(\[Sigma])\) \((X)\)\), " ", 
            "+", " ", 
            RowBox[{\(\(\[Tau]\_\(n - 1\)\)(\[Sigma])\), 
              RowBox[{"(", 
                SubscriptBox[
                  StyleBox["1",
                    FontWeight->"Bold"], "m"], 
                StyleBox[")",
                  FontWeight->"Plain"]}]}]}]}], TraditionalForm]]],
  ".\n\t\nWe need to find a row pattern ",
  Cell[BoxData[
      \(TraditionalForm\`X\)]],
  " such that  ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          RowBox[{
            SubscriptBox["ext", 
              StyleBox["1",
                FontWeight->"Bold"]], "(", \(X, n + 1\), ")"}], "=", 
          StyleBox["0",
            FontWeight->"Bold"]}], TraditionalForm]]],
  ".  Let's write this equation as  ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          StyleBox["0",
            FontWeight->"Bold"], " ", "=", " ", \(M\[CenterDot]X\  + \ A\)}], 
        TraditionalForm]]],
  ", so we have to compute a solution to ",
  Cell[BoxData[
      \(TraditionalForm\`M\[CenterDot]X = A\)]],
  "  where ",
  Cell[BoxData[
      FormBox[
        RowBox[{"A", " ", "=", 
          RowBox[{\(\(\[Tau]\_n\)(\[Sigma])\), 
            RowBox[{"(", 
              StyleBox["1",
                FontWeight->"Bold"], 
              StyleBox[")",
                FontWeight->"Plain"]}]}]}], TraditionalForm]]],
  ".\nFor the 8 by 8 board, ",
  Cell[BoxData[
      \(TraditionalForm\`M\)]],
  " is non-singular, so we can use matrix inversion to tackle this problem. \
First, we compute ",
  Cell[BoxData[
      \(TraditionalForm\`M\)]],
  " to make sure the solution is indeed unique."
}], "Text"],

Cell[BoxData[{
    \(\(M = PolynomialToMatrix[\ 9, 8\ ];\)\), "\n", 
    \(\(PlotMatrix[M];\)\)}], "Input",
  CellLabel->"In[125]:=",
  AspectRatioFixed->True],

Cell["\<\
Clearly, this matrix has full rank and we can compute it's inverse \
rather than just solving the system of linear equations.\
\>", "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[BoxData[{
    \(\(Minv = Inverse[M, Modulus \[Rule] 2];\)\), "\n", 
    \(\(A\  = \ 
        mod2[\ PolynomialToMatrix[\ \ tau[8], \ 8, \ 
              Full \[Rule] True\ ]\  . \ One[8]\ ];\)\), "\n", 
    \(X\  = \ Minv . A\)}], "Input",
  CellLabel->"In[127]:=",
  AspectRatioFixed->True],

Cell["\<\
Here is the pattern that will leave all pebbles white side up. \
\
\>", "Text"],

Cell[BoxData[{
    \(\(pat\  = \ 
        ExtendPatternOne[\ X, 8, \ 
          Full \[Rule] True];\)\), "\[IndentingNewLine]", 
    \(\(PlotMatrix[pat, GridLines \[Rule] True];\)\)}], "Input",
  CellLabel->"In[130]:="],

Cell["\<\
The pattern is completely symmetric, as required by the uniqueness \
of the solution. We can verify that we really have a solution to the pebbling \
problem by applying \[Sigma].\
\>", "Text"],

Cell[BoxData[
    \(\(Sigma[pat] // \ Flatten\)\  // \ Union\)], "Input",
  CellLabel->"In[132]:="]
}, Closed]],

Cell[CellGroupData[{

Cell["40 by 40", "Subsubsection",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  "Here is a more interesting case, at least with regard to visuals: the ",
  Cell[BoxData[
      \(TraditionalForm\`40\[Cross]40\)]],
  " grid.  As in the previous case, we compute ",
  Cell[BoxData[
      \(TraditionalForm\`\(\[Pi]\_41\)(\[Sigma]\_40)\)]],
  ", verify that the matrix is non-singular, and abuse matrix inversion to \
obtain the seed pattern for the all-ones configuration. "
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[BoxData[{
    \(\(M = PolynomialToMatrix[41, 40];\)\), "\n", 
    \(\(PlotMatrix[M];\)\), "\[IndentingNewLine]", 
    \(Rank2[M]\)}], "Input",
  CellLabel->"In[133]:=",
  AspectRatioFixed->True],

Cell["Now for the inverse.  Any guesses about the next picture? ", "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[BoxData[{
    \(\(Minv = Inverse[M, Modulus \[Rule] 2];\)\), "\n", 
    \(\(PlotMatrix[Minv];\)\)}], "Input",
  CellLabel->"In[136]:=",
  AspectRatioFixed->True],

Cell["\<\
This time, the pebbling pattern exhibits a nice lacy structure. \
\
\>", "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[BoxData[{
    \(\(A\  = \ 
        mod2[\ PolynomialToMatrix[\ \ tau[40], \ 40, \ 
              Full \[Rule] True\ \ ]\  . \ One[40]\ ];\)\), "\n", 
    \(\(X\  = \ mod2[Minv . A];\)\), "\n", 
    \(\(PlotMatrix[
        ExtendPatternOne[X, 40, \ Full \[Rule] True]];\)\)}], "Input",
  CellLabel->"In[138]:="]
}, Closed]],

Cell[CellGroupData[{

Cell["41 by 40", "Subsubsection",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["Let's expand the last grid by one more row. ", "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[BoxData[{
    \(\(M = PolynomialToMatrix[42, 40];\)\), "\n", 
    \(\(PlotMatrix[M];\)\)}], "Input",
  CellLabel->"In[141]:=",
  AspectRatioFixed->True],

Cell["\<\
Again, the matrix is non-singular and we its inverse to obtain a \
suitable seed configuration.  \
\>", "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[BoxData[{
    \(\(Minv = Inverse[M, Modulus \[Rule] 2];\)\), "\n", 
    \(\(PlotMatrix[Minv];\)\)}], "Input",
  CellLabel->"In[143]:=",
  AspectRatioFixed->True],

Cell["The seed and the pebbling pattern.", "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[BoxData[{
    \(\(A\  = \ 
        mod2[\ PolynomialToMatrix[\ \ tau[41], \ 40, 
              Full \[Rule] True\ ]\  . \ One[40]\ ];\)\), "\n", 
    \(\(X\  = \ mod2[Minv . A];\)\), "\n", 
    \(\(PlotMatrix[ExtendPatternOne[X, 41, Full \[Rule] True]];\)\)}], "Input",\

  CellLabel->"In[145]:="]
}, Closed]],

Cell[CellGroupData[{

Cell["11 by 11", "Subsubsection",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  "As an example for a case where the solution fails to be unique, and \
therefore may fail to be nicely symmetric as in the previous cases,  consider \
the ",
  Cell[BoxData[
      \(TraditionalForm\`11\[Cross]11\)]],
  " grid. "
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[BoxData[{
    \(\(M = PolynomialToMatrix[12, 11, \ Full \[Rule] True];\)\), "\n", 
    \(Rank2[M]\)}], "Input",
  CellLabel->"In[148]:=",
  AspectRatioFixed->True],

Cell[TextData[{
  "The rank of ",
  Cell[BoxData[
      \(TraditionalForm\`M\)]],
  " this time is only 5, hence there are ",
  Cell[BoxData[
      \(TraditionalForm\`2\^6\)]],
  " solutions.  We can expand a basis for the null space of ",
  Cell[BoxData[
      \(TraditionalForm\`M\)]],
  " to a base for the kernel patterns. "
}], "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["\<\
null = NullSpace2[M]
base = ExtendPatternZero[#,11, Full\[Rule]True]& /@ null; 
grf = PlotMatrix[#,DisplayFunction->Identity]& /@ base; 
ShowArray[grf,3]; \
\>", "Input",
  CellLabel->"In[150]:=",
  AspectRatioFixed->True],

Cell["\<\
All patterns in the kernel are now obtained as linear combinations \
of these 6.  Here are some examples. \
\>", "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["\<\
sols = mod2[Transpose[base,{3,1,2}].#]& /@ \
Rest[Take[Tuples[{0,1},6],17]]; 
grf = PlotMatrix[#,DisplayFunction->Identity]& /@ sols; 
ShowArray[grf, 4, GraphicsSpacing->{0.2,0.1}]; \
\>", "Input",
  CellLabel->"In[154]:=",
  AspectRatioFixed->True],

Cell["This time we have to solve a linear system. ", "Text",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[BoxData[{
    \(\(A\  = \ 
        mod2[\ PolynomialToMatrix[\ \ tau[11], \ 11\ , 
              Full \[Rule] True]\  . \ One[11]\ ];\)\), "\n", 
    \(\(X\  = \ LinearSolve[\ M, \ A, \ Modulus \[Rule] 2\ ];\)\), "\n", 
    \(\(pat = ExtendPatternOne[X, 11, Full \[Rule] True];\)\), "\n", 
    \(\(gr1\  = \ PlotMatrix[pat, GridLines \[Rule] True];\)\)}], "Input",
  CellLabel->"In[157]:="],

Cell["\<\
Of course, we can add any first-row pattern from the kernel of \
\[Sigma] to obtain other solutions. \
\>", "Text"],

Cell[BoxData[
    \(\(gr2\  = 
        PlotMatrix[
          ExtendPatternOne[\ mod2[\ X\  + \ sols[\([2, 1]\)]], 11, 
            Full \[Rule] True], \ GridLines \[Rule] True\ ];\)\)], "Input",
  CellLabel->"In[161]:="],

Cell[BoxData[
    \(\(\(\(sols = \(mod2[Transpose[null] . #] &\) /@ 
          Tuples[\ {0, 1}, 6];\)\n
    \(\(PlotMatrix[\ ExtendPatternOne[mod2[X + #], 11, Full \[Rule] True], 
            DisplayFunction \[Rule] \ Identity] &\)\  /@ \ sols;\)\ \n
    \(ShowArray[%, \ 8, \ 
        GraphicsSpacing \[Rule] \ {0.2, 0.1}];\)\)\(\ \)\)\)], "Input",
  CellLabel->"In[162]:="],

Cell[BoxData[""], "Input",
  CellLabel->"In[165]:="]
}, Closed]]
}, Closed]]
}, Open  ]]
},
FrontEndVersion->"5.0 for X",
ScreenRectangle->{{0, 1280}, {0, 1024}},
ScreenStyleEnvironment->"Working",
WindowToolbars->{},
WindowSize->{1016, 740},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
PrintingPageRange->{Automatic, Automatic},
PrintingOptions->{"PaperSize"->{612, 792},
"PaperOrientation"->"Portrait",
"PostScriptOutputFile":>FrontEnd`FileName[{$RootDirectory, "home", "sutner", \
"research", "CompAlg", "Chebyshev"}, "FibonacciP.nb.ps", CharacterEncoding -> \
"iso8859-1"],
"Magnification"->1},
PrivateNotebookOptions->{"ColorPalette"->{RGBColor, 128}},
ShowSelection->True,
ShowCellLabel->True,
ShowCellTags->False,
ImageSize->{200, 200},
RenderingOptions->{"ObjectDithering"->True,
"RasterDithering"->False},
CharacterEncoding->Automatic,
Magnification->1.5,
StyleDefinitions -> "ArticleClassic.nb"
]

(*******************************************************************
Cached data follows.  If you edit this Notebook file directly, not
using Mathematica, you must remove the line containing CacheID at
the top of  the file.  The cache data will then be recreated when
you save this file from within Mathematica.
*******************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{
Cell[1754, 51, 48, 0, 41, "SmallText"],

Cell[CellGroupData[{
Cell[1827, 55, 136, 3, 211, "Title",
  Evaluatable->False],
Cell[1966, 60, 233, 8, 61, "Text",
  Evaluatable->False],

Cell[CellGroupData[{
Cell[2224, 72, 38, 0, 45, "Subsubsection"],
Cell[2265, 74, 330, 9, 61, "Text"],
Cell[2598, 85, 121, 3, 82, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2756, 93, 90, 2, 90, "Section",
  Evaluatable->False],

Cell[CellGroupData[{
Cell[2871, 99, 38, 0, 67, "Subsection"],
Cell[2912, 101, 900, 15, 176, "Text",
  Evaluatable->False],
Cell[3815, 118, 523, 11, 107, "Text",
  Evaluatable->False]
}, Closed]],

Cell[CellGroupData[{
Cell[4375, 134, 37, 0, 67, "Subsection"],
Cell[4415, 136, 2997, 79, 386, "Text",
  Evaluatable->False],
Cell[7415, 217, 2333, 62, 223, "Text",
  Evaluatable->False]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[9797, 285, 85, 2, 49, "Section",
  Evaluatable->False],
Cell[9885, 289, 551, 17, 61, "Text",
  Evaluatable->False],
Cell[10439, 308, 140, 3, 40, "Input"],
Cell[10582, 313, 479, 13, 85, "Text"],
Cell[11064, 328, 97, 2, 40, "Input"],
Cell[11164, 332, 134, 3, 40, "Input"],
Cell[11301, 337, 108, 2, 40, "Input"],
Cell[11412, 341, 785, 19, 109, "Text",
  Evaluatable->False],
Cell[12200, 362, 154, 3, 40, "Input"],
Cell[12357, 367, 155, 5, 38, "Text",
  Evaluatable->False],
Cell[12515, 374, 156, 3, 40, "Input"],
Cell[12674, 379, 442, 13, 61, "Text",
  Evaluatable->False],
Cell[13119, 394, 87, 2, 40, "Input"],
Cell[13209, 398, 583, 17, 84, "Text",
  Evaluatable->False],
Cell[13795, 417, 129, 3, 40, "Input"],
Cell[13927, 422, 108, 2, 40, "Input"],
Cell[14038, 426, 303, 10, 38, "Text",
  Evaluatable->False],
Cell[14344, 438, 108, 2, 40, "Input"],
Cell[14455, 442, 108, 2, 40, "Input"],
Cell[14566, 446, 540, 14, 84, "Text"],
Cell[15109, 462, 222, 5, 63, "Input"],
Cell[15334, 469, 50, 0, 38, "Text"],
Cell[15387, 471, 100, 2, 40, "Input"],
Cell[15490, 475, 180, 5, 38, "Text"],
Cell[15673, 482, 242, 5, 63, "Input"],
Cell[15918, 489, 254, 6, 61, "Text"],
Cell[16175, 497, 100, 2, 40, "Input"],
Cell[16278, 501, 382, 9, 61, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[16697, 515, 98, 2, 49, "Section",
  Evaluatable->False],

Cell[CellGroupData[{
Cell[16820, 521, 43, 0, 45, "Subsubsection"],
Cell[16866, 523, 3550, 105, 452, "Text",
  Evaluatable->False],
Cell[20419, 630, 178, 4, 63, "Input"],
Cell[20600, 636, 245, 6, 61, "Text",
  Evaluatable->False],
Cell[20848, 644, 158, 3, 63, "Input"],
Cell[21009, 649, 217, 7, 38, "Text",
  Evaluatable->False],
Cell[21229, 658, 94, 2, 40, "Input"],
Cell[21326, 662, 772, 25, 61, "Text",
  Evaluatable->False],
Cell[22101, 689, 860, 27, 85, "Text",
  Evaluatable->False],
Cell[22964, 718, 317, 8, 61, "Text",
  Evaluatable->False],
Cell[23284, 728, 300, 6, 86, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[23621, 739, 53, 0, 31, "Subsubsection"],
Cell[23677, 741, 2190, 64, 268, "Text",
  Evaluatable->False],
Cell[25870, 807, 164, 4, 63, "Input"],
Cell[26037, 813, 175, 7, 38, "Text",
  Evaluatable->False],
Cell[26215, 822, 121, 3, 40, "Input"],
Cell[26339, 827, 420, 12, 70, "Text",
  Evaluatable->False],
Cell[26762, 841, 90, 2, 39, "Input"],
Cell[26855, 845, 1024, 31, 107, "Text",
  Evaluatable->False],
Cell[27882, 878, 156, 4, 40, "Input"],
Cell[28041, 884, 1613, 45, 245, "Text"],
Cell[29657, 931, 305, 6, 109, "Input"],
Cell[29965, 939, 396, 7, 109, "Input"],
Cell[30364, 948, 1964, 52, 245, "Text"],
Cell[32331, 1002, 286, 6, 86, "Input"],
Cell[32620, 1010, 266, 6, 86, "Input"],
Cell[32889, 1018, 52, 0, 38, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[32978, 1023, 104, 2, 31, "Subsubsection",
  Evaluatable->False],
Cell[33085, 1027, 2335, 74, 410, "Text",
  Evaluatable->False],
Cell[35423, 1103, 127, 3, 40, "Input"],
Cell[35553, 1108, 594, 20, 58, "Text",
  Evaluatable->False],
Cell[36150, 1130, 133, 3, 40, "Input"],
Cell[36286, 1135, 159, 5, 38, "Text",
  Evaluatable->False],
Cell[36448, 1142, 152, 3, 40, "Input"],
Cell[36603, 1147, 1439, 41, 201, "Text",
  Evaluatable->False]
}, Closed]],

Cell[CellGroupData[{
Cell[38079, 1193, 86, 2, 31, "Subsubsection",
  Evaluatable->False],
Cell[38168, 1197, 1398, 42, 130, "Text",
  Evaluatable->False],
Cell[39569, 1241, 186, 4, 63, "Input"],
Cell[39758, 1247, 381, 9, 61, "Text"],
Cell[40142, 1258, 261, 5, 86, "Input"],
Cell[40406, 1265, 840, 20, 107, "Text"],
Cell[41249, 1287, 175, 5, 40, "Input"],
Cell[41427, 1294, 877, 33, 84, "Text"],
Cell[42307, 1329, 240, 5, 86, "Input"],
Cell[42550, 1336, 1574, 40, 176, "Text"],
Cell[44127, 1378, 96, 2, 40, "Input"],
Cell[44226, 1382, 240, 6, 61, "Text"],
Cell[44469, 1390, 115, 2, 40, "Input"],
Cell[44587, 1394, 183, 5, 38, "Text",
  Evaluatable->False],
Cell[44773, 1401, 126, 3, 40, "Input"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[44948, 1410, 94, 2, 49, "Section",
  Evaluatable->False],

Cell[CellGroupData[{
Cell[45067, 1416, 99, 2, 67, "Subsection",
  Evaluatable->False],
Cell[45169, 1420, 1339, 37, 154, "Text",
  Evaluatable->False],
Cell[46511, 1459, 944, 26, 155, "Text",
  Evaluatable->False],
Cell[47458, 1487, 184, 6, 109, "Input"],
Cell[47645, 1495, 353, 9, 61, "Text",
  Evaluatable->False],
Cell[48001, 1506, 96, 3, 40, "Input"],
Cell[48100, 1511, 71, 2, 40, "Input"],
Cell[48174, 1515, 2909, 85, 432, "Text",
  Evaluatable->False],
Cell[51086, 1602, 95, 3, 40, "Input"],
Cell[51184, 1607, 102, 4, 59, "Input"],
Cell[51289, 1613, 590, 15, 107, "Text",
  Evaluatable->False],
Cell[51882, 1630, 241, 6, 63, "Input"],
Cell[52126, 1638, 163, 4, 38, "Text"],
Cell[52292, 1644, 145, 4, 40, "Input"],
Cell[52440, 1650, 141, 5, 38, "Text",
  Evaluatable->False],
Cell[52584, 1657, 123, 3, 40, "Input"],
Cell[52710, 1662, 989, 28, 108, "Text",
  Evaluatable->False],
Cell[53702, 1692, 208, 5, 109, "Input"],
Cell[53913, 1699, 68, 0, 38, "Text"],
Cell[53984, 1701, 125, 3, 63, "Input"],
Cell[54112, 1706, 176, 5, 38, "Text"],
Cell[54291, 1713, 208, 5, 109, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[54536, 1723, 85, 2, 67, "Subsection",
  Evaluatable->False],
Cell[54624, 1727, 1658, 49, 202, "Text",
  Evaluatable->False],
Cell[56285, 1778, 100, 3, 45, "Input"],
Cell[56388, 1783, 228, 5, 63, "Input"],
Cell[56619, 1790, 1155, 41, 85, "Text",
  Evaluatable->False],
Cell[57777, 1833, 183, 4, 68, "Input"],
Cell[57963, 1839, 568, 15, 85, "Text",
  Evaluatable->False],
Cell[58534, 1856, 184, 7, 99, "Input"],
Cell[58721, 1865, 110, 2, 38, "Text",
  Evaluatable->False],
Cell[58834, 1869, 124, 3, 40, "Input"],
Cell[58961, 1874, 147, 5, 38, "Text",
  Evaluatable->False],
Cell[59111, 1881, 116, 3, 40, "Input"],
Cell[59230, 1886, 187, 7, 38, "Text",
  Evaluatable->False],
Cell[59420, 1895, 129, 4, 63, "Input"],
Cell[59552, 1901, 103, 3, 40, "Input"],
Cell[59658, 1906, 269, 10, 38, "Text",
  Evaluatable->False],
Cell[59930, 1918, 99, 3, 40, "Input"],
Cell[60032, 1923, 718, 18, 86, "Text",
  Evaluatable->False],
Cell[60753, 1943, 195, 4, 63, "Input"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[60997, 1953, 93, 2, 49, "Section",
  Evaluatable->False],
Cell[61093, 1957, 1229, 32, 155, "Text"],
Cell[62325, 1991, 864, 15, 153, "Text"],
Cell[63192, 2008, 1561, 41, 153, "Text"],
Cell[64756, 2051, 263, 6, 62, "Text"],
Cell[65022, 2059, 249, 6, 109, "Input"],
Cell[65274, 2067, 203, 5, 39, "Text"],
Cell[65480, 2074, 84, 2, 59, "Input"],
Cell[65567, 2078, 185, 5, 38, "Text"],
Cell[65755, 2085, 101, 2, 59, "Input"],
Cell[65859, 2089, 275, 6, 105, "Input"],
Cell[66137, 2097, 655, 20, 63, "Text"],
Cell[66795, 2119, 146, 3, 82, "Input"],
Cell[66944, 2124, 178, 5, 38, "Text"],
Cell[67125, 2131, 117, 2, 59, "Input"],
Cell[67245, 2135, 139, 3, 38, "Text"],
Cell[67387, 2140, 140, 3, 59, "Input"],
Cell[67530, 2145, 106, 3, 38, "Text"],
Cell[67639, 2150, 145, 3, 59, "Input"],
Cell[67787, 2155, 398, 11, 62, "Text"],
Cell[68188, 2168, 333, 7, 105, "Input"],
Cell[68524, 2177, 411, 8, 84, "Text"],
Cell[68938, 2187, 444, 9, 174, "Input"],
Cell[69385, 2198, 117, 3, 38, "Text"],
Cell[69505, 2203, 62, 2, 59, "Input"],
Cell[69570, 2207, 64, 0, 38, "Text"],
Cell[69637, 2209, 73, 2, 59, "Input"],
Cell[69713, 2213, 138, 3, 38, "Text"],
Cell[69854, 2218, 134, 3, 59, "Input"],
Cell[69991, 2223, 103, 3, 38, "Text"],
Cell[70097, 2228, 139, 3, 59, "Input"],
Cell[70239, 2233, 216, 5, 59, "Input"],
Cell[70458, 2240, 216, 5, 59, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[70711, 2250, 82, 2, 49, "Section",
  Evaluatable->False],

Cell[CellGroupData[{
Cell[70818, 2256, 87, 2, 45, "Subsubsection",
  Evaluatable->False],
Cell[70908, 2260, 458, 9, 107, "Text",
  Evaluatable->False],
Cell[71369, 2271, 2116, 64, 199, "Text"],
Cell[73488, 2337, 159, 4, 82, "Input"],
Cell[73650, 2343, 197, 5, 38, "Text",
  Evaluatable->False],
Cell[73850, 2350, 296, 7, 105, "Input"],
Cell[74149, 2359, 89, 3, 38, "Text"],
Cell[74241, 2364, 219, 5, 82, "Input"],
Cell[74463, 2371, 202, 4, 61, "Text"],
Cell[74668, 2377, 99, 2, 59, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[74804, 2384, 81, 2, 45, "Subsubsection",
  Evaluatable->False],
Cell[74888, 2388, 469, 11, 84, "Text",
  Evaluatable->False],
Cell[75360, 2401, 199, 5, 105, "Input"],
Cell[75562, 2408, 122, 2, 38, "Text",
  Evaluatable->False],
Cell[75687, 2412, 166, 4, 82, "Input"],
Cell[75856, 2418, 138, 5, 38, "Text",
  Evaluatable->False],
Cell[75997, 2425, 315, 7, 105, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[76349, 2437, 81, 2, 45, "Subsubsection",
  Evaluatable->False],
Cell[76433, 2441, 108, 2, 38, "Text",
  Evaluatable->False],
Cell[76544, 2445, 157, 4, 82, "Input"],
Cell[76704, 2451, 169, 5, 38, "Text",
  Evaluatable->False],
Cell[76876, 2458, 166, 4, 82, "Input"],
Cell[77045, 2464, 98, 2, 38, "Text",
  Evaluatable->False],
Cell[77146, 2468, 302, 7, 105, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[77485, 2480, 81, 2, 45, "Subsubsection",
  Evaluatable->False],
Cell[77569, 2484, 306, 9, 61, "Text",
  Evaluatable->False],
Cell[77878, 2495, 168, 4, 82, "Input"],
Cell[78049, 2501, 388, 13, 62, "Text",
  Evaluatable->False],
Cell[78440, 2516, 232, 7, 118, "Input"],
Cell[78675, 2525, 178, 5, 38, "Text",
  Evaluatable->False],
Cell[78856, 2532, 259, 7, 98, "Input"],
Cell[79118, 2541, 108, 2, 38, "Text",
  Evaluatable->False],
Cell[79229, 2545, 395, 7, 128, "Input"],
Cell[79627, 2554, 125, 3, 38, "Text"],
Cell[79755, 2559, 220, 5, 82, "Input"],
Cell[79978, 2566, 375, 7, 128, "Input"],
Cell[80356, 2575, 52, 1, 59, "Input"]
}, Closed]]
}, Closed]]
}, Open  ]]
}
]
*)



(*******************************************************************
End of Mathematica Notebook file.
*******************************************************************)

